@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 704 $
- * $Date: 2013-06-04 23:04:00 +0200 (Tue, 04 Jun 2013) $
+ * $Revision: 917 $
+ * $Date: 2013-12-10 21:37:02 +0100 (Tue, 10 Dec 2013) $
  *
  */
 
@@ -77,22 +77,10 @@ ArticleDownloader::~ArticleDownloader()
 {
 	debug("Destroying ArticleDownloader");
 
-	if (m_szTempFilename)
-	{
-		free(m_szTempFilename);
-	}
-	if (m_szArticleFilename)
-	{
-		free(m_szArticleFilename);
-	}
-	if (m_szInfoName)
-	{
-		free(m_szInfoName);
-	}
-	if (m_szOutputFilename)
-	{
-		free(m_szOutputFilename);
-	}
+	free(m_szTempFilename);
+	free(m_szArticleFilename);
+	free(m_szInfoName);
+	free(m_szOutputFilename);
 }
 
 void ArticleDownloader::SetTempFilename(const char* v)
@@ -158,18 +146,19 @@ void ArticleDownloader::Run()
 
 	int iRetries = g_pOptions->GetRetries() > 0 ? g_pOptions->GetRetries() : 1;
 	int iRemainedRetries = iRetries;
-	ServerPool::Servers failedServers;
+	Servers failedServers;
 	failedServers.reserve(g_pServerPool->GetServers()->size());
 	NewsServer* pWantServer = NULL;
 	NewsServer* pLastServer = NULL;
 	int iLevel = 0;
+	int iServerConfigGeneration = g_pServerPool->GetGeneration();
 
 	while (!IsStopped())
 	{
 		Status = adFailed;
 
 		SetStatus(adWaiting);
-		while (!IsStopped() && !m_pConnection)
+		while (!m_pConnection && !(IsStopped() || iServerConfigGeneration != g_pServerPool->GetGeneration()))
 		{
 			m_pConnection = g_pServerPool->GetConnection(iLevel, pWantServer, &failedServers);
 			usleep(5 * 1000);
@@ -177,7 +166,8 @@ void ArticleDownloader::Run()
 		SetLastUpdateTimeNow();
 		SetStatus(adRunning);
 
-		if (IsStopped() || g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2())
+		if (IsStopped() || g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2() ||
+			iServerConfigGeneration != g_pServerPool->GetGeneration())
 		{
 			Status = adRetry;
 			break;
@@ -191,10 +181,15 @@ void ArticleDownloader::Run()
 		bool bConnected = m_pConnection && m_pConnection->Connect();
 		if (bConnected && !IsStopped())
 		{
-			// Okay, we got a Connection. Now start downloading.
-			detail("Downloading %s @ server%i (%s)", m_szInfoName,
-				m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost());
+			NewsServer* pNewsServer = m_pConnection->GetNewsServer();
+			detail("Downloading %s @ %s (%s)", m_szInfoName, pNewsServer->GetName(), m_pConnection->GetHost());
+
 			Status = Download();
+
+			if (Status == adFinished || Status == adFailed || Status == adNotFound || Status == adCrcError)
+			{
+				m_ServerStats.SetStat(pNewsServer->GetID(), Status == adFinished ? 1 : 0, Status == adFinished ? 0 : 1, false);
+			}
 		}
 
 		if (bConnected)
@@ -234,14 +229,16 @@ void ArticleDownloader::Run()
 			pWantServer = pLastServer;
 		}
 
-		if (pWantServer && !IsStopped() &&
-			!(g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2()))
+		if (pWantServer && 
+			!(IsStopped() || g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2() ||
+			 iServerConfigGeneration != g_pServerPool->GetGeneration()))
 		{
 			detail("Waiting %i sec to retry", g_pOptions->GetRetryInterval());
 			SetStatus(adWaiting);
 			int msec = 0;
-			while (!IsStopped() && (msec < g_pOptions->GetRetryInterval() * 1000) && 
-				!(g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2()))
+			while (!(IsStopped() || g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2() ||
+				  iServerConfigGeneration != g_pServerPool->GetGeneration()) &&
+				  msec < g_pOptions->GetRetryInterval() * 1000)
 			{
 				usleep(100 * 1000);
 				msec += 100;
@@ -250,7 +247,8 @@ void ArticleDownloader::Run()
 			SetStatus(adRunning);
 		}
 
-		if (IsStopped() || g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2())
+		if (IsStopped() || g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2() ||
+			iServerConfigGeneration != g_pServerPool->GetGeneration())
 		{
 			Status = adRetry;
 			break;
@@ -264,22 +262,25 @@ void ArticleDownloader::Run()
 			// if all servers from all levels were tried, break the loop with failure status
 
 			bool bAllServersOnLevelFailed = true;
-			for (ServerPool::Servers::iterator it = g_pServerPool->GetServers()->begin(); it != g_pServerPool->GetServers()->end(); it++)
+			for (Servers::iterator it = g_pServerPool->GetServers()->begin(); it != g_pServerPool->GetServers()->end(); it++)
 			{
 				NewsServer* pCandidateServer = *it;
-				if (pCandidateServer->GetLevel() == iLevel)
+				if (pCandidateServer->GetNormLevel() == iLevel)
 				{
-					bool bServerFailed = false;
-					for (ServerPool::Servers::iterator it = failedServers.begin(); it != failedServers.end(); it++)
+					bool bServerFailed = !pCandidateServer->GetActive() || pCandidateServer->GetMaxConnections() == 0;
+					if (!bServerFailed)
 					{
-						NewsServer* pIgnoreServer = *it;
-						if (pIgnoreServer == pCandidateServer ||
-							(pIgnoreServer->GetGroup() > 0 && pIgnoreServer->GetGroup() == pCandidateServer->GetGroup() &&
-							 pIgnoreServer->GetLevel() == pCandidateServer->GetLevel()))
+						for (Servers::iterator it = failedServers.begin(); it != failedServers.end(); it++)
 						{
-							bServerFailed = true;
-							break;
-						}					
+							NewsServer* pIgnoreServer = *it;
+							if (pIgnoreServer == pCandidateServer ||
+								(pIgnoreServer->GetGroup() > 0 && pIgnoreServer->GetGroup() == pCandidateServer->GetGroup() &&
+								 pIgnoreServer->GetNormLevel() == pCandidateServer->GetNormLevel()))
+							{
+								bServerFailed = true;
+								break;
+							}					
+						}
 					}
 					if (!bServerFailed)
 					{
@@ -291,14 +292,14 @@ void ArticleDownloader::Run()
 
 			if (bAllServersOnLevelFailed)
 			{
-				if (iLevel < g_pServerPool->GetMaxLevel())
+				if (iLevel < g_pServerPool->GetMaxNormLevel())
 				{
 					detail("Article %s @ all level %i servers failed, increasing level", m_szInfoName, iLevel);
 					iLevel++;
 				}
 				else
 				{
-					warn("Article %s @ all servers failed", m_szInfoName);
+					detail("Article %s @ all servers failed", m_szInfoName);
 					Status = adFailed;
 					break;
 				}
@@ -328,7 +329,7 @@ void ArticleDownloader::Run()
 
 	if (Status == adFailed)
 	{
-		warn("Download %s failed", m_szInfoName);
+		detail("Download %s failed", m_szInfoName);
 	}
 
 	SetStatus(Status);
@@ -420,8 +421,8 @@ ArticleDownloader::EStatus ArticleDownloader::Download()
 		{
 			if (!IsStopped())
 			{
-				warn("Article %s @ server%i (%s) failed: Unexpected end of article", m_szInfoName,
-					m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost());
+				detail("Article %s @ %s (%s) failed: Unexpected end of article", m_szInfoName,
+					m_pConnection->GetNewsServer()->GetName(), m_pConnection->GetHost());
 			}
 			Status = adFailed;
 			break;
@@ -455,8 +456,8 @@ ArticleDownloader::EStatus ArticleDownloader::Download()
 				if (strncmp(p, m_pArticleInfo->GetMessageID(), strlen(m_pArticleInfo->GetMessageID())))
 				{
 					if (char* e = strrchr(p, '\r')) *e = '\0'; // remove trailing CR-character
-					warn("Article %s @ server%i (%s) failed: Wrong message-id, expected %s, returned %s", m_szInfoName,
-						m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost(), m_pArticleInfo->GetMessageID(), p);
+					detail("Article %s @ %s (%s) failed: Wrong message-id, expected %s, returned %s", m_szInfoName,
+						m_pConnection->GetNewsServer()->GetName(), m_pConnection->GetHost(), m_pArticleInfo->GetMessageID(), p);
 					Status = adFailed;
 					break;
 				}
@@ -484,8 +485,8 @@ ArticleDownloader::EStatus ArticleDownloader::Download()
 
 	if (!bEnd && Status == adRunning && !IsStopped())
 	{
-		warn("Article %s @ server%i (%s) failed: article incomplete", m_szInfoName,
-			m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost());
+		detail("Article %s @ %s (%s) failed: article incomplete", m_szInfoName,
+			m_pConnection->GetNewsServer()->GetName(), m_pConnection->GetHost());
 		Status = adFailed;
 	}
 
@@ -512,21 +513,21 @@ ArticleDownloader::EStatus ArticleDownloader::CheckResponse(const char* szRespon
 	{
 		if (!IsStopped())
 		{
-			warn("Article %s @ server%i (%s) failed, %s: Connection closed by remote host", m_szInfoName, 
-				m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost(), szComment);
+			detail("Article %s @ %s (%s) failed, %s: Connection closed by remote host", m_szInfoName, 
+				m_pConnection->GetNewsServer()->GetName(), m_pConnection->GetHost(), szComment);
 		}
 		return adConnectError;
 	}
 	else if (m_pConnection->GetAuthError() || !strncmp(szResponse, "400", 3) || !strncmp(szResponse, "499", 3))
 	{
-		warn("Article %s @ server%i (%s) failed, %s: %s", m_szInfoName,
-			 m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost(), szComment, szResponse);
+		detail("Article %s @ %s (%s) failed, %s: %s", m_szInfoName,
+			 m_pConnection->GetNewsServer()->GetName(), m_pConnection->GetHost(), szComment, szResponse);
 		return adConnectError;
 	}
 	else if (!strncmp(szResponse, "41", 2) || !strncmp(szResponse, "42", 2) || !strncmp(szResponse, "43", 2))
 	{
-		warn("Article %s @ server%i (%s) failed, %s: %s", m_szInfoName,
-			 m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost(), szComment, szResponse);
+		detail("Article %s @ %s (%s) failed, %s: %s", m_szInfoName,
+			 m_pConnection->GetNewsServer()->GetName(), m_pConnection->GetHost(), szComment, szResponse);
 		return adNotFound;
 	}
 	else if (!strncmp(szResponse, "2", 1))
@@ -537,8 +538,8 @@ ArticleDownloader::EStatus ArticleDownloader::CheckResponse(const char* szRespon
 	else 
 	{
 		// unknown error, no special handling
-		warn("Article %s @ server%i (%s) failed, %s: %s", m_szInfoName,
-			 m_pConnection->GetNewsServer()->GetID(), m_pConnection->GetHost(), szComment, szResponse);
+		detail("Article %s @ %s (%s) failed, %s: %s", m_szInfoName,
+			 m_pConnection->GetNewsServer()->GetName(), m_pConnection->GetHost(), szComment, szResponse);
 		return adFailed;
 	}
 }
@@ -563,13 +564,13 @@ bool ArticleDownloader::Write(char* szLine, int iLen)
 		}
 		else
 		{
-			warn("Decoding %s failed: unsupported encoding", m_szInfoName);
+			detail("Decoding %s failed: unsupported encoding", m_szInfoName);
 			return false;
 		}
 		if (!bOK)
 		{
 			debug("Failed line: %s", szLine);
-			warn("Decoding %s failed", m_szInfoName);
+			detail("Decoding %s failed", m_szInfoName);
 		}
 		return bOK;
 	}
@@ -588,7 +589,9 @@ bool ArticleDownloader::PrepareFile(char* szLine)
 	{
 		if (!strncmp(szLine, "=ybegin ", 8))
 		{
-			if (g_pOptions->GetDupeCheck())
+			if (g_pOptions->GetDupeCheck() &&
+				m_pFileInfo->GetNZBInfo()->GetDupeMode() != dmForce &&
+				!m_pFileInfo->GetNZBInfo()->GetManyDupeFiles())
 			{
 				m_pFileInfo->LockOutputFile();
 				bool bOutputInitialized = m_pFileInfo->GetOutputInitialized();
@@ -613,7 +616,8 @@ bool ArticleDownloader::PrepareFile(char* szLine)
 					m_pFileInfo->SetOutputInitialized(true);
 				}
 				m_pFileInfo->UnlockOutputFile();
-				if (!bOutputInitialized && m_szArticleFilename && m_pFileInfo->IsDupe(m_szArticleFilename))
+				if (!bOutputInitialized && m_szArticleFilename &&
+					Util::FileExists(m_pFileInfo->GetNZBInfo()->GetDestDir(), m_szArticleFilename))
 				{
 					m_bDuplicate = true;
 					return false;
@@ -659,7 +663,9 @@ bool ArticleDownloader::PrepareFile(char* szLine)
 		m_pOutFile = fopen(szFilename, bDirectWrite ? "rb+" : "wb");
 		if (!m_pOutFile)
 		{
-			error("Could not %s file %s", bDirectWrite ? "open" : "create", szFilename);
+			char szSysErrStr[256];
+			error("Could not %s file %s! Errcode: %i, %s", bDirectWrite ? "open" : "create", szFilename,
+				errno, Util::GetLastErrorMessage(szSysErrStr, sizeof(szSysErrStr)));
 			return false;
 		}
 		if (g_pOptions->GetWriteBufferSize() == -1)
@@ -766,7 +772,7 @@ ArticleDownloader::EStatus ArticleDownloader::DecodeCheck()
 		}
 		else
 		{
-			warn("Decoding %s failed: no binary data or unsupported encoding format", m_szInfoName);
+			detail("Decoding %s failed: no binary data or unsupported encoding format", m_szInfoName);
 			return adFailed;
 		}
 
@@ -811,27 +817,27 @@ ArticleDownloader::EStatus ArticleDownloader::DecodeCheck()
 			remove(m_szResultFilename);
 			if (eStatus == Decoder::eCrcError)
 			{
-				warn("Decoding %s failed: CRC-Error", m_szInfoName);
+				detail("Decoding %s failed: CRC-Error", m_szInfoName);
 				return adCrcError;
 			}
 			else if (eStatus == Decoder::eArticleIncomplete)
 			{
-				warn("Decoding %s failed: article incomplete", m_szInfoName);
+				detail("Decoding %s failed: article incomplete", m_szInfoName);
 				return adFailed;
 			}
 			else if (eStatus == Decoder::eInvalidSize)
 			{
-				warn("Decoding %s failed: size mismatch", m_szInfoName);
+				detail("Decoding %s failed: size mismatch", m_szInfoName);
 				return adFailed;
 			}
 			else if (eStatus == Decoder::eNoBinaryData)
 			{
-				warn("Decoding %s failed: no binary data found", m_szInfoName);
+				detail("Decoding %s failed: no binary data found", m_szInfoName);
 				return adFailed;
 			}
 			else
 			{
-				warn("Decoding %s failed", m_szInfoName);
+				detail("Decoding %s failed", m_szInfoName);
 				return adFailed;
 			}
 		}
@@ -954,17 +960,7 @@ void ArticleDownloader::CompleteFileParts()
 	}
 
 	char ofn[1024];
-	snprintf(ofn, 1024, "%s%c%s", szNZBDestDir, (int)PATH_SEPARATOR, m_pFileInfo->GetFilename());
-	ofn[1024-1] = '\0';
-
-	// prevent overwriting existing files
-	int dupcount = 0;
-	while (Util::FileExists(ofn))
-	{
-		dupcount++;
-		snprintf(ofn, 1024, "%s%c%s_duplicate%d", szNZBDestDir, (int)PATH_SEPARATOR, m_pFileInfo->GetFilename(), dupcount);
-		ofn[1024-1] = '\0';
-	}
+	Util::MakeUniqueFilename(ofn, 1024, szNZBDestDir, m_pFileInfo->GetFilename());
 
 	FILE* outfile = NULL;
 	char tmpdestfile[1024];
@@ -1058,10 +1054,7 @@ void ArticleDownloader::CompleteFileParts()
 		}
 	}
 
-	if (buffer)
-	{
-		free(buffer);
-	}
+	free(buffer);
 
 	if (outfile)
 	{
@@ -1113,7 +1106,9 @@ void ArticleDownloader::CompleteFileParts()
 	}
 	else
 	{
-		warn("%i of %i article downloads failed for \"%s\"", iBrokenCount, m_pFileInfo->GetArticles()->size(), InfoFilename);
+		warn("%i of %i article downloads failed for \"%s\"",
+			iBrokenCount + m_pFileInfo->GetMissedArticles(),
+			m_pFileInfo->GetTotalArticles(), InfoFilename);
 
 		if (g_pOptions->GetCreateBrokenLog())
 		{
@@ -1121,12 +1116,14 @@ void ArticleDownloader::CompleteFileParts()
 			snprintf(szBrokenLogName, 1024, "%s%c_brokenlog.txt", szNZBDestDir, (int)PATH_SEPARATOR);
 			szBrokenLogName[1024-1] = '\0';
 			FILE* file = fopen(szBrokenLogName, "ab");
-			fprintf(file, "%s (%i/%i)%s", m_pFileInfo->GetFilename(), m_pFileInfo->GetArticles()->size() - iBrokenCount, m_pFileInfo->GetArticles()->size(), LINE_ENDING);
+			fprintf(file, "%s (%i/%i)%s", m_pFileInfo->GetFilename(),
+				m_pFileInfo->GetTotalArticles() - iBrokenCount - m_pFileInfo->GetMissedArticles(),
+				m_pFileInfo->GetTotalArticles(), LINE_ENDING);
 			fclose(file);
 		}
 	}
 
-	// the locking is needed for accessing the memebers of NZBInfo
+	// the locking is needed for accessing the members of NZBInfo
 	g_pDownloadQueueHolder->LockQueue();
 	m_pFileInfo->GetNZBInfo()->GetCompletedFiles()->push_back(strdup(ofn));
 	if (strcmp(m_pFileInfo->GetNZBInfo()->GetDestDir(), szNZBDestDir))
@@ -1166,13 +1163,7 @@ bool ArticleDownloader::MoveCompletedFiles(NZBInfo* pNZBInfo, const char* szOldD
 		if (strcmp(szFileName, szNewFileName))
 		{
 			// prevent overwriting of existing files
-			int dupcount = 0;
-			while (Util::FileExists(szNewFileName))
-			{
-				dupcount++;
-				snprintf(szNewFileName, 1024, "%s%c%s_duplicate%d", pNZBInfo->GetDestDir(), (int)PATH_SEPARATOR, Util::BaseFileName(szFileName), dupcount);
-				szNewFileName[1024-1] = '\0';
-			}
+			Util::MakeUniqueFilename(szNewFileName, 1024, pNZBInfo->GetDestDir(), Util::BaseFileName(szFileName));
 
 			detail("Moving file %s to %s", szFileName, szNewFileName);
 			if (Util::MoveFile(szFileName, szNewFileName))
