@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 711 $
- * $Date: 2013-06-17 22:39:46 +0200 (Mon, 17 Jun 2013) $
+ * $Revision: 911 $
+ * $Date: 2013-11-24 20:29:52 +0100 (Sun, 24 Nov 2013) $
  *
  */
 
@@ -358,7 +358,7 @@ void DownloadBinCommand::Execute()
 	bool bAddTop = ntohl(DownloadRequest.m_bAddFirst);
 
 	bool bOK = g_pScanner->AddExternalFile(DownloadRequest.m_szFilename, DownloadRequest.m_szCategory,
-		iPriority, NULL, bAddTop, bAddPaused, NULL, pRecvBuffer, iBufLen, true);
+		iPriority, NULL, 0, dmScore, NULL, bAddTop, bAddPaused, NULL, pRecvBuffer, iBufLen) != Scanner::asFailed;
 
 	char tmp[1024];
 	snprintf(tmp, 1024, bOK ? "Collection %s added to queue" : "Download Request failed for %s",
@@ -563,10 +563,7 @@ void ListBinCommand::Execute()
 
 		g_pQueueCoordinator->UnlockQueue();
 
-		if (pRegEx)
-		{
-			delete pRegEx;
-		}
+		delete pRegEx;
 
 		ListResponse.m_iNrTrailingNZBEntries = htonl(iNrNZBEntries);
 		ListResponse.m_iNrTrailingPPPEntries = htonl(iNrPPPEntries);
@@ -612,10 +609,7 @@ void ListBinCommand::Execute()
 		m_pConnection->Send(buf, bufsize);
 	}
 
-	if (buf)
-	{
-		free(buf);
-	}
+	free(buf);
 }
 
 void LogBinCommand::Execute()
