@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 708 $
- * $Date: 2013-06-13 22:51:38 +0200 (Thu, 13 Jun 2013) $
+ * $Revision: 921 $
+ * $Date: 2013-12-18 21:19:42 +0100 (Wed, 18 Dec 2013) $
  *
  */
 
@@ -42,16 +42,21 @@
 #include "nzbget.h"
 #include "DownloadInfo.h"
 #include "Options.h"
-#include "Log.h"
 #include "Util.h"
 
 extern Options* g_pOptions;
 
 int FileInfo::m_iIDGen = 0;
+int FileInfo::m_iIDMax = 0;
 int NZBInfo::m_iIDGen = 0;
+int NZBInfo::m_iIDMax = 0;
 int PostInfo::m_iIDGen = 0;
+int PostInfo::m_iIDMax = 0;
 int UrlInfo::m_iIDGen = 0;
+int UrlInfo::m_iIDMax = 0;
 int HistoryInfo::m_iIDGen = 0;
+int HistoryInfo::m_iIDMax = 0;
+
 
 NZBParameter::NZBParameter(const char* szName)
 {
@@ -61,26 +66,31 @@ NZBParameter::NZBParameter(const char* szName)
 
 NZBParameter::~NZBParameter()
 {
-	if (m_szName)
-	{
-		free(m_szName);
-	}
-	if (m_szValue)
-	{
-		free(m_szValue);
-	}
+	free(m_szName);
+	free(m_szValue);
 }
 
 void NZBParameter::SetValue(const char* szValue)
 {
-	if (m_szValue)
-	{
-		free(m_szValue);
-	}
+	free(m_szValue);
 	m_szValue = strdup(szValue);
 }
 
 
+NZBParameterList::~NZBParameterList()
+{
+	Clear();
+}
+
+void NZBParameterList::Clear()
+{
+	for (iterator it = begin(); it != end(); it++)
+	{
+		delete *it;
+	}
+	clear();
+}
+
 void NZBParameterList::SetParameter(const char* szName, const char* szValue)
 {
 	NZBParameter* pParameter = NULL;
@@ -116,6 +126,30 @@ void NZBParameterList::SetParameter(const char* szName, const char* szValue)
 	pParameter->SetValue(szValue);
 }
 
+NZBParameter* NZBParameterList::Find(const char* szName, bool bCaseSensitive)
+{
+	for (iterator it = begin(); it != end(); it++)
+	{
+		NZBParameter* pParameter = *it;
+		if ((bCaseSensitive && !strcmp(pParameter->GetName(), szName)) ||
+			(!bCaseSensitive && !strcasecmp(pParameter->GetName(), szName)))
+		{
+			return pParameter;
+		}
+	}
+	
+	return NULL;
+}
+
+void NZBParameterList::CopyFrom(NZBParameterList* pSourceParameters)
+{
+	for (iterator it = pSourceParameters->begin(); it != pSourceParameters->end(); it++)
+	{
+		NZBParameter* pParameter = *it;
+		SetParameter(pParameter->GetName(), pParameter->GetValue());
+	}
+}									  
+
 
 ScriptStatus::ScriptStatus(const char* szName, EStatus eStatus)
 {
@@ -125,10 +159,7 @@ ScriptStatus::ScriptStatus(const char* szName, EStatus eStatus)
 
 ScriptStatus::~ScriptStatus()
 {
-	if (m_szName)
-	{
-		free(m_szName);
-	}
+	free(m_szName);
 }
 
 
@@ -170,17 +201,85 @@ ScriptStatus::EStatus ScriptStatusList::CalcTotalStatus()
 }
 
 
-NZBInfo::NZBInfo()
+ServerStat::ServerStat(int iServerID)
+{
+	m_iServerID = iServerID;
+	m_iSuccessArticles = 0;
+	m_iFailedArticles = 0;
+}
+
+
+ServerStatList::~ServerStatList()
+{
+	Clear();
+}
+
+void ServerStatList::Clear()
+{
+	for (iterator it = begin(); it != end(); it++)
+	{
+		delete *it;
+	}
+	clear();
+}
+
+void ServerStatList::SetStat(int iServerID, int iSuccessArticles, int iFailedArticles, bool bAdd)
+{
+	ServerStat* pServerStat = NULL;
+	for (iterator it = begin(); it != end(); it++)
+	{
+		ServerStat* pServerStat1 = *it;
+		if (pServerStat1->GetServerID() == iServerID)
+		{
+			pServerStat = pServerStat1;
+			break;
+		}
+	}
+
+	if (!pServerStat)
+	{
+		pServerStat = new ServerStat(iServerID);
+		push_back(pServerStat);
+	}
+
+	pServerStat->SetSuccessArticles((bAdd ? pServerStat->GetSuccessArticles() : 0) + iSuccessArticles);
+	pServerStat->SetFailedArticles((bAdd ? pServerStat->GetFailedArticles() : 0) + iFailedArticles);
+}
+
+void ServerStatList::Add(ServerStatList* pServerStats)
+{
+	for (iterator it = pServerStats->begin(); it != pServerStats->end(); it++)
+	{
+		ServerStat* pServerStat = *it;
+		SetStat(pServerStat->GetServerID(), pServerStat->GetSuccessArticles(), pServerStat->GetFailedArticles(), true);
+	}
+}
+
+
+NZBInfo::NZBInfo(bool bPersistent)
 {
 	debug("Creating NZBInfo");
 
 	m_szFilename = NULL;
 	m_szDestDir = NULL;
+	m_szFinalDir = strdup("");
 	m_szCategory = strdup("");
 	m_szName = NULL;
 	m_iFileCount = 0;
 	m_iParkedFileCount = 0;
 	m_lSize = 0;
+	m_lSuccessSize = 0;
+	m_lFailedSize = 0;
+	m_lCurrentSuccessSize = 0;
+	m_lCurrentFailedSize = 0;
+	m_lParSize = 0;
+	m_lParSuccessSize = 0;
+	m_lParFailedSize = 0;
+	m_lParCurrentSuccessSize = 0;
+	m_lParCurrentFailedSize = 0;
+	m_iTotalArticles = 0;
+	m_iSuccessArticles = 0;
+	m_iFailedArticles = 0;
 	m_iRefCount = 0;
 	m_bPostProcess = false;
 	m_eRenameStatus = rsNone;
@@ -188,51 +287,42 @@ NZBInfo::NZBInfo()
 	m_eUnpackStatus = usNone;
 	m_eCleanupStatus = csNone;
 	m_eMoveStatus = msNone;
-	m_bDeleted = false;
+	m_eDeleteStatus = dsNone;
+	m_eMarkStatus = ksNone;
+	m_bDeleting = false;
+	m_bDeletePaused = false;
+	m_bManyDupeFiles = false;
+	m_bAvoidHistory = false;
+	m_bHealthPaused = false;
 	m_bParCleanup = false;
 	m_bCleanupDisk = false;
 	m_bUnpackCleanedUpDisk = false;
 	m_szQueuedFilename = strdup("");
+	m_szDupeKey = strdup("");
+	m_iDupeScore = 0;
+	m_eDupeMode = dmScore;
+	m_iFullContentHash = 0;
+	m_iFilteredContentHash = 0;
 	m_Owner = NULL;
 	m_Messages.clear();
 	m_iIDMessageGen = 0;
-	m_iIDGen++;
-	m_iID = m_iIDGen;
+	m_iID = bPersistent ? ++m_iIDGen : 0;
 }
 
 NZBInfo::~NZBInfo()
 {
 	debug("Destroying NZBInfo");
 
-	if (m_szFilename)
-	{
-		free(m_szFilename);
-	}
-	if (m_szDestDir)
-	{
-		free(m_szDestDir);
-	}
-	if (m_szCategory)
-	{
-		free(m_szCategory);
-	}
-	if (m_szName)
-	{
-		free(m_szName);
-	}
-	if (m_szQueuedFilename)
-	{
-		free(m_szQueuedFilename);
-	}
+	free(m_szFilename);
+	free(m_szDestDir);
+	free(m_szFinalDir);
+	free(m_szCategory);
+	free(m_szName);
+	free(m_szQueuedFilename);
+	free(m_szDupeKey);
 
 	ClearCompletedFiles();
 
-	for (NZBParameterList::iterator it = m_ppParameters.begin(); it != m_ppParameters.end(); it++)
-	{
-		delete *it;
-	}
-	m_ppParameters.clear();
-
 	for (Messages::iterator it = m_Messages.begin(); it != m_Messages.end(); it++)
 	{
 		delete *it;
@@ -245,7 +335,7 @@ NZBInfo::~NZBInfo()
 	}
 }
 
-void NZBInfo::AddReference()
+void NZBInfo::Retain()
 {
 	m_iRefCount++;
 }
@@ -262,9 +352,22 @@ void NZBInfo::Release()
 void NZBInfo::SetID(int iID)
 {
 	m_iID = iID;
-	if (m_iIDGen < m_iID)
+	if (m_iIDMax < m_iID)
 	{
-		m_iIDGen = m_iID;
+		m_iIDMax = m_iID;
+	}
+}
+
+void NZBInfo::ResetGenID(bool bMax)
+{
+	if (bMax)
+	{
+		m_iIDGen = m_iIDMax;
+	}
+	else
+	{
+		m_iIDGen = 0;
+		m_iIDMax = 0;
 	}
 }
 
@@ -279,19 +382,19 @@ void NZBInfo::ClearCompletedFiles()
 
 void NZBInfo::SetDestDir(const char* szDestDir)
 {
-	if (m_szDestDir)
-	{
-		free(m_szDestDir);
-	}
+	free(m_szDestDir);
 	m_szDestDir = strdup(szDestDir);
 }
 
+void NZBInfo::SetFinalDir(const char* szFinalDir)
+{
+	free(m_szFinalDir);
+	m_szFinalDir = strdup(szFinalDir);
+}
+
 void NZBInfo::SetFilename(const char * szFilename)
 {
-	if (m_szFilename)
-	{
-		free(m_szFilename);
-	}
+	free(m_szFilename);
 	m_szFilename = strdup(szFilename);
 
 	if (!m_szName)
@@ -299,37 +402,37 @@ void NZBInfo::SetFilename(const char * szFilename)
 		char szNZBNicename[1024];
 		MakeNiceNZBName(m_szFilename, szNZBNicename, sizeof(szNZBNicename), true);
 		szNZBNicename[1024-1] = '\0';
+#ifdef WIN32
+		WebUtil::AnsiToUtf8(szNZBNicename, 1024);
+#endif
 		SetName(szNZBNicename);
 	}
 }
 
 void NZBInfo::SetName(const char* szName)
 {
-	if (m_szName)
-	{
-		free(m_szName);
-	}
+	free(m_szName);
 	m_szName = szName ? strdup(szName) : NULL;
 }
 
 void NZBInfo::SetCategory(const char* szCategory)
 {
-	if (m_szCategory)
-	{
-		free(m_szCategory);
-	}
+	free(m_szCategory);
 	m_szCategory = strdup(szCategory);
 }
 
 void NZBInfo::SetQueuedFilename(const char * szQueuedFilename)
 {
-	if (m_szQueuedFilename)
-	{
-		free(m_szQueuedFilename);
-	}
+	free(m_szQueuedFilename);
 	m_szQueuedFilename = strdup(szQueuedFilename);
 }
 
+void NZBInfo::SetDupeKey(const char* szDupeKey)
+{
+	free(m_szDupeKey);
+	m_szDupeKey = strdup(szDupeKey ? szDupeKey : "");
+}
+
 void NZBInfo::MakeNiceNZBName(const char * szNZBFilename, char * szBuffer, int iSize, bool bRemoveExt)
 {
 	char postname[1024];
@@ -355,16 +458,20 @@ void NZBInfo::BuildDestDirName()
 {
 	char szDestDir[1024];
 
-	if (strlen(g_pOptions->GetInterDir()) == 0)
+	if (Util::EmptyStr(g_pOptions->GetInterDir()))
 	{
 		BuildFinalDirName(szDestDir, 1024);
 	}
 	else
 	{
-		snprintf(szDestDir, 1024, "%s%s", g_pOptions->GetInterDir(), GetName());
+		snprintf(szDestDir, 1024, "%s%s.#%i", g_pOptions->GetInterDir(), GetName(), GetID());
 		szDestDir[1024-1] = '\0';
 	}
 
+#ifdef WIN32
+	WebUtil::Utf8ToAnsi(szDestDir, 1024);
+#endif
+
 	SetDestDir(szDestDir);
 }
 
@@ -378,7 +485,7 @@ void NZBInfo::BuildFinalDirName(char* szFinalDirBuf, int iBufSize)
 
 	if (bUseCategory)
 	{
-		Options::Category *pCategory = g_pOptions->FindCategory(m_szCategory);
+		Options::Category *pCategory = g_pOptions->FindCategory(m_szCategory, false);
 		if (pCategory && pCategory->GetDestDir() && pCategory->GetDestDir()[0] != '\0')
 		{
 			snprintf(szFinalDirBuf, iBufSize, "%s", pCategory->GetDestDir());
@@ -402,11 +509,47 @@ void NZBInfo::BuildFinalDirName(char* szFinalDirBuf, int iBufSize)
 	snprintf(szBuffer, 1024, "%s%s", szFinalDirBuf, GetName());
 	szBuffer[1024-1] = '\0';
 	strncpy(szFinalDirBuf, szBuffer, iBufSize);
+
+#ifdef WIN32
+	WebUtil::Utf8ToAnsi(szFinalDirBuf, iBufSize);
+#endif
 }
 
-void NZBInfo::SetParameter(const char* szName, const char* szValue)
+int NZBInfo::CalcHealth()
 {
-	m_ppParameters.SetParameter(szName, szValue);
+	if (m_lCurrentFailedSize == 0 || m_lSize == m_lParSize)
+	{
+		return 1000;
+	}
+
+	int iHealth = (int)(Util::Int64ToFloat(m_lSize - m_lParSize -
+		(m_lCurrentFailedSize - m_lParCurrentFailedSize)) * 1000.0 /
+		Util::Int64ToFloat(m_lSize - m_lParSize));
+
+	if (iHealth == 1000 && m_lCurrentFailedSize - m_lParCurrentFailedSize > 0)
+	{
+		iHealth = 999;
+	}
+
+	return iHealth;
+}
+
+int NZBInfo::CalcCriticalHealth()
+{
+	long long lGoodParSize = m_lParSize - m_lParCurrentFailedSize;
+	int iCriticalHealth = (int)(Util::Int64ToFloat(m_lSize - lGoodParSize*2) * 1000.0 /
+		Util::Int64ToFloat(m_lSize - lGoodParSize));
+
+	if (lGoodParSize*2 > m_lSize)
+	{
+		iCriticalHealth = 0;
+	}
+	else if (iCriticalHealth == 1000 && m_lParSize > 0)
+	{
+		iCriticalHealth = 999;
+	}
+
+	return iCriticalHealth;
 }
 
 NZBInfo::Messages* NZBInfo::LockMessages()
@@ -485,27 +628,19 @@ ArticleInfo::~ ArticleInfo()
 {
 	//debug("Destroying ArticleInfo");
 
-	if (m_szMessageID)
-	{
-		free(m_szMessageID);
-	}
-	if (m_szResultFilename)
-	{
-		free(m_szResultFilename);
-	}
+	free(m_szMessageID);
+	free(m_szResultFilename);
 }
 
 void ArticleInfo::SetMessageID(const char * szMessageID)
 {
+	free(m_szMessageID);
 	m_szMessageID = strdup(szMessageID);
 }
 
 void ArticleInfo::SetResultFilename(const char * v)
 {
-	if (m_szResultFilename)
-	{
-		free(m_szResultFilename);
-	}
+	free(m_szResultFilename);
 	m_szResultFilename = strdup(v);
 }
 
@@ -523,40 +658,35 @@ FileInfo::FileInfo()
 	m_bFilenameConfirmed = false;
 	m_lSize = 0;
 	m_lRemainingSize = 0;
+	m_lMissedSize = 0;
+	m_lSuccessSize = 0;
+	m_lFailedSize = 0;
+	m_iTotalArticles = 0;
+	m_iMissedArticles = 0;
+	m_iFailedArticles = 0;
+	m_iSuccessArticles = 0;
 	m_tTime = 0;
 	m_bPaused = false;
 	m_bDeleted = false;
 	m_iCompleted = 0;
+	m_bParFile = false;
 	m_bOutputInitialized = false;
 	m_pNZBInfo = NULL;
 	m_iPriority = 0;
 	m_bExtraPriority = false;
 	m_iActiveDownloads = 0;
 	m_bAutoDeleted = false;
-	m_iIDGen++;
-	m_iID = m_iIDGen;
+	m_iID = ++m_iIDGen;
 }
 
 FileInfo::~ FileInfo()
 {
 	debug("Destroying FileInfo");
 
-	if (m_szSubject)
-	{
-		free(m_szSubject);
-	}
-	if (m_szFilename)
-	{
-		free(m_szFilename);
-	}
-	if (m_szOutputFilename)
-	{
-		free(m_szOutputFilename);
-	}
-	if (m_pMutexOutputFile)
-	{
-		delete m_pMutexOutputFile;
-	}
+	free(m_szSubject);
+	free(m_szFilename);
+	free(m_szOutputFilename);
+	delete m_pMutexOutputFile;
 
 	for (Groups::iterator it = m_Groups.begin(); it != m_Groups.end() ;it++)
 	{
@@ -584,9 +714,22 @@ void FileInfo::ClearArticles()
 void FileInfo::SetID(int iID)
 {
 	m_iID = iID;
-	if (m_iIDGen < m_iID)
+	if (m_iIDMax < m_iID)
 	{
-		m_iIDGen = m_iID;
+		m_iIDMax = m_iID;
+	}
+}
+
+void FileInfo::ResetGenID(bool bMax)
+{
+	if (bMax)
+	{
+		m_iIDGen = m_iIDMax;
+	}
+	else
+	{
+		m_iIDGen = 0;
+		m_iIDMax = 0;
 	}
 }
 
@@ -597,7 +740,7 @@ void FileInfo::SetNZBInfo(NZBInfo* pNZBInfo)
 		m_pNZBInfo->Release();
 	}
 	m_pNZBInfo = pNZBInfo;
-	m_pNZBInfo->AddReference();
+	m_pNZBInfo->Retain();
 }
 
 void FileInfo::SetSubject(const char* szSubject)
@@ -607,10 +750,7 @@ void FileInfo::SetSubject(const char* szSubject)
 
 void FileInfo::SetFilename(const char* szFilename)
 {
-	if (m_szFilename)
-	{
-		free(m_szFilename);
-	}
+	free(m_szFilename);
 	m_szFilename = strdup(szFilename);
 }
 
@@ -631,10 +771,7 @@ void FileInfo::UnlockOutputFile()
 
 void FileInfo::SetOutputFilename(const char* szOutputFilename)
 {
-	if (m_szOutputFilename)
-	{
-		free(m_szOutputFilename);
-	}
+	free(m_szOutputFilename);
 	m_szOutputFilename = strdup(szOutputFilename);
 }
 
@@ -653,24 +790,6 @@ void FileInfo::SetActiveDownloads(int iActiveDownloads)
 	}
 }
 
-bool FileInfo::IsDupe(const char* szFilename)
-{
-	char fileName[1024];
-	snprintf(fileName, 1024, "%s%c%s", m_pNZBInfo->GetDestDir(), (int)PATH_SEPARATOR, szFilename);
-	fileName[1024-1] = '\0';
-	if (Util::FileExists(fileName))
-	{
-		return true;
-	}
-	snprintf(fileName, 1024, "%s%c%s_broken", m_pNZBInfo->GetDestDir(), (int)PATH_SEPARATOR, szFilename);
-	fileName[1024-1] = '\0';
-	if (Util::FileExists(fileName))
-	{
-		return true;
-	}
-
-	return false;
-}
 
 GroupInfo::GroupInfo()
 {
@@ -696,6 +815,22 @@ GroupInfo::~GroupInfo()
 	}
 }
 
+
+GroupQueue::~GroupQueue()
+{
+	Clear();
+}
+
+void GroupQueue::Clear()
+{
+	for (iterator it = begin(); it != end(); it++)
+	{
+		delete *it;
+	}
+	clear();
+}
+
+
 PostInfo::PostInfo()
 {
 	debug("Creating PostInfo");
@@ -705,7 +840,6 @@ PostInfo::PostInfo()
 	m_bWorking = false;
 	m_bDeleted = false;
 	m_bRequestParCheck = false;
-	m_bRequestParRename = false;
 	m_szProgressLabel = strdup("");
 	m_iFileProgress = 0;
 	m_iStageProgress = 0;
@@ -715,22 +849,15 @@ PostInfo::PostInfo()
 	m_pPostThread = NULL;
 	m_Messages.clear();
 	m_iIDMessageGen = 0;
-	m_iIDGen++;
-	m_iID = m_iIDGen;
+	m_iID = ++m_iIDGen;
 }
 
 PostInfo::~ PostInfo()
 {
 	debug("Destroying PostInfo");
 
-	if (m_szInfoName)
-	{
-		free(m_szInfoName);
-	}
-	if (m_szProgressLabel)
-	{
-		free(m_szProgressLabel);
-	}
+	free(m_szInfoName);
+	free(m_szProgressLabel);
 
 	for (Messages::iterator it = m_Messages.begin(); it != m_Messages.end(); it++)
 	{
@@ -751,7 +878,7 @@ void PostInfo::SetNZBInfo(NZBInfo* pNZBInfo)
 		m_pNZBInfo->Release();
 	}
 	m_pNZBInfo = pNZBInfo;
-	m_pNZBInfo->AddReference();
+	m_pNZBInfo->Retain();
 }
 
 void PostInfo::SetInfoName(const char* szInfoName)
@@ -761,10 +888,7 @@ void PostInfo::SetInfoName(const char* szInfoName)
 
 void PostInfo::SetProgressLabel(const char* szProgressLabel)
 {
-	if (m_szProgressLabel)
-	{
-		free(m_szProgressLabel);
-	}
+	free(m_szProgressLabel);
 	m_szProgressLabel = strdup(szProgressLabel);
 }
 
@@ -794,6 +918,7 @@ void PostInfo::AppendMessage(Message::EKind eKind, const char * szText)
 	m_mutexLog.Unlock();
 }
 
+
 void DownloadQueue::BuildGroups(GroupQueue* pGroupQueue)
 {
 	std::map<int, GroupInfo*> groupMap;
@@ -806,7 +931,7 @@ void DownloadQueue::BuildGroups(GroupQueue* pGroupQueue)
 		{
 			pGroupInfo = new GroupInfo();
 			pGroupInfo->m_pNZBInfo = pFileInfo->GetNZBInfo();
-			pGroupInfo->m_pNZBInfo->AddReference();
+			pGroupInfo->m_pNZBInfo->Retain();
 			pGroupInfo->m_iFirstID = pFileInfo->GetID();
 			pGroupInfo->m_iLastID = pFileInfo->GetID();
 			pGroupInfo->m_tMinTime = pFileInfo->GetTime();
@@ -866,70 +991,75 @@ void DownloadQueue::BuildGroups(GroupQueue* pGroupQueue)
 
 UrlInfo::UrlInfo()
 {
-	//debug("Creating ArticleInfo");
+	//debug("Creating UrlInfo");
 	m_szURL = NULL;
 	m_szNZBFilename = strdup("");
 	m_szCategory = strdup("");
 	m_iPriority = 0;
+	m_iDupeScore = 0;
+	m_szDupeKey = strdup("");
+	m_eDupeMode = dmScore;
 	m_bAddTop = false;
 	m_bAddPaused = false;
+	m_bForce = false;
 	m_eStatus = aiUndefined;
-	m_iIDGen++;
-	m_iID = m_iIDGen;
+	m_iID = ++m_iIDGen;
 }
 
 UrlInfo::~ UrlInfo()
 {
-	if (m_szURL)
-	{
-		free(m_szURL);
-	}
-	if (m_szNZBFilename)
-	{
-		free(m_szNZBFilename);
-	}
-	if (m_szCategory)
-	{
-		free(m_szCategory);
-	}
+	free(m_szURL);
+	free(m_szNZBFilename);
+	free(m_szCategory);
+	free(m_szDupeKey);
 }
 
 void UrlInfo::SetURL(const char* szURL)
 {
-	if (m_szURL)
-	{
-		free(m_szURL);
-	}
+	free(m_szURL);
 	m_szURL = strdup(szURL);
 }
 
 void UrlInfo::SetID(int iID)
 {
 	m_iID = iID;
-	if (m_iIDGen < m_iID)
+	if (m_iIDMax < m_iID)
 	{
-		m_iIDGen = m_iID;
+		m_iIDMax = m_iID;
+	}
+}
+
+void UrlInfo::ResetGenID(bool bMax)
+{
+	if (bMax)
+	{
+		m_iIDGen = m_iIDMax;
+	}
+	else
+	{
+		m_iIDGen = 0;
+		m_iIDMax = 0;
 	}
 }
 
 void UrlInfo::SetNZBFilename(const char* szNZBFilename)
 {
-	if (m_szNZBFilename)
-	{
-		free(m_szNZBFilename);
-	}
+	free(m_szNZBFilename);
 	m_szNZBFilename = strdup(szNZBFilename);
 }
 
 void UrlInfo::SetCategory(const char* szCategory)
 {
-	if (m_szCategory)
-	{
-		free(m_szCategory);
-	}
+	free(m_szCategory);
 	m_szCategory = strdup(szCategory);
 }
 
+void UrlInfo::SetDupeKey(const char* szDupeKey)
+{
+	free(m_szDupeKey);
+	m_szDupeKey = strdup(szDupeKey);
+}
+
 void UrlInfo::GetName(char* szBuffer, int iSize)
 {
 	MakeNiceName(m_szURL, m_szNZBFilename, szBuffer, iSize);
@@ -954,14 +1084,44 @@ void UrlInfo::MakeNiceName(const char* szURL, const char* szNZBFilename, char* s
 }
 
 
+DupInfo::DupInfo()
+{
+	m_szName = NULL;
+	m_szDupeKey = NULL;
+	m_iDupeScore = 0;
+	m_eDupeMode = dmScore;
+	m_lSize = 0;
+	m_iFullContentHash = 0;
+	m_iFilteredContentHash = 0;
+	m_eStatus = dsUndefined;
+}
+
+DupInfo::~DupInfo()
+{
+	free(m_szName);
+	free(m_szDupeKey);
+}
+
+void DupInfo::SetName(const char* szName)
+{
+	free(m_szName);
+	m_szName = strdup(szName);
+}
+
+void DupInfo::SetDupeKey(const char* szDupeKey)
+{
+	free(m_szDupeKey);
+	m_szDupeKey = strdup(szDupeKey);
+}
+
+
 HistoryInfo::HistoryInfo(NZBInfo* pNZBInfo)
 {
 	m_eKind = hkNZBInfo;
 	m_pInfo = pNZBInfo;
-	pNZBInfo->AddReference();
+	pNZBInfo->Retain();
 	m_tTime = 0;
-	m_iIDGen++;
-	m_iID = m_iIDGen;
+	m_iID = ++m_iIDGen;
 }
 
 HistoryInfo::HistoryInfo(UrlInfo* pUrlInfo)
@@ -969,8 +1129,15 @@ HistoryInfo::HistoryInfo(UrlInfo* pUrlInfo)
 	m_eKind = hkUrlInfo;
 	m_pInfo = pUrlInfo;
 	m_tTime = 0;
-	m_iIDGen++;
-	m_iID = m_iIDGen;
+	m_iID = ++m_iIDGen;
+}
+
+HistoryInfo::HistoryInfo(DupInfo* pDupInfo)
+{
+	m_eKind = hkDupInfo;
+	m_pInfo = pDupInfo;
+	m_tTime = 0;
+	m_iID = ++m_iIDGen;
 }
 
 HistoryInfo::~HistoryInfo()
@@ -983,14 +1150,31 @@ HistoryInfo::~HistoryInfo()
 	{
 		delete (UrlInfo*)m_pInfo;
 	}
+	else if (m_eKind == hkDupInfo && m_pInfo)
+	{
+		delete (DupInfo*)m_pInfo;
+	}
 }
 
 void HistoryInfo::SetID(int iID)
 {
 	m_iID = iID;
-	if (m_iIDGen < m_iID)
+	if (m_iIDMax < m_iID)
 	{
-		m_iIDGen = m_iID;
+		m_iIDMax = m_iID;
+	}
+}
+
+void HistoryInfo::ResetGenID(bool bMax)
+{
+	if (bMax)
+	{
+		m_iIDGen = m_iIDMax;
+	}
+	else
+	{
+		m_iIDGen = 0;
+		m_iIDMax = 0;
 	}
 }
 
@@ -1005,6 +1189,11 @@ void HistoryInfo::GetName(char* szBuffer, int iSize)
 	{
 		GetUrlInfo()->GetName(szBuffer, iSize);
 	}
+	else if (m_eKind == hkDupInfo)
+	{
+		strncpy(szBuffer, GetDupInfo()->GetName(), iSize);
+		szBuffer[iSize-1] = '\0';
+	}
 	else
 	{
 		strncpy(szBuffer, "<unknown>", iSize);
