@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 708 $
- * $Date: 2013-06-13 22:51:38 +0200 (Thu, 13 Jun 2013) $
+ * $Revision: 921 $
+ * $Date: 2013-12-18 21:19:42 +0100 (Wed, 18 Dec 2013) $
  *
  */
 
@@ -85,10 +85,18 @@ private:
 	char*				m_szFilename;
 	long long 			m_lSize;
 	long long 			m_lRemainingSize;
+	long long			m_lSuccessSize;
+	long long			m_lFailedSize;
+	long long			m_lMissedSize;
+	int					m_iTotalArticles;
+	int					m_iMissedArticles;
+	int					m_iFailedArticles;
+	int					m_iSuccessArticles;
 	time_t				m_tTime;
 	bool				m_bPaused;
 	bool				m_bDeleted;
 	bool				m_bFilenameConfirmed;
+	bool				m_bParFile;
 	int					m_iCompleted;
 	bool				m_bOutputInitialized;
 	char*				m_szOutputFilename;
@@ -99,12 +107,14 @@ private:
 	bool				m_bAutoDeleted;
 
 	static int			m_iIDGen;
+	static int			m_iIDMax;
 
 public:
 						FileInfo();
 						~FileInfo();
 	int					GetID() { return m_iID; }
 	void				SetID(int iID);
+	static void			ResetGenID(bool bMax);
 	NZBInfo*			GetNZBInfo() { return m_pNZBInfo; }
 	void				SetNZBInfo(NZBInfo* pNZBInfo);
 	Articles* 			GetArticles() { return &m_Articles; }
@@ -116,10 +126,24 @@ public:
 	void				MakeValidFilename();
 	bool				GetFilenameConfirmed() { return m_bFilenameConfirmed; }
 	void				SetFilenameConfirmed(bool bFilenameConfirmed) { m_bFilenameConfirmed = bFilenameConfirmed; }
-	void 				SetSize(long long s) { m_lSize = s; m_lRemainingSize = s; }
+	void 				SetSize(long long lSize) { m_lSize = lSize; m_lRemainingSize = lSize; }
 	long long 			GetSize() { return m_lSize; }
 	long long 			GetRemainingSize() { return m_lRemainingSize; }
-	void 				SetRemainingSize(long long s) { m_lRemainingSize = s; }
+	void 				SetRemainingSize(long long lRemainingSize) { m_lRemainingSize = lRemainingSize; }
+	long long			GetMissedSize() { return m_lMissedSize; }
+	void 				SetMissedSize(long long lMissedSize) { m_lMissedSize = lMissedSize; }
+	long long			GetSuccessSize() { return m_lSuccessSize; }
+	void 				SetSuccessSize(long long lSuccessSize) { m_lSuccessSize = lSuccessSize; }
+	long long			GetFailedSize() { return m_lFailedSize; }
+	void 				SetFailedSize(long long lFailedSize) { m_lFailedSize = lFailedSize; }
+	int					GetTotalArticles() { return m_iTotalArticles; }
+	void 				SetTotalArticles(int iTotalArticles) { m_iTotalArticles = iTotalArticles; }
+	int					GetMissedArticles() { return m_iMissedArticles; }
+	void 				SetMissedArticles(int iMissedArticles) { m_iMissedArticles = iMissedArticles; }
+	int					GetFailedArticles() { return m_iFailedArticles; }
+	void 				SetFailedArticles(int iFailedArticles) { m_iFailedArticles = iFailedArticles; }
+	int					GetSuccessArticles() { return m_iSuccessArticles; }
+	void 				SetSuccessArticles(int iSuccessArticles) { m_iSuccessArticles = iSuccessArticles; }
 	time_t				GetTime() { return m_tTime; }
 	void				SetTime(time_t tTime) { m_tTime = tTime; }
 	bool				GetPaused() { return m_bPaused; }
@@ -127,7 +151,9 @@ public:
 	bool				GetDeleted() { return m_bDeleted; }
 	void				SetDeleted(bool Deleted) { m_bDeleted = Deleted; }
 	int					GetCompleted() { return m_iCompleted; }
-	void				SetCompleted(int s) { m_iCompleted = s; }
+	void				SetCompleted(int iCompleted) { m_iCompleted = iCompleted; }
+	bool				GetParFile() { return m_bParFile; }
+	void				SetParFile(bool bParFile) { m_bParFile = bParFile; }
 	void				ClearArticles();
 	void				LockOutputFile();
 	void				UnlockOutputFile();
@@ -135,7 +161,6 @@ public:
 	void 				SetOutputFilename(const char* szOutputFilename);
 	bool				GetOutputInitialized() { return m_bOutputInitialized; }
 	void				SetOutputInitialized(bool bOutputInitialized) { m_bOutputInitialized = bOutputInitialized; }
-	bool				IsDupe(const char* szFilename);
 	int					GetPriority() { return m_iPriority; }
 	void				SetPriority(int iPriority) { m_iPriority = iPriority; }
 	bool				GetExtraPriority() { return m_bExtraPriority; }
@@ -185,7 +210,15 @@ public:
 	int					GetActiveDownloads() { return m_iActiveDownloads; }
 };
 
-typedef std::deque<GroupInfo*> GroupQueue;
+typedef std::deque<GroupInfo*> GroupQueueBase;
+
+class GroupQueue : public GroupQueueBase
+{
+public:
+						~GroupQueue();
+	void				Clear();
+};
+
 
 class NZBParameter
 {
@@ -209,7 +242,11 @@ typedef std::deque<NZBParameter*> NZBParameterListBase;
 class NZBParameterList : public NZBParameterListBase
 {
 public:
+						~NZBParameterList();
 	void				SetParameter(const char* szName, const char* szValue);
+	NZBParameter*		Find(const char* szName, bool bCaseSensitive);
+	void				Clear();
+	void				CopyFrom(NZBParameterList* pSourceParameters);
 };
 
 class ScriptStatus
@@ -246,6 +283,40 @@ public:
 	ScriptStatus::EStatus	CalcTotalStatus();
 };
 
+class ServerStat
+{
+private:
+	int					m_iServerID;
+	int					m_iSuccessArticles;
+	int					m_iFailedArticles;
+
+public:
+						ServerStat(int iServerID);
+	int					GetServerID() { return m_iServerID; }
+	int					GetSuccessArticles() { return m_iSuccessArticles; }
+	void				SetSuccessArticles(int iSuccessArticles) { m_iSuccessArticles = iSuccessArticles; }
+	int					GetFailedArticles() { return m_iFailedArticles; }
+	void				SetFailedArticles(int iFailedArticles) { m_iFailedArticles = iFailedArticles; }
+};
+
+typedef std::vector<ServerStat*>	ServerStatListBase;
+
+class ServerStatList : public ServerStatListBase
+{
+public:
+						~ServerStatList();
+	void				SetStat(int iServerID, int iSuccessArticles, int iFailedArticles, bool bAdd);
+	void				Add(ServerStatList* pServerStats);
+	void				Clear();
+};
+
+enum EDupeMode
+{
+	dmScore,
+	dmAll,
+	dmForce
+};
+
 class NZBInfoList;
 
 class NZBInfo
@@ -274,7 +345,9 @@ public:
 		usNone,
 		usSkipped,
 		usFailure,
-		usSuccess
+		usSuccess,
+		usSpace,
+		usPassword
 	};
 
 	enum ECleanupStatus
@@ -291,6 +364,21 @@ public:
 		msSuccess
 	};
 
+	enum EDeleteStatus
+	{
+		dsNone,
+		dsManual,
+		dsHealth,
+		dsDupe
+	};
+
+	enum EMarkStatus
+	{
+		ksNone,
+		ksBad,
+		ksGood
+	};
+
 	typedef std::vector<char*>			Files;
 	typedef std::deque<Message*>		Messages;
 
@@ -300,10 +388,23 @@ private:
 	char* 				m_szFilename;
 	char*				m_szName;
 	char* 				m_szDestDir;
+	char* 				m_szFinalDir;
 	char* 				m_szCategory;
 	int		 			m_iFileCount;
 	int		 			m_iParkedFileCount;
 	long long 			m_lSize;
+	long long			m_lSuccessSize;
+	long long			m_lFailedSize;
+	long long			m_lCurrentSuccessSize;
+	long long			m_lCurrentFailedSize;
+	long long			m_lParSize;
+	long long			m_lParSuccessSize;
+	long long			m_lParFailedSize;
+	long long			m_lParCurrentSuccessSize;
+	long long			m_lParCurrentFailedSize;
+	int					m_iTotalArticles;
+	int					m_iSuccessArticles;
+	int					m_iFailedArticles;
 	Files				m_completedFiles;
 	bool				m_bPostProcess;
 	ERenameStatus		m_eRenameStatus;
@@ -311,45 +412,85 @@ private:
 	EUnpackStatus		m_eUnpackStatus;
 	ECleanupStatus		m_eCleanupStatus;
 	EMoveStatus			m_eMoveStatus;
+	EDeleteStatus		m_eDeleteStatus;
+	EMarkStatus			m_eMarkStatus;
+	bool				m_bDeletePaused;
+	bool				m_bManyDupeFiles;
 	char*				m_szQueuedFilename;
-	bool				m_bDeleted;
+	bool				m_bDeleting;
+	bool				m_bAvoidHistory;
+	bool				m_bHealthPaused;
 	bool				m_bParCleanup;
 	bool				m_bParManual;
 	bool				m_bCleanupDisk;
 	bool				m_bUnpackCleanedUpDisk;
+	char*				m_szDupeKey;
+	int					m_iDupeScore;
+	EDupeMode			m_eDupeMode;
+	unsigned int		m_iFullContentHash;
+	unsigned int		m_iFilteredContentHash;
 	NZBInfoList*		m_Owner;
 	NZBParameterList	m_ppParameters;
 	ScriptStatusList	m_scriptStatuses;
+	ServerStatList		m_ServerStats;
 	Mutex				m_mutexLog;
 	Messages			m_Messages;
 	int					m_iIDMessageGen;
 
 	static int			m_iIDGen;
+	static int			m_iIDMax;
 
 	friend class NZBInfoList;
 
 public:
-						NZBInfo();
+						NZBInfo(bool bPersistent = true);
 						~NZBInfo();
-	void				AddReference();
+	void				Retain();
 	void				Release();
 	int					GetID() { return m_iID; }
 	void				SetID(int iID);
+	static void			ResetGenID(bool bMax);
 	const char*			GetFilename() { return m_szFilename; }
 	void				SetFilename(const char* szFilename);
 	static void			MakeNiceNZBName(const char* szNZBFilename, char* szBuffer, int iSize, bool bRemoveExt);
 	const char*			GetDestDir() { return m_szDestDir; }   // needs locking (for shared objects)
 	void				SetDestDir(const char* szDestDir);     // needs locking (for shared objects)
+	const char*			GetFinalDir() { return m_szFinalDir; }   // needs locking (for shared objects)
+	void				SetFinalDir(const char* szFinalDir);     // needs locking (for shared objects)
 	const char*			GetCategory() { return m_szCategory; } // needs locking (for shared objects)
 	void				SetCategory(const char* szCategory);   // needs locking (for shared objects)
 	const char*			GetName() { return m_szName; } 	   // needs locking (for shared objects)
 	void				SetName(const char* szName);	   // needs locking (for shared objects)
-	long long 			GetSize() { return m_lSize; }
-	void 				SetSize(long long lSize) { m_lSize = lSize; }
 	int					GetFileCount() { return m_iFileCount; }
 	void 				SetFileCount(int iFileCount) { m_iFileCount = iFileCount; }
 	int					GetParkedFileCount() { return m_iParkedFileCount; }
 	void 				SetParkedFileCount(int iParkedFileCount) { m_iParkedFileCount = iParkedFileCount; }
+	long long 			GetSize() { return m_lSize; }
+	void 				SetSize(long long lSize) { m_lSize = lSize; }
+	long long			GetSuccessSize() { return m_lSuccessSize; }
+	void 				SetSuccessSize(long long lSuccessSize) { m_lSuccessSize = lSuccessSize; }
+	long long			GetFailedSize() { return m_lFailedSize; }
+	void 				SetFailedSize(long long lFailedSize) { m_lFailedSize = lFailedSize; }
+	long long			GetCurrentSuccessSize() { return m_lCurrentSuccessSize; }
+	void 				SetCurrentSuccessSize(long long lCurrentSuccessSize) { m_lCurrentSuccessSize = lCurrentSuccessSize; }
+	long long			GetCurrentFailedSize() { return m_lCurrentFailedSize; }
+	void 				SetCurrentFailedSize(long long lCurrentFailedSize) { m_lCurrentFailedSize = lCurrentFailedSize; }
+	long long			GetParSize() { return m_lParSize; }
+	void 				SetParSize(long long lParSize) { m_lParSize = lParSize; }
+	long long			GetParSuccessSize() { return m_lParSuccessSize; }
+	void 				SetParSuccessSize(long long lParSuccessSize) { m_lParSuccessSize = lParSuccessSize; }
+	long long			GetParFailedSize() { return m_lParFailedSize; }
+	void 				SetParFailedSize(long long lParFailedSize) { m_lParFailedSize = lParFailedSize; }
+	long long			GetParCurrentSuccessSize() { return m_lParCurrentSuccessSize; }
+	void 				SetParCurrentSuccessSize(long long lParCurrentSuccessSize) { m_lParCurrentSuccessSize = lParCurrentSuccessSize; }
+	long long			GetParCurrentFailedSize() { return m_lParCurrentFailedSize; }
+	void 				SetParCurrentFailedSize(long long lParCurrentFailedSize) { m_lParCurrentFailedSize = lParCurrentFailedSize; }
+	int					GetTotalArticles() { return m_iTotalArticles; }
+	void 				SetTotalArticles(int iTotalArticles) { m_iTotalArticles = iTotalArticles; }
+	int					GetSuccessArticles() { return m_iSuccessArticles; }
+	void 				SetSuccessArticles(int iSuccessArticles) { m_iSuccessArticles = iSuccessArticles; }
+	int					GetFailedArticles() { return m_iFailedArticles; }
+	void 				SetFailedArticles(int iFailedArticles) { m_iFailedArticles = iFailedArticles; }
 	void				BuildDestDirName();
 	void				BuildFinalDirName(char* szFinalDirBuf, int iBufSize);
 	Files*				GetCompletedFiles() { return &m_completedFiles; }		// needs locking (for shared objects)
@@ -366,10 +507,22 @@ public:
 	void				SetCleanupStatus(ECleanupStatus eCleanupStatus) { m_eCleanupStatus = eCleanupStatus; }
 	EMoveStatus			GetMoveStatus() { return m_eMoveStatus; }
 	void				SetMoveStatus(EMoveStatus eMoveStatus) { m_eMoveStatus = eMoveStatus; }
+	EDeleteStatus		GetDeleteStatus() { return m_eDeleteStatus; }
+	void				SetDeleteStatus(EDeleteStatus eDeleteStatus) { m_eDeleteStatus = eDeleteStatus; }
+	EMarkStatus			GetMarkStatus() { return m_eMarkStatus; }
+	void				SetMarkStatus(EMarkStatus eMarkStatus) { m_eMarkStatus = eMarkStatus; }
 	const char*			GetQueuedFilename() { return m_szQueuedFilename; }
 	void				SetQueuedFilename(const char* szQueuedFilename);
-	bool				GetDeleted() { return m_bDeleted; }
-	void				SetDeleted(bool bDeleted) { m_bDeleted = bDeleted; }
+	bool				GetDeleting() { return m_bDeleting; }
+	void				SetDeleting(bool bDeleting) { m_bDeleting = bDeleting; }
+	bool				GetDeletePaused() { return m_bDeletePaused; }
+	void				SetDeletePaused(bool bDeletePaused) { m_bDeletePaused = bDeletePaused; }
+	bool				GetManyDupeFiles() { return m_bManyDupeFiles; }
+	void				SetManyDupeFiles(bool bManyDupeFiles) { m_bManyDupeFiles = bManyDupeFiles; }
+	bool				GetAvoidHistory() { return m_bAvoidHistory; }
+	void				SetAvoidHistory(bool bAvoidHistory) { m_bAvoidHistory = bAvoidHistory; }
+	bool				GetHealthPaused() { return m_bHealthPaused; }
+	void				SetHealthPaused(bool bHealthPaused) { m_bHealthPaused = bHealthPaused; }
 	bool				GetParCleanup() { return m_bParCleanup; }
 	void				SetParCleanup(bool bParCleanup) { m_bParCleanup = bParCleanup; }
 	bool				GetCleanupDisk() { return m_bCleanupDisk; }
@@ -377,8 +530,20 @@ public:
 	bool				GetUnpackCleanedUpDisk() { return m_bUnpackCleanedUpDisk; }
 	void				SetUnpackCleanedUpDisk(bool bUnpackCleanedUpDisk) { m_bUnpackCleanedUpDisk = bUnpackCleanedUpDisk; }
 	NZBParameterList*	GetParameters() { return &m_ppParameters; }				// needs locking (for shared objects)
-	void				SetParameter(const char* szName, const char* szValue);	// needs locking (for shared objects)
 	ScriptStatusList*	GetScriptStatuses() { return &m_scriptStatuses; }        // needs locking (for shared objects)
+	ServerStatList*		GetServerStats() { return &m_ServerStats; }
+	int					CalcHealth();
+	int					CalcCriticalHealth();
+	const char*			GetDupeKey() { return m_szDupeKey; }					// needs locking (for shared objects)
+	void				SetDupeKey(const char* szDupeKey);						// needs locking (for shared objects)
+	int					GetDupeScore() { return m_iDupeScore; }
+	void				SetDupeScore(int iDupeScore) { m_iDupeScore = iDupeScore; }
+	EDupeMode			GetDupeMode() { return m_eDupeMode; }
+	void				SetDupeMode(EDupeMode eDupeMode) { m_eDupeMode = eDupeMode; }
+	unsigned int		GetFullContentHash() { return m_iFullContentHash; }
+	void				SetFullContentHash(unsigned int iFullContentHash) { m_iFullContentHash = iFullContentHash; }
+	unsigned int		GetFilteredContentHash() { return m_iFilteredContentHash; }
+	void				SetFilteredContentHash(unsigned int iFilteredContentHash) { m_iFilteredContentHash = iFilteredContentHash; }
 	void				AppendMessage(Message::EKind eKind, time_t tTime, const char* szText);
 	Messages*			LockMessages();
 	void				UnlockMessages();
@@ -420,7 +585,6 @@ private:
 	bool				m_bWorking;
 	bool				m_bDeleted;
 	bool				m_bRequestParCheck;
-	bool				m_bRequestParRename;
 	EStage				m_eStage;
 	char*				m_szProgressLabel;
 	int					m_iFileProgress;
@@ -434,6 +598,7 @@ private:
 	int					m_iIDMessageGen;
 
 	static int			m_iIDGen;
+	static int			m_iIDMax;
 
 public:
 						PostInfo();
@@ -461,8 +626,6 @@ public:
 	void				SetDeleted(bool bDeleted) { m_bDeleted = bDeleted; }
 	bool				GetRequestParCheck() { return m_bRequestParCheck; }
 	void				SetRequestParCheck(bool bRequestParCheck) { m_bRequestParCheck = bRequestParCheck; }
-	bool				GetRequestParRename() { return m_bRequestParRename; }
-	void				SetRequestParRename(bool bRequestParRename) { m_bRequestParRename = bRequestParRename; }
 	void				AppendMessage(Message::EKind eKind, const char* szText);
 	Thread*				GetPostThread() { return m_pPostThread; }
 	void				SetPostThread(Thread* pPostThread) { m_pPostThread = pPostThread; }
@@ -485,7 +648,9 @@ public:
 		aiRunning,
 		aiFinished,
 		aiFailed,
-		aiRetry
+		aiRetry,
+		aiScanSkipped,
+		aiScanFailed
 	};
 
 private:
@@ -494,17 +659,23 @@ private:
 	char*				m_szNZBFilename;
 	char* 				m_szCategory;
 	int					m_iPriority;
+	char*				m_szDupeKey;
+	int					m_iDupeScore;
+	EDupeMode			m_eDupeMode;
 	bool				m_bAddTop;
 	bool				m_bAddPaused;
+	bool				m_bForce;
 	EStatus				m_eStatus;
 
 	static int			m_iIDGen;
+	static int			m_iIDMax;
 
 public:
 						UrlInfo();
 						~UrlInfo();
 	int					GetID() { return m_iID; }
 	void				SetID(int iID);
+	static void			ResetGenID(bool bMax);
 	const char*			GetURL() { return m_szURL; }			// needs locking (for shared objects)
 	void				SetURL(const char* szURL);				// needs locking (for shared objects)
 	const char*			GetNZBFilename() { return m_szNZBFilename; }		// needs locking (for shared objects)
@@ -513,18 +684,71 @@ public:
 	void				SetCategory(const char* szCategory);	// needs locking (for shared objects)
 	int					GetPriority() { return m_iPriority; }
 	void				SetPriority(int iPriority) { m_iPriority = iPriority; }
+	const char*			GetDupeKey() { return m_szDupeKey; }
+	void				SetDupeKey(const char* szDupeKey);
+	int					GetDupeScore() { return m_iDupeScore; }
+	void				SetDupeScore(int iDupeScore) { m_iDupeScore = iDupeScore; }
+	EDupeMode			GetDupeMode() { return m_eDupeMode; }
+	void				SetDupeMode(EDupeMode eDupeMode) { m_eDupeMode = eDupeMode; }
 	bool				GetAddTop() { return m_bAddTop; }
 	void				SetAddTop(bool bAddTop) { m_bAddTop = bAddTop; }
 	bool				GetAddPaused() { return m_bAddPaused; }
 	void				SetAddPaused(bool bAddPaused) { m_bAddPaused = bAddPaused; }
 	void				GetName(char* szBuffer, int iSize);		// needs locking (for shared objects)
 	static void			MakeNiceName(const char* szURL, const char* szNZBFilename, char* szBuffer, int iSize);
+	bool				GetForce() { return m_bForce; }
+	void				SetForce(bool bForce) { m_bForce = bForce; }
 	EStatus				GetStatus() { return m_eStatus; }
 	void				SetStatus(EStatus Status) { m_eStatus = Status; }
 };
 
 typedef std::deque<UrlInfo*> UrlQueue;
 
+class DupInfo
+{
+public:
+	enum EStatus
+	{
+		dsUndefined,
+		dsSuccess,
+		dsFailed,
+		dsDeleted,
+		dsDupe,
+		dsBad,
+		dsGood
+	};
+
+private:
+	char*				m_szName;
+	char*				m_szDupeKey;
+	int					m_iDupeScore;
+	EDupeMode			m_eDupeMode;
+	long long 			m_lSize;
+	unsigned int		m_iFullContentHash;
+	unsigned int		m_iFilteredContentHash;
+	EStatus				m_eStatus;
+
+public:
+						DupInfo();
+						~DupInfo();
+	const char*			GetName() { return m_szName; }			// needs locking (for shared objects)
+	void				SetName(const char* szName);			// needs locking (for shared objects)
+	const char*			GetDupeKey() { return m_szDupeKey; }	// needs locking (for shared objects)
+	void				SetDupeKey(const char* szDupeKey);		// needs locking (for shared objects)
+	int					GetDupeScore() { return m_iDupeScore; }
+	void				SetDupeScore(int iDupeScore) { m_iDupeScore = iDupeScore; }
+	EDupeMode			GetDupeMode() { return m_eDupeMode; }
+	void				SetDupeMode(EDupeMode eDupeMode) { m_eDupeMode = eDupeMode; }
+	long long			GetSize() { return m_lSize; }
+	void 				SetSize(long long lSize) { m_lSize = lSize; }
+	unsigned int		GetFullContentHash() { return m_iFullContentHash; }
+	void				SetFullContentHash(unsigned int iFullContentHash) { m_iFullContentHash = iFullContentHash; }
+	unsigned int		GetFilteredContentHash() { return m_iFilteredContentHash; }
+	void				SetFilteredContentHash(unsigned int iFilteredContentHash) { m_iFilteredContentHash = iFilteredContentHash; }
+	EStatus				GetStatus() { return m_eStatus; }
+	void				SetStatus(EStatus Status) { m_eStatus = Status; }
+};
+
 class HistoryInfo
 {
 public:
@@ -532,7 +756,8 @@ public:
 	{
 		hkUnknown,
 		hkNZBInfo,
-		hkUrlInfo
+		hkUrlInfo,
+		hkDupInfo
 	};
 
 private:
@@ -542,16 +767,20 @@ private:
 	time_t				m_tTime;
 
 	static int			m_iIDGen;
+	static int			m_iIDMax;
 
 public:
 						HistoryInfo(NZBInfo* pNZBInfo);
 						HistoryInfo(UrlInfo* pUrlInfo);
+						HistoryInfo(DupInfo* pDupInfo);
 						~HistoryInfo();
 	int					GetID() { return m_iID; }
 	void				SetID(int iID);
+	static void			ResetGenID(bool bMax);
 	EKind				GetKind() { return m_eKind; }
 	NZBInfo*			GetNZBInfo() { return (NZBInfo*)m_pInfo; }
 	UrlInfo*			GetUrlInfo() { return (UrlInfo*)m_pInfo; }
+	DupInfo*			GetDupInfo() { return (DupInfo*)m_pInfo; }
 	void				DiscardUrlInfo() { m_pInfo = NULL; }
 	time_t				GetTime() { return m_tTime; }
 	void				SetTime(time_t tTime) { m_tTime = tTime; }
