@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 711 $
- * $Date: 2013-06-17 22:39:46 +0200 (Mon, 17 Jun 2013) $
+ * $Revision: 924 $
+ * $Date: 2013-12-21 22:39:49 +0100 (Sat, 21 Dec 2013) $
  *
  */
 
@@ -27,7 +27,7 @@
 #ifndef MESSAGEBASE_H
 #define MESSAGEBASE_H
 
-static const int32_t NZBMESSAGE_SIGNATURE = 0x6E7A6217; // = "nzb-XX" (protocol version)
+static const int32_t NZBMESSAGE_SIGNATURE = 0x6E7A621B; // = "nzb-XX" (protocol version)
 static const int NZBREQUESTFILENAMESIZE = 512;
 static const int NZBREQUESTPASSWORDSIZE = 32;
 
@@ -88,6 +88,8 @@ enum eRemoteEditAction
 	eRemoteEditActionGroupPause,			// pause group
 	eRemoteEditActionGroupResume,			// resume (unpause) group
 	eRemoteEditActionGroupDelete,			// delete group
+	eRemoteEditActionGroupDupeDelete,		// delete group
+	eRemoteEditActionGroupFinalDelete,		// delete group
 	eRemoteEditActionGroupPauseAllPars,		// pause only (all) pars (does not affect other files) in group
 	eRemoteEditActionGroupPauseExtraPars,	// pause only (almost all) pars in group, except main par-file (does not affect other files)
 	eRemoteEditActionGroupSetPriority,		// set priority for groups
@@ -95,14 +97,25 @@ enum eRemoteEditAction
 	eRemoteEditActionGroupMerge,			// merge group
 	eRemoteEditActionGroupSetParameter,		// set post-process parameter for group
 	eRemoteEditActionGroupSetName,			// set group name (rename group)
+	eRemoteEditActionGroupSetDupeKey,		// (reserved)
+	eRemoteEditActionGroupSetDupeScore,		// (reserved)
+	eRemoteEditActionGroupSetDupeMode,		// (reserved)
 	eRemoteEditActionPostMoveOffset = 51,	// move post-job to m_iOffset relative to the current position in post-queue
 	eRemoteEditActionPostMoveTop,			// move post-job to the top of post-queue
 	eRemoteEditActionPostMoveBottom,		// move post-job to the bottom of post-queue
 	eRemoteEditActionPostDelete,			// delete post-job
-	eRemoteEditActionHistoryDelete,			// delete history-item
+	eRemoteEditActionHistoryDelete,			// hide history-item
+	eRemoteEditActionHistoryFinalDelete,	// delete history-item
 	eRemoteEditActionHistoryReturn,			// move history-item back to download queue
 	eRemoteEditActionHistoryProcess,		// move history-item back to download queue and start postprocessing
-	eRemoteEditActionHistorySetParameter	// set post-process parameter for history-item
+	eRemoteEditActionHistoryRedownload,		// move history-item back to download queue for redownload
+	eRemoteEditActionHistorySetParameter,	// set post-process parameter for history-item
+	eRemoteEditActionHistorySetDupeKey,		// (reserved)
+	eRemoteEditActionHistorySetDupeScore,	// (reserved)
+	eRemoteEditActionHistorySetDupeMode,	// (reserved)
+	eRemoteEditActionHistorySetDupeBackup,	// (reserved)
+	eRemoteEditActionHistoryMarkBad,		// mark history-item as bad (and download other duplicate)
+	eRemoteEditActionHistoryMarkGood		// mark history-item as good (and push it into dup-history)
 };
 
 // Possible values for field "m_iAction" of struct "SNZBPauseUnpauseRequest":
