@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 544 $
- * $Date: 2013-01-18 22:36:17 +0100 (Fri, 18 Jan 2013) $
+ * $Revision: 914 $
+ * $Date: 2013-11-28 22:03:01 +0100 (Thu, 28 Nov 2013) $
  *
  */
 
@@ -202,14 +202,9 @@ NCursesFrontend::NCursesFrontend()
 NCursesFrontend::~NCursesFrontend()
 {
 #ifdef WIN32
-	if (m_pScreenBuffer)
-	{
-		free(m_pScreenBuffer);
-	}
-	if (m_pOldScreenBuffer)
-	{
-		free(m_pOldScreenBuffer);
-	}
+	free(m_pScreenBuffer);
+	free(m_pOldScreenBuffer);
+
 	m_ColorAttr.clear();
 
 	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
@@ -550,6 +545,8 @@ int NCursesFrontend::PrintMessage(Message* Msg, int iRow, int iMaxLines)
 		szText = (char*)malloc(iLen);
 
 		time_t rawtime = Msg->GetTime();
+		rawtime += g_pOptions->GetTimeCorrection();
+
 		char szTime[50];
 #ifdef HAVE_CTIME_R_3
 		ctime_r(&rawtime, szTime, 50);
@@ -723,11 +720,8 @@ void NCursesFrontend::PrintKeyInputBar()
 
 void NCursesFrontend::SetHint(const char* szHint)
 {
-	if (m_szHint)
-	{
-		free(m_szHint);
-		m_szHint = NULL;
-	}
+	free(m_szHint);
+	m_szHint = NULL;
 	if (szHint)
 	{
 		m_szHint = strdup(szHint);
@@ -792,8 +786,8 @@ void NCursesFrontend::PrintFileQueue()
 		
 		char szBuffer[MAX_SCREEN_WIDTH];
 		snprintf(szBuffer, sizeof(szBuffer), " %sFiles for downloading - %i / %i files in queue - %s / %s", 
-			m_bUseColor ? "" : "*** ", pDownloadQueue->GetFileQueue()->size(), 
-			pDownloadQueue->GetFileQueue()->size() - iPausedFiles, szRemaining, szUnpaused);
+			m_bUseColor ? "" : "*** ", (int)pDownloadQueue->GetFileQueue()->size(),
+			(int)pDownloadQueue->GetFileQueue()->size() - iPausedFiles, szRemaining, szUnpaused);
 		szBuffer[MAX_SCREEN_WIDTH - 1] = '\0';
 		PrintTopHeader(szBuffer, m_iQueueWinTop, true);
     }
@@ -977,7 +971,7 @@ void NCursesFrontend::PrintGroupQueue()
 		
 		char szBuffer[MAX_SCREEN_WIDTH];
 		snprintf(szBuffer, sizeof(szBuffer), " %sNZBs for downloading - %i NZBs in queue - %s / %s", 
-			m_bUseColor ? "" : "*** ", pGroupQueue->size(), szRemaining, szUnpaused);
+			m_bUseColor ? "" : "*** ", (int)pGroupQueue->size(), szRemaining, szUnpaused);
 		szBuffer[MAX_SCREEN_WIDTH - 1] = '\0';
 		PrintTopHeader(szBuffer, m_iQueueWinTop, false);
     }
@@ -1128,11 +1122,7 @@ void NCursesFrontend::PrepareGroupQueue()
 
 void NCursesFrontend::ClearGroupQueue()
 {
-	for (GroupQueue::iterator it = m_groupQueue.begin(); it != m_groupQueue.end(); it++)
-	{
-		delete *it;
-	}
-	m_groupQueue.clear();
+	m_groupQueue.Clear();
 }
 
 bool NCursesFrontend::EditQueue(QueueEditor::EEditAction eAction, int iOffset)
