@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 414 $
- * $Date: 2012-05-03 15:47:44 +0200 (Thu, 03 May 2012) $
+ * $Revision: 634 $
+ * $Date: 2013-04-17 22:34:15 +0200 (Wed, 17 Apr 2013) $
  *
  */
 
@@ -33,26 +33,26 @@
 class NNTPConnection : public Connection
 {
 private:
-	NewsServer*				m_pNewsServer;
-	char* 					m_szActiveGroup;
-	char*					m_szLineBuf;
-	bool					m_bAuthError;
+	NewsServer*			m_pNewsServer;
+	char* 				m_szActiveGroup;
+	char*				m_szLineBuf;
+	bool				m_bAuthError;
 
-	virtual bool 			DoConnect();
-	virtual bool 			DoDisconnect();
-	void					Clear();
-	void					ReportErrorAnswer(const char* szMsgPrefix, const char* szAnswer);
+	void				Clear();
+	void				ReportErrorAnswer(const char* szMsgPrefix, const char* szAnswer);
+	bool 				Authenticate();
+	bool 				AuthInfoUser(int iRecur);
+	bool 				AuthInfoPass(int iRecur);
 
 public:
-							NNTPConnection(NewsServer* pNewsServer);
-	virtual					~NNTPConnection();
-	NewsServer*				GetNewsServer() { return m_pNewsServer; }
-	const char* 			Request(const char* req);
-	bool 					Authenticate();
-	bool 					AuthInfoUser(int iRecur = 0);
-	bool 					AuthInfoPass(int iRecur = 0);
-	const char*				JoinGroup(const char* grp);
-	bool					GetAuthError() { return m_bAuthError; }
+						NNTPConnection(NewsServer* pNewsServer);
+	virtual				~NNTPConnection();
+	virtual bool 		Connect();
+	virtual bool		Disconnect();
+	NewsServer*			GetNewsServer() { return m_pNewsServer; }
+	const char* 		Request(const char* req);
+	const char*			JoinGroup(const char* grp);
+	bool				GetAuthError() { return m_bAuthError; }
 };
 
 #endif
