@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 711 $
- * $Date: 2013-06-17 22:39:46 +0200 (Mon, 17 Jun 2013) $
+ * $Revision: 911 $
+ * $Date: 2013-11-24 20:29:52 +0100 (Sun, 24 Nov 2013) $
  *
  */
 
@@ -34,6 +34,7 @@
 
 #include <string.h>
 #include <list>
+#include <ctype.h>
 #ifdef WIN32
 #include <comutil.h>
 #import <msxml.tlb> named_guids 
@@ -61,13 +62,15 @@ NZBFile::NZBFile(const char* szFileName, const char* szCategory)
     debug("Creating NZBFile");
 
     m_szFileName = strdup(szFileName);
+	m_szPassword = NULL;
 	m_pNZBInfo = new NZBInfo();
-	m_pNZBInfo->AddReference();
+	m_pNZBInfo->Retain();
 	m_pNZBInfo->SetFilename(szFileName);
 	m_pNZBInfo->SetCategory(szCategory);
 	m_pNZBInfo->BuildDestDirName();
 
 #ifndef WIN32
+	m_bPassword = false;
 	m_pFileInfo = NULL;
 	m_pArticle = NULL;
 	m_szTagContent = NULL;
@@ -82,10 +85,8 @@ NZBFile::~NZBFile()
     debug("Destroying NZBFile");
 
     // Cleanup
-    if (m_szFileName)
-    {
-        free(m_szFileName);
-    }
+    free(m_szFileName);
+    free(m_szPassword);
 
     for (FileInfos::iterator it = m_FileInfos.begin(); it != m_FileInfos.end(); it++)
     {
@@ -99,15 +100,8 @@ NZBFile::~NZBFile()
 	}
 
 #ifndef WIN32
-	if (m_pFileInfo)
-	{
-		delete m_pFileInfo;
-	}
-	
-	if (m_szTagContent)
-	{
-		free(m_szTagContent);
-	}
+	delete m_pFileInfo;
+	free(m_szTagContent);
 #endif
 }
 
@@ -127,39 +121,70 @@ void NZBFile::AddArticle(FileInfo* pFileInfo, ArticleInfo* pArticleInfo)
 	while ((int)pFileInfo->GetArticles()->size() < pArticleInfo->GetPartNumber())
 		pFileInfo->GetArticles()->push_back(NULL);
 
-	(*pFileInfo->GetArticles())[pArticleInfo->GetPartNumber() - 1] = pArticleInfo;
+	int index = pArticleInfo->GetPartNumber() - 1;
+	if ((*pFileInfo->GetArticles())[index])
+	{
+		delete (*pFileInfo->GetArticles())[index];
+	}
+	(*pFileInfo->GetArticles())[index] = pArticleInfo;
 }
 
 void NZBFile::AddFileInfo(FileInfo* pFileInfo)
 {
-	// deleting empty articles
+	// calculate file size and delete empty articles
+
+	long long lSize = 0;
+	long long lMissedSize = 0;
+	long long lOneSize = 0;
+	int iUncountedArticles = 0;
+	int iMissedArticles = 0;
 	FileInfo::Articles* pArticles = pFileInfo->GetArticles();
+	int iTotalArticles = (int)pArticles->size();
 	int i = 0;
-	for (FileInfo::Articles::iterator it = pArticles->begin(); it != pArticles->end();)
+	for (FileInfo::Articles::iterator it = pArticles->begin(); it != pArticles->end(); )
 	{
-		if (*it == NULL)
+		ArticleInfo* pArticle = *it;
+		if (!pArticle)
 		{
 			pArticles->erase(it);
 			it = pArticles->begin() + i;
+			iMissedArticles++;
+			if (lOneSize > 0)
+			{
+				lMissedSize += lOneSize;
+			}
+			else
+			{
+				iUncountedArticles++;
+			}
 		}
 		else
 		{
+			lSize += pArticle->GetSize();
+			if (lOneSize == 0)
+			{
+				lOneSize = pArticle->GetSize();
+			}
 			it++;
 			i++;
 		}
 	}
 
-	if (!pArticles->empty())
+	if (pArticles->empty())
 	{
-		m_FileInfos.push_back(pFileInfo);
-		pFileInfo->SetNZBInfo(m_pNZBInfo);
-		m_pNZBInfo->SetSize(m_pNZBInfo->GetSize() + pFileInfo->GetSize());
-		m_pNZBInfo->SetFileCount(m_pNZBInfo->GetFileCount() + 1);
-	}
-	else
-	{
-		delete pFileInfo; 
+		delete pFileInfo;
+		return;
 	}
+
+	lMissedSize += iUncountedArticles * lOneSize;
+	lSize += lMissedSize;
+	m_FileInfos.push_back(pFileInfo);
+	pFileInfo->SetNZBInfo(m_pNZBInfo);
+	pFileInfo->SetSize(lSize);
+	pFileInfo->SetRemainingSize(lSize - lMissedSize);
+	pFileInfo->SetMissedSize(lMissedSize);
+	pFileInfo->SetTotalArticles(iTotalArticles);
+	pFileInfo->SetMissedArticles(iMissedArticles);
 }
 
 void NZBFile::ParseSubject(FileInfo* pFileInfo, bool TryQuotes)
@@ -308,7 +333,7 @@ bool NZBFile::HasDuplicateFilenames()
 /**
  * Generate filenames from subjects and check if the parsing of subject was correct
  */
-void NZBFile::ProcessFilenames()
+void NZBFile::BuildFilenames()
 {
 	for (FileInfos::iterator it = m_FileInfos.begin(); it != m_FileInfos.end(); it++)
     {
@@ -327,23 +352,185 @@ void NZBFile::ProcessFilenames()
 
 	if (HasDuplicateFilenames())
     {
+		m_pNZBInfo->SetManyDupeFiles(true);
 		for (FileInfos::iterator it = m_FileInfos.begin(); it != m_FileInfos.end(); it++)
 		{
 			FileInfo* pFileInfo = *it;
 			pFileInfo->SetFilename(pFileInfo->GetSubject());
 		}
     }
+}
+
+bool CompareFileInfo(FileInfo* pFirst, FileInfo* pSecond)
+{
+	return strcmp(pFirst->GetFilename(), pSecond->GetFilename()) > 0;
+}
+
+void NZBFile::CalcHashes()
+{
+	FileInfoList fileList;
 
 	for (FileInfos::iterator it = m_FileInfos.begin(); it != m_FileInfos.end(); it++)
-    {
-        FileInfo* pFileInfo = *it;
-		pFileInfo->MakeValidFilename();
-		if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
+	{
+		fileList.push_back(*it);
+	}
+
+	fileList.sort(CompareFileInfo);
+
+	// split ExtCleanupDisk into tokens and create a list
+	ExtList extList;
+	char* szExtCleanupDisk = strdup(g_pOptions->GetExtCleanupDisk());
+	char* saveptr;
+	char* szExt = strtok_r(szExtCleanupDisk, ",; ", &saveptr);
+	while (szExt)
+	{
+		extList.push_back(szExt);
+		szExt = strtok_r(NULL, ",; ", &saveptr);
+	}
+
+	unsigned int iFullContentHash = 0;
+	unsigned int iFilteredContentHash = 0;
+	int iUseForFilteredCount = 0;
+
+	for (FileInfoList::iterator it = fileList.begin(); it != fileList.end(); it++)
+	{
+		FileInfo* pFileInfo = *it;
+
+		// check file extension
+		int iFilenameLen = strlen(pFileInfo->GetFilename());
+		bool bSkip = false;
+		for (ExtList::iterator it = extList.begin(); it != extList.end(); it++)
 		{
+			const char* szExt = *it;
+			int iExtLen = strlen(szExt);
+			if (iFilenameLen >= iExtLen && !strcasecmp(szExt, pFileInfo->GetFilename() + iFilenameLen - iExtLen))
+			{
+				bSkip = true;
+				break;
+			}
+		}
+
+		bSkip = bSkip && !pFileInfo->GetParFile();
+
+		for (FileInfo::Articles::iterator it = pFileInfo->GetArticles()->begin(); it != pFileInfo->GetArticles()->end(); it++)
+		{
+			ArticleInfo* pArticle = *it;
+			int iLen = strlen(pArticle->GetMessageID());
+			iFullContentHash = Util::HashBJ96(pArticle->GetMessageID(), iLen, iFullContentHash);
+			if (!bSkip)
+			{
+				iFilteredContentHash = Util::HashBJ96(pArticle->GetMessageID(), iLen, iFilteredContentHash);
+				iUseForFilteredCount++;
+			}
+		}
+	}
+
+	free(szExtCleanupDisk);
+
+	// if filtered hash is based on less than a half of files - do not use filtered hash at all
+	if (iUseForFilteredCount < (int)fileList.size() / 2)
+	{
+		iFilteredContentHash = 0;
+	}
+
+	m_pNZBInfo->SetFullContentHash(iFullContentHash);
+	m_pNZBInfo->SetFilteredContentHash(iFilteredContentHash);
+}
+
+void NZBFile::ProcessFiles()
+{
+	BuildFilenames();
+
+	for (FileInfos::iterator it = m_FileInfos.begin(); it != m_FileInfos.end(); it++)
+	{
+		FileInfo* pFileInfo = *it;
+		pFileInfo->MakeValidFilename();
+
+		char szLoFileName[1024];
+		strncpy(szLoFileName, pFileInfo->GetFilename(), 1024);
+		szLoFileName[1024-1] = '\0';
+		for (char* p = szLoFileName; *p; p++) *p = tolower(*p); // convert string to lowercase
+		bool bParFile = strstr(szLoFileName, ".par2");
+
+		m_pNZBInfo->SetFileCount(m_pNZBInfo->GetFileCount() + 1);
+		m_pNZBInfo->SetTotalArticles(m_pNZBInfo->GetTotalArticles() + pFileInfo->GetTotalArticles());
+		m_pNZBInfo->SetSize(m_pNZBInfo->GetSize() + pFileInfo->GetSize());
+		m_pNZBInfo->SetFailedSize(m_pNZBInfo->GetFailedSize() + pFileInfo->GetMissedSize());
+		m_pNZBInfo->SetCurrentFailedSize(m_pNZBInfo->GetFailedSize());
+
+		pFileInfo->SetParFile(bParFile);
+		if (bParFile)
+		{
+			m_pNZBInfo->SetParSize(m_pNZBInfo->GetParSize() + pFileInfo->GetSize());
+			m_pNZBInfo->SetParFailedSize(m_pNZBInfo->GetParFailedSize() + pFileInfo->GetMissedSize());
+			m_pNZBInfo->SetParCurrentFailedSize(m_pNZBInfo->GetParFailedSize());
+		}
+	}
+
+	CalcHashes();
+
+	if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
+	{
+		for (FileInfos::iterator it = m_FileInfos.begin(); it != m_FileInfos.end(); it++)
+		{
+			FileInfo* pFileInfo = *it;
 			g_pDiskState->SaveFile(pFileInfo);
 			pFileInfo->ClearArticles();
 		}
 	}
+
+	if (m_szPassword)
+	{
+		ReadPassword();
+	}
+}
+
+/**
+ * Password read using XML-parser may have special characters (such as TAB) stripped.
+ * This function rereads password directly from file to keep all characters intact.
+ */
+void NZBFile::ReadPassword()
+{
+    FILE* pFile = fopen(m_szFileName, "rb");
+    if (!pFile)
+    {
+        return;
+    }
+
+    // obtain file size.
+    fseek(pFile , 0 , SEEK_END);
+    int iSize  = ftell(pFile);
+    rewind(pFile);
+
+	// reading first 4KB of the file
+
+    // allocate memory to contain the whole file.
+    char* buf = (char*)malloc(4096);
+
+	iSize = iSize < 4096 ? iSize : 4096;
+
+    // copy the file into the buffer.
+    fread(buf, 1, iSize, pFile);
+
+    fclose(pFile);
+
+    buf[iSize-1] = '\0';
+
+	char* szMetaPassword = strstr(buf, "<meta type=\"password\">");
+	if (szMetaPassword)
+	{
+		szMetaPassword += 22; // length of '<meta type="password">'
+		char* szEnd = strstr(szMetaPassword, "</meta>");
+		if (szEnd)
+		{
+			*szEnd = '\0';
+			WebUtil::XmlDecode(szMetaPassword);
+			free(m_szPassword);
+			m_szPassword = strdup(szMetaPassword);
+		}
+	}
+
+	free(buf);
 }
 
 #ifdef WIN32
@@ -384,7 +571,7 @@ NZBFile* NZBFile::Create(const char* szFileName, const char* szCategory)
     NZBFile* pFile = new NZBFile(szFileName, szCategory);
     if (pFile->ParseNZB(doc))
 	{
-		pFile->ProcessFilenames();
+		pFile->ProcessFiles();
 	}
 	else
 	{
@@ -408,7 +595,7 @@ void NZBFile::EncodeURL(const char* szFilename, char* szURL)
 		else
 		{
 			*szURL++ = '%';
-			int a = ch >> 4;
+			int a = (unsigned char)ch >> 4;
 			*szURL++ = a > 9 ? a - 10 + 'a' : a + '0';
 			a = ch & 0xF;
 			*szURL++ = a > 9 ? a - 10 + 'a' : a + '0';
@@ -422,10 +609,17 @@ bool NZBFile::ParseNZB(IUnknown* nzb)
 	MSXML::IXMLDOMDocumentPtr doc = nzb;
 	MSXML::IXMLDOMNodePtr root = doc->documentElement;
 
+	MSXML::IXMLDOMNodePtr node = root->selectSingleNode("/nzb/head/meta[@type='password']");
+	if (node)
+	{
+		_bstr_t password(node->Gettext());
+		m_szPassword = strdup(password);
+	}
+
 	MSXML::IXMLDOMNodeListPtr fileList = root->selectNodes("/nzb/file");
 	for (int i = 0; i < fileList->Getlength(); i++)
 	{
-		MSXML::IXMLDOMNodePtr node = fileList->Getitem(i);
+		node = fileList->Getitem(i);
 		MSXML::IXMLDOMNodePtr attribute = node->Getattributes()->getNamedItem("subject");
 		if (!attribute) return false;
 		_bstr_t subject(attribute->Gettext());
@@ -474,11 +668,6 @@ bool NZBFile::ParseNZB(IUnknown* nzb)
 				pArticle->SetSize(lsize);
 				AddArticle(pFileInfo, pArticle);
 			}
-
-            if (lsize > 0)
-            {
-                pFileInfo->SetSize(pFileInfo->GetSize() + lsize);
-            }
 		}
 
 		AddFileInfo(pFileInfo);
@@ -505,7 +694,7 @@ NZBFile* NZBFile::Create(const char* szFileName, const char* szCategory)
     
     if (ret == 0)
 	{
-		pFile->ProcessFilenames();
+		pFile->ProcessFiles();
 	}
 	else
 	{
@@ -569,10 +758,6 @@ void NZBFile::Parse_StartElement(const char *name, const char **atts)
 				partNumber = atol(attrvalue);
 			}
 		}
-		if (lsize > 0)
-		{
-			m_pFileInfo->SetSize(m_pFileInfo->GetSize() + lsize);
-		}
 
 		if (partNumber > 0)
 		{
@@ -583,6 +768,10 @@ void NZBFile::Parse_StartElement(const char *name, const char **atts)
 			AddArticle(m_pFileInfo, m_pArticle);
 		}
 	}
+	else if (!strcmp("meta", name))
+	{
+		m_bPassword = atts[0] && atts[1] && !strcmp("type", atts[0]) && !strcmp("password", atts[1]);
+	}
 }
 
 void NZBFile::Parse_EndElement(const char *name)
@@ -620,6 +809,10 @@ void NZBFile::Parse_EndElement(const char *name)
 		m_pArticle->SetMessageID(ID);
 		m_pArticle = NULL;
 	}
+	else if (!strcmp("meta", name) && m_bPassword)
+	{
+		m_szPassword = strdup(m_szTagContent);
+	}
 }
 
 void NZBFile::Parse_Content(const char *buf, int len)
