@@ -2,7 +2,7 @@
  *  This file if part of nzbget
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2008 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 602 $
- * $Date: 2013-03-17 13:21:46 +0100 (Sun, 17 Mar 2013) $
+ * $Revision: 911 $
+ * $Date: 2013-11-24 20:29:52 +0100 (Sun, 24 Nov 2013) $
  *
  */
 
@@ -34,45 +34,47 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <stdio.h>
 
 #include "nzbget.h"
 #include "NewsServer.h"
 
-NewsServer::NewsServer(int iID, const char* szHost, int iPort, const char* szUser, const char* szPass, bool bJoinGroup,
-	bool bTLS, const char* szCipher, int iMaxConnections, int iLevel, int iGroup)
+NewsServer::NewsServer(int iID, bool bActive, const char* szName, const char* szHost, int iPort,
+	const char* szUser, const char* szPass, bool bJoinGroup, bool bTLS,
+	const char* szCipher, int iMaxConnections, int iLevel, int iGroup)
 {
 	m_iID = iID;
-	m_szHost = NULL;
+	m_iStateID = 0;
+	m_bActive = bActive;
 	m_iPort = iPort;
-	m_szUser = NULL;
-	m_szPassword = NULL;
 	m_iLevel = iLevel;
+	m_iNormLevel = iLevel;
 	m_iGroup = iGroup;
 	m_iMaxConnections = iMaxConnections;
 	m_bJoinGroup = bJoinGroup;
 	m_bTLS = bTLS;
-	m_szHost = szHost ? strdup(szHost) : NULL;
-	m_szUser = szUser ? strdup(szUser) : NULL;
-	m_szPassword = szPass ? strdup(szPass) : NULL;
-	m_szCipher = szCipher ? strdup(szCipher) : NULL;
+	m_szHost = strdup(szHost ? szHost : "");
+	m_szUser = strdup(szUser ? szUser : "");
+	m_szPassword = strdup(szPass ? szPass : "");
+	m_szCipher = strdup(szCipher ? szCipher : "");
+
+	if (szName && strlen(szName) > 0)
+	{
+		m_szName = strdup(szName);
+	}
+	else
+	{
+		m_szName = (char*)malloc(20);
+		snprintf(m_szName, 20, "server%i", iID);
+		m_szName[20-1] = '\0';
+	}
 }
 
 NewsServer::~NewsServer()
 {
-	if (m_szHost)
-	{
-		free(m_szHost);
-	}
-	if (m_szUser)
-	{
-		free(m_szUser);
-	}
-	if (m_szPassword)
-	{
-		free(m_szPassword);
-	}
-	if (m_szCipher)
-	{
-		free(m_szCipher);
-	}
+	free(m_szName);
+	free(m_szHost);
+	free(m_szUser);
+	free(m_szPassword);
+	free(m_szCipher);
 }
