@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 713 $
- * $Date: 2013-06-18 23:56:23 +0200 (Tue, 18 Jun 2013) $
+ * $Revision: 911 $
+ * $Date: 2013-11-24 20:29:52 +0100 (Sun, 24 Nov 2013) $
  *
  */
 
@@ -163,6 +163,7 @@ ParChecker::ParChecker()
 	m_szInfoName = NULL;
 	m_szErrMsg = NULL;
 	m_szProgressLabel = (char*)malloc(1024);
+	m_pRepairer = NULL;
 	m_iFileProgress = 0;
 	m_iStageProgress = 0;
 	m_iExtraFiles = 0;
@@ -175,22 +176,9 @@ ParChecker::~ParChecker()
 {
     debug("Destroying ParChecker");
 
-	if (m_szDestDir)
-	{
-		free(m_szDestDir);
-	}
-	if (m_szNZBName)
-	{
-		free(m_szNZBName);
-	}
-	if (m_szInfoName)
-	{
-		free(m_szInfoName);
-	}
-	if (m_szErrMsg)
-	{
-		free(m_szErrMsg);
-	}
+	free(m_szDestDir);
+	free(m_szNZBName);
+	free(m_szInfoName);
 	free(m_szProgressLabel);
 
 	Cleanup();
@@ -198,6 +186,9 @@ ParChecker::~ParChecker()
 
 void ParChecker::Cleanup()
 {
+	delete (Repairer*)m_pRepairer;
+	m_pRepairer = NULL;
+
 	for (FileList::iterator it = m_QueuedParFiles.begin(); it != m_QueuedParFiles.end() ;it++)
 	{
 		free(*it);
@@ -211,32 +202,26 @@ void ParChecker::Cleanup()
 	m_ProcessedFiles.clear();
 
 	m_sourceFiles.clear();
+
+	free(m_szErrMsg);
+	m_szErrMsg = NULL;
 }
 
 void ParChecker::SetDestDir(const char * szDestDir)
 {
-	if (m_szDestDir)
-	{
-		free(m_szDestDir);
-	}
+	free(m_szDestDir);
 	m_szDestDir = strdup(szDestDir);
 }
 
 void ParChecker::SetNZBName(const char * szNZBName)
 {
-	if (m_szNZBName)
-	{
-		free(m_szNZBName);
-	}
+	free(m_szNZBName);
 	m_szNZBName = strdup(szNZBName);
 }
 
 void ParChecker::SetInfoName(const char * szInfoName)
 {
-	if (m_szInfoName)
-	{
-		free(m_szInfoName);
-	}
+	free(m_szInfoName);
 	m_szInfoName = strdup(szInfoName);
 }
 
@@ -298,49 +283,6 @@ void ParChecker::Run()
 	Completed();
 }
 
-void ParChecker::WriteBrokenLog(EStatus eStatus)
-{
-	char szBrokenLogName[1024];
-	snprintf(szBrokenLogName, 1024, "%s%c_brokenlog.txt", m_szDestDir, (int)PATH_SEPARATOR);
-	szBrokenLogName[1024-1] = '\0';
-	
-	if (eStatus != psRepairNotNeeded || Util::FileExists(szBrokenLogName))
-	{
-		FILE* file = fopen(szBrokenLogName, "ab");
-		if (file)
-		{
-			if (eStatus == psFailed)
-			{
-				if (m_bCancelled)
-				{
-					fprintf(file, "Repair cancelled for %s\n", m_szInfoName);
-				}
-				else
-				{
-					fprintf(file, "Repair failed for %s: %s\n", m_szInfoName, m_szErrMsg ? m_szErrMsg : "");
-				}
-			}
-			else if (eStatus == psRepairPossible)
-			{
-				fprintf(file, "Repair possible for %s\n", m_szInfoName);
-			}
-			else if (eStatus == psRepaired)
-			{
-				fprintf(file, "Successfully repaired %s\n", m_szInfoName);
-			}
-			else if (eStatus == psRepairNotNeeded)
-			{
-				fprintf(file, "Repair not needed for %s\n", m_szInfoName);
-			}
-			fclose(file);
-		}
-		else
-		{
-			PrintMessage(Message::mkError, "Could not open file %s", szBrokenLogName);
-		}
-	}
-}
-
 ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
 {
 	Cleanup();
@@ -355,51 +297,22 @@ ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
 
     debug("par: %s", m_szParFilename);
 	
-    Result res;
-
-	Repairer* pRepairer = new Repairer();
-	m_pRepairer = pRepairer;
-
-	pRepairer->sig_filename.connect(sigc::mem_fun(*this, &ParChecker::signal_filename));
-	pRepairer->sig_progress.connect(sigc::mem_fun(*this, &ParChecker::signal_progress));
-	pRepairer->sig_done.connect(sigc::mem_fun(*this, &ParChecker::signal_done));
-
 	snprintf(m_szProgressLabel, 1024, "Verifying %s", m_szInfoName);
 	m_szProgressLabel[1024-1] = '\0';
 	m_iFileProgress = 0;
 	m_iStageProgress = 0;
 	UpdateProgress();
 
-    res = pRepairer->PreProcess(m_szParFilename);
-    debug("ParChecker: PreProcess-result=%i", res);
-
-	if (res != eSuccess || IsStopped())
+    Result res = (Result)PreProcessPar();
+	if (IsStopped() || res != eSuccess)
 	{
-		if (res == eInvalidCommandLineArguments)
-		{
-			PrintMessage(Message::mkError, "Could not start par-check for %s. Par-file: %s", m_szInfoName, m_szParFilename);
-			m_szErrMsg = strdup("Command line could not be parsed");
-		}
-		else
-		{
-			PrintMessage(Message::mkError, "Could not verify %s: %s", m_szInfoName, IsStopped() ? "due stopping" : "par2-file could not be processed");
-			m_szErrMsg = strdup("par2-file could not be processed");
-		}
-		delete pRepairer;
 		Cleanup();
 		return psFailed;
 	}
-
-	char BufReason[1024];
-	BufReason[0] = '\0';
-	if (m_szErrMsg)
-	{
-		free(m_szErrMsg);
-	}
-	m_szErrMsg = NULL;
 	
 	m_eStage = ptVerifyingSources;
-    res = pRepairer->Process(false);
+	Repairer* pRepairer = (Repairer*)m_pRepairer;
+	res = pRepairer->Process(false);
     debug("ParChecker: Process-result=%i", res);
 
 	if (!IsStopped() && pRepairer->missingfilecount > 0 && g_pOptions->GetParScan() == Options::psAuto && AddMissingFiles())
@@ -415,75 +328,13 @@ ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
 		debug("ParChecker: Process-result=%i", res);
 	}
 
-	bool bMoreFilesLoaded = true;
-	while (!IsStopped() && res == eRepairNotPossible)
+	if (!IsStopped() && res == eRepairNotPossible)
 	{
-		int missingblockcount = pRepairer->missingblockcount - pRepairer->recoverypacketmap.size();
-		if (bMoreFilesLoaded)
-		{
-			PrintMessage(Message::mkInfo, "Need more %i par-block(s) for %s", missingblockcount, m_szInfoName);
-		}
-		
-		m_mutexQueuedParFiles.Lock();
-        bool hasMorePars = !m_QueuedParFiles.empty();
-		m_mutexQueuedParFiles.Unlock();
-		
-		if (!hasMorePars)
-		{
-			int iBlockFound = 0;
-			bool requested = RequestMorePars(missingblockcount, &iBlockFound);
-			if (requested)
-			{
-				strncpy(m_szProgressLabel, "Awaiting additional par-files", 1024);
-				m_szProgressLabel[1024-1] = '\0';
-				m_iFileProgress = 0;
-				UpdateProgress();
-			}
-			
-			m_mutexQueuedParFiles.Lock();
-			hasMorePars = !m_QueuedParFiles.empty();
-			m_bQueuedParFilesChanged = false;
-			m_mutexQueuedParFiles.Unlock();
-			
-			if (!requested && !hasMorePars)
-			{
-				snprintf(BufReason, 1024, "not enough par-blocks, %i block(s) needed, but %i block(s) available", missingblockcount, iBlockFound);
-                BufReason[1024-1] = '\0';
-				m_szErrMsg = strdup(BufReason);
-				break;
-			}
-			
-			if (!hasMorePars)
-			{
-				// wait until new files are added by "AddParFile" or a change is signaled by "QueueChanged"
-				bool bQueuedParFilesChanged = false;
-				while (!bQueuedParFilesChanged && !IsStopped())
-				{
-					m_mutexQueuedParFiles.Lock();
-					bQueuedParFilesChanged = m_bQueuedParFilesChanged;
-					m_mutexQueuedParFiles.Unlock();
-					usleep(100 * 1000);
-				}
-			}
-		}
-
-		if (IsStopped())
-		{
-			break;
-		}
-
-		bMoreFilesLoaded = LoadMorePars();
-		if (bMoreFilesLoaded)
-		{
-			pRepairer->UpdateVerificationResults();
-			res = pRepairer->Process(false);
-			debug("ParChecker: Process-result=%i", res);
-		}
+		res = (Result)ProcessMorePars();
 	}
 
 	if (IsStopped())
 	{
-		delete pRepairer;
 		Cleanup();
 		return psFailed;
 	}
@@ -551,11 +402,192 @@ ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
 		PrintMessage(Message::mkError, "Repair failed for %s: %s", m_szInfoName, m_szErrMsg ? m_szErrMsg : "");
 	}
 	
-	delete pRepairer;
 	Cleanup();
 	return eStatus;
 }
 
+int ParChecker::PreProcessPar()
+{
+	Result res = eRepairFailed;
+	while (!IsStopped() && res != eSuccess)
+	{
+		Cleanup();
+
+		Repairer* pRepairer = new Repairer();
+		m_pRepairer = pRepairer;
+		pRepairer->sig_filename.connect(sigc::mem_fun(*this, &ParChecker::signal_filename));
+		pRepairer->sig_progress.connect(sigc::mem_fun(*this, &ParChecker::signal_progress));
+		pRepairer->sig_done.connect(sigc::mem_fun(*this, &ParChecker::signal_done));
+
+		res = pRepairer->PreProcess(m_szParFilename);
+		debug("ParChecker: PreProcess-result=%i", res);
+
+		if (IsStopped())
+		{
+			PrintMessage(Message::mkError, "Could not verify %s: stopping", m_szInfoName);
+			m_szErrMsg = strdup("par-check was stopped");
+			return eRepairFailed;
+		}
+
+		if (res == eInvalidCommandLineArguments)
+		{
+			PrintMessage(Message::mkError, "Could not start par-check for %s. Par-file: %s", m_szInfoName, m_szParFilename);
+			m_szErrMsg = strdup("Command line could not be parsed");
+			return res;
+		}
+
+		if (res != eSuccess)
+		{
+			PrintMessage(Message::mkWarning, "Could not verify %s: par2-file could not be processed", m_szInfoName);
+			PrintMessage(Message::mkInfo, "Requesting more par2-files for %s", m_szInfoName);
+			bool bHasMorePars = LoadMainParBak();
+			if (!bHasMorePars)
+			{
+				PrintMessage(Message::mkWarning, "No more par2-files found");
+				break;
+			}
+		}
+	}
+
+	if (res != eSuccess)
+	{
+		PrintMessage(Message::mkError, "Could not verify %s: par2-file could not be processed", m_szInfoName);
+		m_szErrMsg = strdup("par2-file could not be processed");
+		return res;
+	}
+
+	return res;
+}
+
+bool ParChecker::LoadMainParBak()
+{
+	while (!IsStopped())
+	{
+		m_mutexQueuedParFiles.Lock();
+        bool hasMorePars = !m_QueuedParFiles.empty();
+		for (FileList::iterator it = m_QueuedParFiles.begin(); it != m_QueuedParFiles.end() ;it++)
+		{
+			free(*it);
+		}
+		m_QueuedParFiles.clear();
+		m_mutexQueuedParFiles.Unlock();
+
+		if (hasMorePars)
+		{
+			return true;
+		}
+
+		int iBlockFound = 0;
+		bool requested = RequestMorePars(1, &iBlockFound);
+		if (requested)
+		{
+			strncpy(m_szProgressLabel, "Awaiting additional par-files", 1024);
+			m_szProgressLabel[1024-1] = '\0';
+			m_iFileProgress = 0;
+			UpdateProgress();
+		}
+
+		m_mutexQueuedParFiles.Lock();
+		hasMorePars = !m_QueuedParFiles.empty();
+		m_bQueuedParFilesChanged = false;
+		m_mutexQueuedParFiles.Unlock();
+
+		if (!requested && !hasMorePars)
+		{
+			return false;
+		}
+
+		if (!hasMorePars)
+		{
+			// wait until new files are added by "AddParFile" or a change is signaled by "QueueChanged"
+			bool bQueuedParFilesChanged = false;
+			while (!bQueuedParFilesChanged && !IsStopped())
+			{
+				m_mutexQueuedParFiles.Lock();
+				bQueuedParFilesChanged = m_bQueuedParFilesChanged;
+				m_mutexQueuedParFiles.Unlock();
+				usleep(100 * 1000);
+			}
+		}
+	}
+
+	return false;
+}
+
+int ParChecker::ProcessMorePars()
+{
+	Result res = eRepairNotPossible;
+	Repairer* pRepairer = (Repairer*)m_pRepairer;
+
+	bool bMoreFilesLoaded = true;
+	while (!IsStopped() && res == eRepairNotPossible)
+	{
+		int missingblockcount = pRepairer->missingblockcount - pRepairer->recoverypacketmap.size();
+		if (bMoreFilesLoaded)
+		{
+			PrintMessage(Message::mkInfo, "Need more %i par-block(s) for %s", missingblockcount, m_szInfoName);
+		}
+		
+		m_mutexQueuedParFiles.Lock();
+        bool hasMorePars = !m_QueuedParFiles.empty();
+		m_mutexQueuedParFiles.Unlock();
+		
+		if (!hasMorePars)
+		{
+			int iBlockFound = 0;
+			bool requested = RequestMorePars(missingblockcount, &iBlockFound);
+			if (requested)
+			{
+				strncpy(m_szProgressLabel, "Awaiting additional par-files", 1024);
+				m_szProgressLabel[1024-1] = '\0';
+				m_iFileProgress = 0;
+				UpdateProgress();
+			}
+			
+			m_mutexQueuedParFiles.Lock();
+			hasMorePars = !m_QueuedParFiles.empty();
+			m_bQueuedParFilesChanged = false;
+			m_mutexQueuedParFiles.Unlock();
+			
+			if (!requested && !hasMorePars)
+			{
+				m_szErrMsg = (char*)malloc(1024);
+				snprintf(m_szErrMsg, 1024, "not enough par-blocks, %i block(s) needed, but %i block(s) available", missingblockcount, iBlockFound);
+                m_szErrMsg[1024-1] = '\0';
+				break;
+			}
+			
+			if (!hasMorePars)
+			{
+				// wait until new files are added by "AddParFile" or a change is signaled by "QueueChanged"
+				bool bQueuedParFilesChanged = false;
+				while (!bQueuedParFilesChanged && !IsStopped())
+				{
+					m_mutexQueuedParFiles.Lock();
+					bQueuedParFilesChanged = m_bQueuedParFilesChanged;
+					m_mutexQueuedParFiles.Unlock();
+					usleep(100 * 1000);
+				}
+			}
+		}
+
+		if (IsStopped())
+		{
+			break;
+		}
+
+		bMoreFilesLoaded = LoadMorePars();
+		if (bMoreFilesLoaded)
+		{
+			pRepairer->UpdateVerificationResults();
+			res = pRepairer->Process(false);
+			debug("ParChecker: Process-result=%i", res);
+		}
+	}
+
+	return res;
+}
+
 bool ParChecker::LoadMorePars()
 {
 	m_mutexQueuedParFiles.Lock();
@@ -867,6 +899,49 @@ void ParChecker::Cancel()
 #endif
 }
 
+void ParChecker::WriteBrokenLog(EStatus eStatus)
+{
+	char szBrokenLogName[1024];
+	snprintf(szBrokenLogName, 1024, "%s%c_brokenlog.txt", m_szDestDir, (int)PATH_SEPARATOR);
+	szBrokenLogName[1024-1] = '\0';
+	
+	if (eStatus != psRepairNotNeeded || Util::FileExists(szBrokenLogName))
+	{
+		FILE* file = fopen(szBrokenLogName, "ab");
+		if (file)
+		{
+			if (eStatus == psFailed)
+			{
+				if (m_bCancelled)
+				{
+					fprintf(file, "Repair cancelled for %s\n", m_szInfoName);
+				}
+				else
+				{
+					fprintf(file, "Repair failed for %s: %s\n", m_szInfoName, m_szErrMsg ? m_szErrMsg : "");
+				}
+			}
+			else if (eStatus == psRepairPossible)
+			{
+				fprintf(file, "Repair possible for %s\n", m_szInfoName);
+			}
+			else if (eStatus == psRepaired)
+			{
+				fprintf(file, "Successfully repaired %s\n", m_szInfoName);
+			}
+			else if (eStatus == psRepairNotNeeded)
+			{
+				fprintf(file, "Repair not needed for %s\n", m_szInfoName);
+			}
+			fclose(file);
+		}
+		else
+		{
+			PrintMessage(Message::mkError, "Could not open file %s", szBrokenLogName);
+		}
+	}
+}
+
 void ParChecker::SaveSourceList()
 {
 	// Buliding a list of DiskFile-objects, marked as source-files
