@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 680 $
- * $Date: 2013-05-15 18:45:36 +0200 (Wed, 15 May 2013) $
+ * $Revision: 919 $
+ * $Date: 2013-12-13 00:43:38 +0100 (Fri, 13 Dec 2013) $
  *
  */
 
@@ -35,6 +35,7 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <ctype.h>
 #ifdef WIN32
 #include <direct.h>
 #else
@@ -215,8 +216,13 @@ bool ParCoordinator::ParseParFilename(const char* szParFilename, int* iBaseNameL
 	char* szEnd = szFilename;
 	while (char* p = strstr(szEnd, ".par2")) szEnd = p + 5;
 	*szEnd = '\0';
+
 	iLen = strlen(szFilename);
-	
+	if (iLen < 6)
+	{
+		return false;
+	}
+
 	if (strcasecmp(szFilename + iLen - 5, ".par2"))
 	{
 		return false;
@@ -272,9 +278,19 @@ void ParCoordinator::StartParCheckJob(PostInfo* pPostInfo)
  */
 void ParCoordinator::StartParRenameJob(PostInfo* pPostInfo)
 {
+	const char* szDestDir = pPostInfo->GetNZBInfo()->GetDestDir();
+
+	char szFinalDir[1024];
+	if (pPostInfo->GetNZBInfo()->GetUnpackStatus() == NZBInfo::usSuccess)
+	{
+		pPostInfo->GetNZBInfo()->BuildFinalDirName(szFinalDir, 1024);
+		szFinalDir[1024-1] = '\0';
+		szDestDir = szFinalDir;
+	}
+
 	m_eCurrentJob = jkParRename;
 	m_ParRenamer.SetPostInfo(pPostInfo);
-	m_ParRenamer.SetDestDir(pPostInfo->GetNZBInfo()->GetDestDir());
+	m_ParRenamer.SetDestDir(szDestDir);
 	m_ParRenamer.SetInfoName(pPostInfo->GetNZBInfo()->GetName());
 	m_ParRenamer.PrintMessage(Message::mkInfo, "Checking renamed files for %s", pPostInfo->GetNZBInfo()->GetName());
 	pPostInfo->SetWorking(true);
@@ -390,7 +406,7 @@ bool ParCoordinator::RequestMorePars(NZBInfo* pNZBInfo, const char* szParFilenam
 		FindPars(pDownloadQueue, pNZBInfo, szParFilename, &blocks, true, false, &iCurBlockFound);
         iBlockFound += iCurBlockFound;
 	}
-	if (iBlockFound < iBlockNeeded && !g_pOptions->GetStrictParName())
+	if (iBlockFound < iBlockNeeded)
 	{
 		FindPars(pDownloadQueue, pNZBInfo, szParFilename, &blocks, false, false, &iCurBlockFound);
         iBlockFound += iCurBlockFound;
