@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 711 $
- * $Date: 2013-06-17 22:39:46 +0200 (Mon, 17 Jun 2013) $
+ * $Revision: 924 $
+ * $Date: 2013-12-21 22:39:49 +0100 (Sat, 21 Dec 2013) $
  *
  */
 
@@ -39,6 +39,8 @@
 #else
 #include <unistd.h>
 #endif
+#include <set>
+#include <algorithm>
 
 #include "nzbget.h"
 #include "PrePostProcessor.h"
@@ -59,6 +61,13 @@ extern DiskState* g_pDiskState;
 extern Scheduler* g_pScheduler;
 extern Scanner* g_pScanner;
 
+void PrePostProcessor::PostDupeCoordinator::HistoryRedownload(DownloadQueue* pDownloadQueue, HistoryInfo* pHistoryInfo)
+{
+	HistoryList::iterator it = std::find(pDownloadQueue->GetHistoryList()->begin(),
+		pDownloadQueue->GetHistoryList()->end(), pHistoryInfo);
+	m_pOwner->HistoryRedownload(pDownloadQueue, it, pHistoryInfo, true);
+}
+
 PrePostProcessor::PrePostProcessor()
 {
 	debug("Creating PrePostProcessor");
@@ -68,10 +77,10 @@ PrePostProcessor::PrePostProcessor()
 
 	m_QueueCoordinatorObserver.m_pOwner = this;
 	g_pQueueCoordinator->Attach(&m_QueueCoordinatorObserver);
-
 #ifndef DISABLE_PARCHECK
 	m_ParCoordinator.m_pOwner = this;
 #endif
+	m_DupeCoordinator.m_pOwner = this;
 }
 
 PrePostProcessor::~PrePostProcessor()
@@ -123,7 +132,7 @@ void PrePostProcessor::Run()
 
 	int iDiskSpaceInterval = 1000;
 	int iSchedulerInterval = 1000;
-	int iHistoryInterval = 60000;
+	int iHistoryInterval = 600000;
 	const int iStepMSec = 200;
 
 	while (!IsStopped())
@@ -154,9 +163,9 @@ void PrePostProcessor::Run()
 		}
 		iSchedulerInterval += iStepMSec;
 
-		if (iHistoryInterval >= 60000)
+		if (iHistoryInterval >= 600000)
 		{
-			// check history (remove old entries) every 1 minute
+			// check history (remove old entries) every 10 minutes
 			CheckHistory();
 			iHistoryInterval = 0;
 		}
@@ -204,7 +213,11 @@ void PrePostProcessor::QueueCoordinatorUpdate(Subject * Caller, void * Aspect)
 	}
 
 	QueueCoordinator::Aspect* pAspect = (QueueCoordinator::Aspect*)Aspect;
-	if (pAspect->eAction == QueueCoordinator::eaNZBFileAdded)
+	if (pAspect->eAction == QueueCoordinator::eaNZBFileFound)
+	{
+		NZBFound(pAspect->pDownloadQueue, pAspect->pNZBInfo);
+	}
+	else if (pAspect->eAction == QueueCoordinator::eaNZBFileAdded)
 	{
 		NZBAdded(pAspect->pDownloadQueue, pAspect->pNZBInfo);
 	}
@@ -218,15 +231,18 @@ void PrePostProcessor::QueueCoordinatorUpdate(Subject * Caller, void * Aspect)
 			IsNZBFileCompleted(pAspect->pDownloadQueue, pAspect->pNZBInfo, true, false) &&
 			(!pAspect->pFileInfo->GetPaused() || IsNZBFileCompleted(pAspect->pDownloadQueue, pAspect->pNZBInfo, false, false)))
 		{
-			if (pAspect->eAction == QueueCoordinator::eaFileCompleted ||
+			if ((pAspect->eAction == QueueCoordinator::eaFileCompleted ||
 				(pAspect->pFileInfo->GetAutoDeleted() &&
-				 IsNZBFileCompleted(pAspect->pDownloadQueue, pAspect->pNZBInfo, false, true)))
+				 IsNZBFileCompleted(pAspect->pDownloadQueue, pAspect->pNZBInfo, false, true))) &&
+				 pAspect->pFileInfo->GetNZBInfo()->GetDeleteStatus() != NZBInfo::dsHealth)
 			{
 				info("Collection %s completely downloaded", pAspect->pNZBInfo->GetName());
 				NZBDownloaded(pAspect->pDownloadQueue, pAspect->pNZBInfo);
 			}
-			else if (pAspect->eAction == QueueCoordinator::eaFileDeleted &&
-				!pAspect->pNZBInfo->GetParCleanup() &&
+			else if ((pAspect->eAction == QueueCoordinator::eaFileDeleted ||
+				(pAspect->eAction == QueueCoordinator::eaFileCompleted &&
+				 pAspect->pFileInfo->GetNZBInfo()->GetDeleteStatus() > NZBInfo::dsNone)) &&
+				!pAspect->pNZBInfo->GetParCleanup() && !pAspect->pNZBInfo->GetPostProcess() &&
 				IsNZBFileCompleted(pAspect->pDownloadQueue, pAspect->pNZBInfo, false, true))
 			{
 				info("Collection %s deleted from queue", pAspect->pNZBInfo->GetName());
@@ -236,20 +252,27 @@ void PrePostProcessor::QueueCoordinatorUpdate(Subject * Caller, void * Aspect)
 	}
 }
 
+void PrePostProcessor::NZBFound(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
+{
+	if (g_pOptions->GetDupeCheck() && pNZBInfo->GetDupeMode() != dmForce)
+	{
+		m_DupeCoordinator.NZBFound(pDownloadQueue, pNZBInfo);
+	}
+}
+
 void PrePostProcessor::NZBAdded(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
 {
-	PostScriptController::InitParamsForNewNZB(pNZBInfo);
-
-	if (g_pOptions->GetMergeNzb())
-	{
-		pNZBInfo = MergeGroups(pDownloadQueue, pNZBInfo);
-	}
-
 	if (g_pOptions->GetParCheck() != Options::pcForce)
 	{
 		m_ParCoordinator.PausePars(pDownloadQueue, pNZBInfo);
 	}
 
+	if (g_pOptions->GetDupeCheck() && pNZBInfo->GetDupeMode() != dmForce &&
+		pNZBInfo->GetDeleteStatus() == NZBInfo::dsDupe)
+	{
+		NZBCompleted(pDownloadQueue, pNZBInfo, false);
+	}
+
 	if (strlen(g_pOptions->GetNZBAddedProcess()) > 0)
 	{
 		NZBAddedScriptController::StartScript(pDownloadQueue, pNZBInfo, g_pOptions->GetNZBAddedProcess());
@@ -271,11 +294,19 @@ void PrePostProcessor::NZBDownloaded(DownloadQueue* pDownloadQueue, NZBInfo* pNZ
 			pNZBInfo->SetParStatus(NZBInfo::psSkipped);
 		}
 
-		if (pNZBInfo->GetRenameStatus() == NZBInfo::rsNone)
+		if (pNZBInfo->GetRenameStatus() == NZBInfo::rsNone && !g_pOptions->GetParRename())
 		{
 			pNZBInfo->SetRenameStatus(NZBInfo::rsSkipped);
 		}
 
+		if (pNZBInfo->GetDeleteStatus() != NZBInfo::dsNone)
+		{
+			pNZBInfo->SetParStatus(NZBInfo::psFailure);
+			pNZBInfo->SetUnpackStatus(NZBInfo::usFailure);
+			pNZBInfo->SetRenameStatus(NZBInfo::rsFailure);
+			pNZBInfo->SetMoveStatus(NZBInfo::msFailure);
+		}
+
 		pNZBInfo->SetPostProcess(true);
 		pDownloadQueue->GetPostQueue()->push_back(pPostInfo);
 		SaveQueue(pDownloadQueue);
@@ -289,7 +320,14 @@ void PrePostProcessor::NZBDownloaded(DownloadQueue* pDownloadQueue, NZBInfo* pNZ
 
 void PrePostProcessor::NZBDeleted(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
 {
-	if (g_pOptions->GetDeleteCleanupDisk() && pNZBInfo->GetCleanupDisk())
+	if (pNZBInfo->GetDeleteStatus() == NZBInfo::dsNone)
+	{
+		pNZBInfo->SetDeleteStatus(NZBInfo::dsManual);
+	}
+	pNZBInfo->SetDeleting(false);
+
+	if ((g_pOptions->GetDeleteCleanupDisk() && pNZBInfo->GetCleanupDisk()) ||
+		pNZBInfo->GetDeleteStatus() == NZBInfo::dsDupe)
 	{
 		// download was cancelled, deleting already downloaded files from disk
 		for (NZBInfo::Files::reverse_iterator it = pNZBInfo->GetCompletedFiles()->rbegin(); it != pNZBInfo->GetCompletedFiles()->rend(); it++)
@@ -323,19 +361,23 @@ void PrePostProcessor::NZBDeleted(DownloadQueue* pDownloadQueue, NZBInfo* pNZBIn
 		{
 			rmdir(pNZBInfo->GetDestDir());
 		}
-
-		if (g_pOptions->GetNzbCleanupDisk())
-		{
-			DeleteQueuedFile(pNZBInfo->GetQueuedFilename());
-		}
 	}
 
-	NZBCompleted(pDownloadQueue, pNZBInfo, true);
+	if (pNZBInfo->GetDeleteStatus() == NZBInfo::dsHealth)
+	{
+		NZBDownloaded(pDownloadQueue, pNZBInfo);
+	}
+	else
+	{
+		NZBCompleted(pDownloadQueue, pNZBInfo, true);
+	}
 }
 
 void PrePostProcessor::NZBCompleted(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo, bool bSaveQueue)
 {
-	if (g_pOptions->GetKeepHistory() > 0)
+	bool bNeedSave = false;
+
+	if (g_pOptions->GetKeepHistory() > 0 && !pNZBInfo->GetAvoidHistory())
 	{
 		//remove old item for the same NZB
 		for (HistoryList::iterator it = pDownloadQueue->GetHistoryList()->begin(); it != pDownloadQueue->GetHistoryList()->end(); it++)
@@ -376,23 +418,34 @@ void PrePostProcessor::NZBCompleted(DownloadQueue* pDownloadQueue, NZBInfo* pNZB
 		}
 		pNZBInfo->SetParkedFileCount(iParkedFiles);
 
-		if (bSaveQueue)
-		{
-			SaveQueue(pDownloadQueue);
-		}
-
 		info("Collection %s added to history", pNZBInfo->GetName());
+		bNeedSave = true;
+	}
+
+	pNZBInfo->SetAvoidHistory(false);
+
+	if (g_pOptions->GetDupeCheck() && pNZBInfo->GetDupeMode() != dmForce &&
+		(pNZBInfo->GetDeleteStatus() == NZBInfo::dsNone ||
+		 pNZBInfo->GetDeleteStatus() == NZBInfo::dsHealth))
+	{
+		m_DupeCoordinator.NZBCompleted(pDownloadQueue, pNZBInfo);
+		bNeedSave = true;
+	}
+
+	if (bSaveQueue && bNeedSave)
+	{
+		SaveQueue(pDownloadQueue);
 	}
 }
 
 /**
- * Removes old entries from history
+ * Removes old entries from (recent) history
  */
 void PrePostProcessor::CheckHistory()
 {
 	DownloadQueue* pDownloadQueue = g_pQueueCoordinator->LockQueue();
 
-	time_t tMinTime = time(NULL) - g_pOptions->GetKeepHistory() * 60000;
+	time_t tMinTime = time(NULL) - g_pOptions->GetKeepHistory() * 60*60*24;
 	bool bChanged = false;
 	int index = 0;
 
@@ -401,13 +454,29 @@ void PrePostProcessor::CheckHistory()
 	for (HistoryList::reverse_iterator it = pDownloadQueue->GetHistoryList()->rbegin(); it != pDownloadQueue->GetHistoryList()->rend(); )
 	{
 		HistoryInfo* pHistoryInfo = *it;
-		if (pHistoryInfo->GetTime() < tMinTime)
+		if (pHistoryInfo->GetKind() != HistoryInfo::hkDupInfo && pHistoryInfo->GetTime() < tMinTime)
 		{
-			char szNiceName[1024];
-			pHistoryInfo->GetName(szNiceName, 1024);
-			pDownloadQueue->GetHistoryList()->erase(pDownloadQueue->GetHistoryList()->end() - 1 - index);
-			delete pHistoryInfo;
-			info("Collection %s removed from history", szNiceName);
+			if (g_pOptions->GetDupeCheck() && pHistoryInfo->GetKind() == HistoryInfo::hkNZBInfo)
+			{
+				// replace history element
+				m_DupeCoordinator.HistoryTransformToDup(pDownloadQueue, pHistoryInfo, index);
+				index++;
+			}
+			else
+			{
+				char szNiceName[1024];
+				pHistoryInfo->GetName(szNiceName, 1024);
+
+				pDownloadQueue->GetHistoryList()->erase(pDownloadQueue->GetHistoryList()->end() - 1 - index);
+				delete pHistoryInfo;
+				
+				if (pHistoryInfo->GetKind() == HistoryInfo::hkNZBInfo)
+				{
+					DeleteQueuedFile(pHistoryInfo->GetNZBInfo()->GetQueuedFilename());
+				}
+				info("Collection %s removed from history", szNiceName);
+			}
+
 			it = pDownloadQueue->GetHistoryList()->rbegin() + index;
 			bChanged = true;
 		}
@@ -428,7 +497,12 @@ void PrePostProcessor::CheckHistory()
 
 void PrePostProcessor::DeleteQueuedFile(const char* szQueuedFile)
 {
-	// szQueuedFile may contain one filename or several filenames separated 
+	if (!g_pOptions->GetNzbCleanupDisk())
+	{
+		return;
+	}
+
+	// szQueuedFile may contain one filename or several filenames separated
 	// with "|"-character (for merged groups)
 	char* szFilename = strdup(szQueuedFile);
 	char* szEnd = szFilename - 1;
@@ -449,44 +523,21 @@ void PrePostProcessor::DeleteQueuedFile(const char* szQueuedFile)
 	free(szFilename);
 }
 
-NZBInfo* PrePostProcessor::MergeGroups(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
+/**
+ * Find ID of any file in the nzb-file
+ */
+int PrePostProcessor::FindGroupID(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
 {
-	int iAddedGroupID = 0;
-
-	// merge(1): find ID of any file in new nzb-file
 	for (FileQueue::iterator it = pDownloadQueue->GetFileQueue()->begin(); it != pDownloadQueue->GetFileQueue()->end(); it++)
 	{
 		FileInfo* pFileInfo = *it;
 		if (pFileInfo->GetNZBInfo() == pNZBInfo)
 		{
-			iAddedGroupID = pFileInfo->GetID();
+			return pFileInfo->GetID();
 			break;
 		}
 	}
-
-	// merge(2): check if queue has other nzb-files with the same name
-	if (iAddedGroupID > 0)
-	{
-		for (FileQueue::iterator it = pDownloadQueue->GetFileQueue()->begin(); it != pDownloadQueue->GetFileQueue()->end(); it++)
-		{
-			FileInfo* pFileInfo = *it;
-			if (pFileInfo->GetNZBInfo() != pNZBInfo &&
-				!strcmp(pFileInfo->GetNZBInfo()->GetName(), pNZBInfo->GetName()))
-			{
-				// file found, do merging
-
-				IDList cIDList;
-				cIDList.push_back(pFileInfo->GetID());
-				cIDList.push_back(iAddedGroupID);
-
-				g_pQueueCoordinator->GetQueueEditor()->LockedEditList(pDownloadQueue, &cIDList, false, QueueEditor::eaGroupMerge, 0, NULL);
-
-				return pFileInfo->GetNZBInfo();
-			}
-		}
-	}
-
-	return pNZBInfo;
+	return 0;
 }
 
 void PrePostProcessor::CheckDiskSpace()
@@ -494,9 +545,19 @@ void PrePostProcessor::CheckDiskSpace()
 	long long lFreeSpace = Util::FreeDiskSize(g_pOptions->GetDestDir());
 	if (lFreeSpace > -1 && lFreeSpace / 1024 / 1024 < g_pOptions->GetDiskSpace())
 	{
-		warn("Low disk space. Pausing download");
+		warn("Low disk space on %s. Pausing download", g_pOptions->GetDestDir());
 		g_pOptions->SetPauseDownload(true);
 	}
+
+	if (!Util::EmptyStr(g_pOptions->GetInterDir()))
+	{
+		lFreeSpace = Util::FreeDiskSize(g_pOptions->GetInterDir());
+		if (lFreeSpace > -1 && lFreeSpace / 1024 / 1024 < g_pOptions->GetDiskSpace())
+		{
+			warn("Low disk space on %s. Pausing download", g_pOptions->GetInterDir());
+			g_pOptions->SetPauseDownload(true);
+		}
+	}
 }
 
 void PrePostProcessor::CheckPostQueue()
@@ -539,14 +600,7 @@ void PrePostProcessor::CheckPostQueue()
 					pPostInfo->SetStage(PostInfo::ptQueued);
 				}
 			}
-			else if (pPostInfo->GetRequestParRename())
-			{
-				pPostInfo->GetNZBInfo()->SetRenameStatus(NZBInfo::rsNone);
-				pPostInfo->SetRequestParRename(false);
-				pPostInfo->SetStage(PostInfo::ptQueued);
-				DeletePostThread(pPostInfo);
-			}
-
+			
 #endif
 			if (pPostInfo->GetDeleted())
 			{
@@ -604,11 +658,8 @@ void PrePostProcessor::SanitisePostQueue(PostQueue* pPostQueue)
 
 void PrePostProcessor::DeletePostThread(PostInfo* pPostInfo)
 {
-	if (pPostInfo->GetPostThread())
-	{
-		delete pPostInfo->GetPostThread();
-		pPostInfo->SetPostThread(NULL);
-	}
+	delete pPostInfo->GetPostThread();
+	pPostInfo->SetPostThread(NULL);
 }
 
 void PrePostProcessor::StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostInfo)
@@ -636,9 +687,29 @@ void PrePostProcessor::StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostIn
 		}
 		return;
 	}
+	else if (pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psSkipped &&
+		pPostInfo->GetNZBInfo()->CalcHealth() < pPostInfo->GetNZBInfo()->CalcCriticalHealth() &&
+		m_ParCoordinator.FindMainPars(pPostInfo->GetNZBInfo()->GetDestDir(), NULL))
+	{
+		warn("Skipping par-check for %s due to health %.1f%% below critical %.1f%%", pPostInfo->GetInfoName(),
+			pPostInfo->GetNZBInfo()->CalcHealth() / 10.0, pPostInfo->GetNZBInfo()->CalcCriticalHealth() / 10.0);
+		pPostInfo->GetNZBInfo()->SetParStatus(NZBInfo::psFailure);
+		return;
+	}
+	else if (pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psSkipped &&
+		pPostInfo->GetNZBInfo()->GetFailedSize() - pPostInfo->GetNZBInfo()->GetParFailedSize() > 0 &&
+		m_ParCoordinator.FindMainPars(pPostInfo->GetNZBInfo()->GetDestDir(), NULL))
+	{
+		info("Collection %s with health %.1f%% needs par-check",
+			pPostInfo->GetInfoName(), pPostInfo->GetNZBInfo()->CalcHealth() / 10.0);
+		pPostInfo->SetRequestParCheck(true);
+		return;
+	}
 #endif
 
-	bool bUnpack = g_pOptions->GetUnpack() && (pPostInfo->GetNZBInfo()->GetUnpackStatus() == NZBInfo::usNone);
+	NZBParameter* pUnpackParameter = pPostInfo->GetNZBInfo()->GetParameters()->Find("*Unpack:", false);
+	bool bUnpackParam = !(pUnpackParameter && !strcasecmp(pUnpackParameter->GetValue(), "no"));
+	bool bUnpack = bUnpackParam && (pPostInfo->GetNZBInfo()->GetUnpackStatus() == NZBInfo::usNone);
 
 	bool bParFailed = pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psFailure ||
 		pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psRepairPossible ||
@@ -654,6 +725,8 @@ void PrePostProcessor::StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostIn
 	bool bMoveInter = !bUnpack &&
 		pPostInfo->GetNZBInfo()->GetMoveStatus() == NZBInfo::msNone &&
 		pPostInfo->GetNZBInfo()->GetUnpackStatus() != NZBInfo::usFailure &&
+		pPostInfo->GetNZBInfo()->GetUnpackStatus() != NZBInfo::usSpace &&
+		pPostInfo->GetNZBInfo()->GetUnpackStatus() != NZBInfo::usPassword &&
 		pPostInfo->GetNZBInfo()->GetParStatus() != NZBInfo::psFailure &&
 		pPostInfo->GetNZBInfo()->GetParStatus() != NZBInfo::psManual &&
 		strlen(g_pOptions->GetInterDir()) > 0 &&
@@ -664,9 +737,8 @@ void PrePostProcessor::StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostIn
 
 	if (bUnpack && bParFailed)
 	{
-		warn("Skipping unpack due to %s for %s",
-			pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psManual ? "required par-repair" : "par-failure",
-			pPostInfo->GetInfoName());
+		warn("Skipping unpack for %s due to %s", pPostInfo->GetInfoName(),
+			pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psManual ? "required par-repair" : "par-failure");
 		pPostInfo->GetNZBInfo()->SetUnpackStatus(NZBInfo::usSkipped);
 		bUnpack = false;
 	}
@@ -729,22 +801,15 @@ void PrePostProcessor::JobCompleted(DownloadQueue* pDownloadQueue, PostInfo* pPo
 			 pPostInfo->GetNZBInfo()->GetUnpackStatus() == NZBInfo::usSuccess ||
 			 (pPostInfo->GetNZBInfo()->GetUnpackStatus() == NZBInfo::usNone &&
 			  pPostInfo->GetNZBInfo()->GetScriptStatuses()->CalcTotalStatus() == ScriptStatus::srSuccess);
-		if ((g_pOptions->GetParCleanupQueue() || g_pOptions->GetNzbCleanupDisk()) && bCanCleanupQueue)
+		if (g_pOptions->GetParCleanupQueue() && bCanCleanupQueue)
 		{
-			if (g_pOptions->GetParCleanupQueue())
+			FileInfo* pFileInfo = GetQueueGroup(pDownloadQueue, pPostInfo->GetNZBInfo());
+			if (pFileInfo)
 			{
-				FileInfo* pFileInfo = GetQueueGroup(pDownloadQueue, pPostInfo->GetNZBInfo());
-				if (pFileInfo)
-				{
-					info("Cleaning up download queue for %s", pPostInfo->GetNZBInfo()->GetName());
-					pFileInfo->GetNZBInfo()->ClearCompletedFiles();
-					pFileInfo->GetNZBInfo()->SetParCleanup(true);
-					g_pQueueCoordinator->GetQueueEditor()->LockedEditEntry(pDownloadQueue, pFileInfo->GetID(), false, QueueEditor::eaGroupDelete, 0, NULL);
-				}
-			}
-			if (g_pOptions->GetNzbCleanupDisk())
-			{
-				DeleteQueuedFile(pPostInfo->GetNZBInfo()->GetQueuedFilename());
+				info("Cleaning up download queue for %s", pPostInfo->GetNZBInfo()->GetName());
+				pFileInfo->GetNZBInfo()->ClearCompletedFiles();
+				pFileInfo->GetNZBInfo()->SetParCleanup(true);
+				g_pQueueCoordinator->GetQueueEditor()->LockedEditEntry(pDownloadQueue, pFileInfo->GetID(), false, QueueEditor::eaGroupDelete, 0, NULL);
 			}
 		}
 
@@ -814,15 +879,9 @@ FileInfo* PrePostProcessor::GetQueueGroup(DownloadQueue* pDownloadQueue, NZBInfo
 
 void PrePostProcessor::ApplySchedulerState()
 {
-	if (g_pScheduler->GetDownloadRateChanged())
-	{
-		info("Scheduler: set download rate to %i KB/s", g_pScheduler->GetDownloadRate() / 1024);
-		g_pOptions->SetDownloadRate(g_pScheduler->GetDownloadRate());
-	}
-
 	if (g_pScheduler->GetPauseDownloadChanged())
 	{
-		info("Scheduler: %s download queue", g_pScheduler->GetPauseDownload() ? "pause" : "unpause");
+		info("Scheduler: %s download queue", g_pScheduler->GetPauseDownload() ? "pausing" : "unpausing");
 		m_bSchedulerPauseChanged = true;
 		m_bSchedulerPause = g_pScheduler->GetPauseDownload();
 		if (!m_bPostPause)
@@ -830,12 +889,6 @@ void PrePostProcessor::ApplySchedulerState()
 			g_pOptions->SetPauseDownload(m_bSchedulerPause);
 		}
 	}
-
-	if (g_pScheduler->GetPauseScanChanged())
-	{
-		info("Scheduler: %s scan", g_pScheduler->GetPauseScan() ? "pause" : "unpause");
-		g_pOptions->SetPauseScan(g_pScheduler->GetPauseScan());
-	}
 }
 
 void PrePostProcessor::UpdatePauseState(bool bNeedPause, const char* szReason)
@@ -915,9 +968,17 @@ bool PrePostProcessor::QueueEditList(IDList* pIDList, EEditAction eAction, int i
 			return PostQueueDelete(pIDList);
 
 		case eaHistoryDelete:
+		case eaHistoryFinalDelete:
 		case eaHistoryReturn:
 		case eaHistoryProcess:
+		case eaHistoryRedownload:
 		case eaHistorySetParameter:
+		case eaHistorySetDupeKey:
+		case eaHistorySetDupeScore:
+		case eaHistorySetDupeMode:
+		case eaHistorySetDupeBackup:
+		case eaHistoryMarkBad:
+		case eaHistoryMarkGood:
 			return HistoryEdit(pIDList, eAction, iOffset, szText);
 
 		default:
@@ -1069,7 +1130,8 @@ bool PrePostProcessor::HistoryEdit(IDList* pIDList, EEditAction eAction, int iOf
 				switch (eAction)
 				{
 					case eaHistoryDelete:
-						HistoryDelete(pDownloadQueue, itHistory, pHistoryInfo);
+					case eaHistoryFinalDelete:
+						HistoryDelete(pDownloadQueue, itHistory, pHistoryInfo, eAction == eaHistoryFinalDelete);
 						break;
 
 					case eaHistoryReturn:
@@ -1077,10 +1139,26 @@ bool PrePostProcessor::HistoryEdit(IDList* pIDList, EEditAction eAction, int iOf
 						HistoryReturn(pDownloadQueue, itHistory, pHistoryInfo, eAction == eaHistoryProcess);
 						break;
 
+					case eaHistoryRedownload:
+						HistoryRedownload(pDownloadQueue, itHistory, pHistoryInfo, false);
+						break;
+
 					case eaHistorySetParameter:
 						HistorySetParameter(pHistoryInfo, szText);
 						break;
-						
+
+					case eaHistorySetDupeKey:
+					case eaHistorySetDupeScore:
+					case eaHistorySetDupeMode:
+					case eaHistorySetDupeBackup:
+						HistorySetDupeParam(pHistoryInfo, eAction, szText);
+						break;
+
+					case eaHistoryMarkBad:
+					case eaHistoryMarkGood:
+						m_DupeCoordinator.HistoryMark(pDownloadQueue, pHistoryInfo, eAction == eaHistoryMarkGood);
+						break;
+
 					default:
 						// nothing, just to avoid compiler warning
 						break;
@@ -1102,7 +1180,8 @@ bool PrePostProcessor::HistoryEdit(IDList* pIDList, EEditAction eAction, int iOf
 	return bOK;
 }
 
-void PrePostProcessor::HistoryDelete(DownloadQueue* pDownloadQueue, HistoryList::iterator itHistory, HistoryInfo* pHistoryInfo)
+void PrePostProcessor::HistoryDelete(DownloadQueue* pDownloadQueue, HistoryList::iterator itHistory,
+	HistoryInfo* pHistoryInfo, bool bFinal)
 {
 	char szNiceName[1024];
 	pHistoryInfo->GetName(szNiceName, 1024);
@@ -1133,10 +1212,36 @@ void PrePostProcessor::HistoryDelete(DownloadQueue* pDownloadQueue, HistoryList:
 				index++;
 			}
 		}
+
+		DeleteQueuedFile(pNZBInfo->GetQueuedFilename());
 	}
 
-	pDownloadQueue->GetHistoryList()->erase(itHistory);
-	delete pHistoryInfo;
+	if (pHistoryInfo->GetKind() == HistoryInfo::hkNZBInfo &&
+		g_pOptions->GetDeleteCleanupDisk() &&
+		(pHistoryInfo->GetNZBInfo()->GetDeleteStatus() != NZBInfo::dsNone ||
+		pHistoryInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psFailure ||
+		pHistoryInfo->GetNZBInfo()->GetUnpackStatus() == NZBInfo::usFailure ||
+		pHistoryInfo->GetNZBInfo()->GetUnpackStatus() == NZBInfo::usPassword) &&
+		Util::DirectoryExists(pHistoryInfo->GetNZBInfo()->GetDestDir()))
+	{
+		info("Deleting %s", pHistoryInfo->GetNZBInfo()->GetDestDir());
+		Util::DeleteDirectoryWithContent(pHistoryInfo->GetNZBInfo()->GetDestDir());
+	}
+
+	if (bFinal || !g_pOptions->GetDupeCheck() || pHistoryInfo->GetKind() == HistoryInfo::hkUrlInfo)
+	{
+		pDownloadQueue->GetHistoryList()->erase(itHistory);
+		delete pHistoryInfo;
+	}
+	else
+	{
+		if (pHistoryInfo->GetKind() == HistoryInfo::hkNZBInfo)
+		{
+			// replace history element
+			int rindex = pDownloadQueue->GetHistoryList()->size() - 1 - (itHistory - pDownloadQueue->GetHistoryList()->begin());
+			m_DupeCoordinator.HistoryTransformToDup(pDownloadQueue, pHistoryInfo, rindex);
+		}
+	}
 }
 
 void PrePostProcessor::HistoryReturn(DownloadQueue* pDownloadQueue, HistoryList::iterator itHistory, HistoryInfo* pHistoryInfo, bool bReprocess)
@@ -1181,11 +1286,18 @@ void PrePostProcessor::HistoryReturn(DownloadQueue* pDownloadQueue, HistoryList:
 		pNZBInfo->SetParCleanup(false);
 		if (!pNZBInfo->GetUnpackCleanedUpDisk())
 		{
-			pNZBInfo->SetParStatus(NZBInfo::psNone);
-			pNZBInfo->SetRenameStatus(NZBInfo::rsNone);
 			pNZBInfo->SetUnpackStatus(NZBInfo::usNone);
 			pNZBInfo->SetCleanupStatus(NZBInfo::csNone);
+
+			if (m_ParCoordinator.FindMainPars(pNZBInfo->GetDestDir(), NULL))
+			{
+				pNZBInfo->SetParStatus(NZBInfo::psNone);
+				pNZBInfo->SetRenameStatus(NZBInfo::rsNone);
+			}
 		}
+		pNZBInfo->SetDeleteStatus(NZBInfo::dsNone);
+		pNZBInfo->SetDeletePaused(false);
+		pNZBInfo->SetMarkStatus(NZBInfo::ksNone);
 		pNZBInfo->GetScriptStatuses()->Clear();
 		pNZBInfo->SetParkedFileCount(0);
 	}
@@ -1223,6 +1335,88 @@ void PrePostProcessor::HistoryReturn(DownloadQueue* pDownloadQueue, HistoryList:
 	}
 }
 
+void PrePostProcessor::HistoryRedownload(DownloadQueue* pDownloadQueue, HistoryList::iterator itHistory,
+	HistoryInfo* pHistoryInfo, bool bRestorePauseState)
+{
+	NZBInfo* pNZBInfo = pHistoryInfo->GetNZBInfo();
+	bool bPaused = bRestorePauseState && pNZBInfo->GetDeletePaused();
+	int iGroupdID = -1;
+
+	if (!Util::FileExists(pNZBInfo->GetQueuedFilename()))
+	{
+		error("Could not return collection %s from history back to queue: could not find source nzb-file %s",
+			pNZBInfo->GetName(), pNZBInfo->GetQueuedFilename());
+		return;
+	}
+
+	NZBFile* pNZBFile = NZBFile::Create(pNZBInfo->GetQueuedFilename(), "");
+	if (pNZBFile == NULL)
+	{
+		error("Could not return collection %s from history back to queue: could not parse nzb-file",
+			pNZBInfo->GetName());
+		return;
+	}
+
+	info("Returning collection %s from history back to queue", pNZBInfo->GetName());
+
+	for (NZBFile::FileInfos::iterator it = pNZBFile->GetFileInfos()->begin(); it != pNZBFile->GetFileInfos()->end(); it++)
+	{
+		FileInfo* pFileInfo = *it;
+		pFileInfo->SetNZBInfo(pNZBInfo);
+		pFileInfo->SetPaused(bPaused);
+		iGroupdID = pFileInfo->GetID();
+	}
+
+	if (Util::DirectoryExists(pNZBInfo->GetDestDir()))
+	{
+		detail("Deleting %s", pNZBInfo->GetDestDir());
+		Util::DeleteDirectoryWithContent(pNZBInfo->GetDestDir());
+	}
+
+	pNZBInfo->BuildDestDirName();
+	if (Util::DirectoryExists(pNZBInfo->GetDestDir()))
+	{
+		detail("Deleting %s", pNZBInfo->GetDestDir());
+		Util::DeleteDirectoryWithContent(pNZBInfo->GetDestDir());
+	}
+
+	// reset status fields (which are not reset by "HistoryReturn")
+	pNZBInfo->SetMoveStatus(NZBInfo::msNone);
+	pNZBInfo->SetUnpackCleanedUpDisk(false);
+	pNZBInfo->SetParStatus(NZBInfo::psNone);
+	pNZBInfo->SetRenameStatus(NZBInfo::rsNone);
+	pNZBInfo->ClearCompletedFiles();
+	pNZBInfo->GetServerStats()->Clear();
+
+	// take file stats from newly read nzb-file
+	pNZBInfo->SetFileCount(pNZBFile->GetNZBInfo()->GetFileCount());
+	pNZBInfo->SetFullContentHash(pNZBFile->GetNZBInfo()->GetFullContentHash());
+	pNZBInfo->SetFilteredContentHash(pNZBFile->GetNZBInfo()->GetFilteredContentHash());
+	pNZBInfo->SetSize(pNZBFile->GetNZBInfo()->GetSize());
+	pNZBInfo->SetSuccessSize(pNZBFile->GetNZBInfo()->GetSuccessSize());
+	pNZBInfo->SetCurrentSuccessSize(pNZBFile->GetNZBInfo()->GetCurrentSuccessSize());
+	pNZBInfo->SetFailedSize(pNZBFile->GetNZBInfo()->GetFailedSize());
+	pNZBInfo->SetCurrentFailedSize(pNZBFile->GetNZBInfo()->GetCurrentFailedSize());
+	pNZBInfo->SetParSize(pNZBFile->GetNZBInfo()->GetParSize());
+	pNZBInfo->SetParSuccessSize(pNZBFile->GetNZBInfo()->GetParSuccessSize());
+	pNZBInfo->SetParCurrentSuccessSize(pNZBFile->GetNZBInfo()->GetParCurrentSuccessSize());
+	pNZBInfo->SetParFailedSize(pNZBFile->GetNZBInfo()->GetParFailedSize());
+	pNZBInfo->SetParCurrentFailedSize(pNZBFile->GetNZBInfo()->GetParCurrentFailedSize());
+	pNZBInfo->SetSuccessArticles(pNZBFile->GetNZBInfo()->GetSuccessArticles());
+	pNZBInfo->SetFailedArticles(pNZBFile->GetNZBInfo()->GetFailedArticles());
+
+	g_pQueueCoordinator->AddFileInfosToFileQueue(pNZBFile, pDownloadQueue->GetParkedFiles(), false);
+
+	delete pNZBFile;
+
+	HistoryReturn(pDownloadQueue, itHistory, pHistoryInfo, false);
+
+	if (!bPaused && g_pOptions->GetParCheck() != Options::pcForce)
+	{
+		g_pQueueCoordinator->GetQueueEditor()->LockedEditEntry(pDownloadQueue, iGroupdID, false, QueueEditor::eaGroupPauseExtraPars, 0, NULL);
+	}
+}
+
 void PrePostProcessor::HistorySetParameter(HistoryInfo* pHistoryInfo, const char* szText)
 {
 	char szNiceName[1024];
@@ -1242,7 +1436,7 @@ void PrePostProcessor::HistorySetParameter(HistoryInfo* pHistoryInfo, const char
 	{
 		*szValue = '\0';
 		szValue++;
-		pHistoryInfo->GetNZBInfo()->SetParameter(szStr, szValue);
+		pHistoryInfo->GetNZBInfo()->GetParameters()->SetParameter(szStr, szValue);
 	}
 	else
 	{
@@ -1251,3 +1445,97 @@ void PrePostProcessor::HistorySetParameter(HistoryInfo* pHistoryInfo, const char
 
 	free(szStr);
 }
+
+void PrePostProcessor::HistorySetDupeParam(HistoryInfo* pHistoryInfo, EEditAction eAction, const char* szText)
+{
+	char szNiceName[1024];
+	pHistoryInfo->GetName(szNiceName, 1024);
+	debug("Setting dupe-parameter '%i'='%s' for '%s'", (int)eAction, szText, szNiceName);
+
+	if (!(pHistoryInfo->GetKind() == HistoryInfo::hkNZBInfo || 
+		pHistoryInfo->GetKind() == HistoryInfo::hkDupInfo))
+	{
+		error("Could not set duplicate parameter for %s: history item has wrong type", szNiceName);
+		return;
+	}
+
+	EDupeMode eMode = dmScore;
+	if (eAction == eaHistorySetDupeMode)
+	{
+		if (!strcasecmp(szText, "SCORE"))
+		{
+			eMode = dmScore;
+		}
+		else if (!strcasecmp(szText, "ALL"))
+		{
+			eMode = dmAll;
+		}
+		else if (!strcasecmp(szText, "FORCE"))
+		{
+			eMode = dmForce;
+		}
+		else
+		{
+			error("Could not set duplicate mode for %s: incorrect mode (%s)", szNiceName, szText);
+			return;
+		}
+	}
+
+	if (pHistoryInfo->GetKind() == HistoryInfo::hkNZBInfo)
+	{
+		switch (eAction) 
+		{
+			case eaHistorySetDupeKey:
+				pHistoryInfo->GetNZBInfo()->SetDupeKey(szText);
+				break;
+
+			case eaHistorySetDupeScore:
+				pHistoryInfo->GetNZBInfo()->SetDupeScore(atoi(szText));
+				break;
+
+			case eaHistorySetDupeMode:
+				pHistoryInfo->GetNZBInfo()->SetDupeMode(eMode);
+				break;
+
+			case eaHistorySetDupeBackup:
+				if (pHistoryInfo->GetNZBInfo()->GetDeleteStatus() != NZBInfo::dsDupe &&
+					pHistoryInfo->GetNZBInfo()->GetDeleteStatus() != NZBInfo::dsManual)
+				{
+					error("Could not set duplicate parameter for %s: history item has wrong delete status", szNiceName);
+					return;
+				}
+				pHistoryInfo->GetNZBInfo()->SetDeleteStatus(!strcasecmp(szText, "YES") ||
+					!strcasecmp(szText, "TRUE") || !strcasecmp(szText, "1") ? NZBInfo::dsDupe : NZBInfo::dsManual);
+				break;
+
+			default:
+				// suppress compiler warning
+				break;
+		}
+	}
+	else if (pHistoryInfo->GetKind() == HistoryInfo::hkDupInfo)
+	{
+		switch (eAction) 
+		{
+			case eaHistorySetDupeKey:
+				pHistoryInfo->GetDupInfo()->SetDupeKey(szText);
+				break;
+
+			case eaHistorySetDupeScore:
+				pHistoryInfo->GetDupInfo()->SetDupeScore(atoi(szText));
+				break;
+
+			case eaHistorySetDupeMode:
+				pHistoryInfo->GetDupInfo()->SetDupeMode(eMode);
+				break;
+
+			case eaHistorySetDupeBackup:
+				error("Could not set duplicate parameter for %s: history item has wrong type", szNiceName);
+				return;
+
+			default:
+				// suppress compiler warning
+				break;
+		}
+	}
+}
