@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 708 $
- * $Date: 2013-06-13 22:51:38 +0200 (Thu, 13 Jun 2013) $
+ * $Revision: 911 $
+ * $Date: 2013-11-24 20:29:52 +0100 (Sun, 24 Nov 2013) $
  *
  */
 
@@ -40,6 +40,7 @@
 #include <unistd.h>
 #include <sys/time.h>
 #endif
+#include <algorithm>
 
 #include "nzbget.h"
 #include "QueueCoordinator.h"
@@ -67,6 +68,7 @@ QueueCoordinator::QueueCoordinator()
 	m_tStartDownload = 0;
 	m_tPausedFrom = 0;
 	m_bStandBy = true;
+	m_iServerConfigGeneration = 0;
 
 	YDecoder::Init();
 }
@@ -101,6 +103,13 @@ void QueueCoordinator::Run()
 
 	m_mutexDownloadQueue.Lock();
 
+	if (g_pOptions->GetServerMode())
+	{
+		g_pDiskState->LoadStats(g_pServerPool->GetServers());
+		// currently there are no any stats but we need to save current server list into diskstate
+		g_pDiskState->SaveStats(g_pServerPool->GetServers());
+	}
+
 	if (g_pOptions->GetServerMode() && g_pOptions->GetSaveQueue() && g_pDiskState->DownloadQueueExists())
 	{
 		if (g_pOptions->GetReloadQueue())
@@ -117,17 +126,7 @@ void QueueCoordinator::Run()
 
 	m_mutexDownloadQueue.Unlock();
 
-	// compute maximum number of allowed download threads
-	m_iDownloadsLimit = 2; // two extra threads for completing files (when connections are not needed)
-	for (ServerPool::Servers::iterator it = g_pServerPool->GetServers()->begin(); it != g_pServerPool->GetServers()->end(); it++)
-	{
-		NewsServer* pNewsServer = *it;
-		if (pNewsServer->GetLevel() == 0)
-		{
-			m_iDownloadsLimit += pNewsServer->GetMaxConnections();
-		}
-	}
-
+	AdjustDownloadsLimit();
 	m_tStartServer = time(NULL);
 	m_tLastCheck = m_tStartServer;
 	bool bWasStandBy = true;
@@ -136,6 +135,7 @@ void QueueCoordinator::Run()
 
 	while (!IsStopped())
 	{
+		bool bDownloadsChecked = false;
 		if (!(g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2()))
 		{
 			NNTPConnection* pConnection = g_pServerPool->GetConnection(0, NULL, NULL);
@@ -149,6 +149,7 @@ void QueueCoordinator::Run()
 				m_mutexDownloadQueue.Lock();
 				bool bHasMoreArticles = GetNextArticle(pFileInfo, pArticleInfo);
 				bArticeDownloadsRunning = !m_ActiveDownloads.empty();
+				bDownloadsChecked = true;
 				m_bHasMoreJobs = bHasMoreArticles || bArticeDownloadsRunning;
 				if (bHasMoreArticles && !IsStopped() && (int)m_ActiveDownloads.size() < m_iDownloadsLimit)
 				{
@@ -167,7 +168,8 @@ void QueueCoordinator::Run()
 				}
 			}
 		}
-		else
+
+		if (!bDownloadsChecked)
 		{
 			m_mutexDownloadQueue.Lock();
 			bArticeDownloadsRunning = !m_ActiveDownloads.empty();
@@ -198,6 +200,7 @@ void QueueCoordinator::Run()
 			ResetHangingDownloads();
 			iResetCounter = 0;
 			AdjustStartTime();
+			AdjustDownloadsLimit();
 		}
 	}
 
@@ -217,16 +220,94 @@ void QueueCoordinator::Run()
 	debug("Exiting QueueCoordinator-loop");
 }
 
+/*
+ * Compute maximum number of allowed download threads
+**/
+void QueueCoordinator::AdjustDownloadsLimit()
+{
+	if (m_iServerConfigGeneration == g_pServerPool->GetGeneration())
+	{
+		return;
+	}
+
+	// two extra threads for completing files (when connections are not needed)
+	int iDownloadsLimit = 2;
+
+	// allow one thread per 0-level (main) and 1-level (backup) server connection
+	for (Servers::iterator it = g_pServerPool->GetServers()->begin(); it != g_pServerPool->GetServers()->end(); it++)
+	{
+		NewsServer* pNewsServer = *it;
+		if ((pNewsServer->GetNormLevel() == 0 || pNewsServer->GetNormLevel() == 1) && pNewsServer->GetActive())
+		{
+			iDownloadsLimit += pNewsServer->GetMaxConnections();
+		}
+	}
+
+	m_iDownloadsLimit = iDownloadsLimit;
+}
+
 void QueueCoordinator::AddNZBFileToQueue(NZBFile* pNZBFile, bool bAddFirst)
 {
 	debug("Adding NZBFile to queue");
 
 	m_mutexDownloadQueue.Lock();
 
+	Aspect foundAspect = { eaNZBFileFound, &m_DownloadQueue, pNZBFile->GetNZBInfo(), NULL };
+	Notify(&foundAspect);
+
+	if (pNZBFile->GetNZBInfo()->GetDeleteStatus() != NZBInfo::dsNone)
+	{
+		bool bAllPaused = !pNZBFile->GetFileInfos()->empty();
+		for (NZBFile::FileInfos::iterator it = pNZBFile->GetFileInfos()->begin(); it != pNZBFile->GetFileInfos()->end(); it++)
+		{
+			FileInfo* pFileInfo = *it;
+			bAllPaused &= pFileInfo->GetPaused();
+			if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
+			{
+				g_pDiskState->DiscardFile(pFileInfo);
+			}
+		}
+		pNZBFile->GetNZBInfo()->SetDeletePaused(bAllPaused);
+	}
+
+	if (pNZBFile->GetNZBInfo()->GetDeleteStatus() == NZBInfo::dsManual)
+	{
+		m_mutexDownloadQueue.Unlock(); // UNLOCK
+		return;
+	}
+
+	m_DownloadQueue.GetNZBInfoList()->Add(pNZBFile->GetNZBInfo());
+
+	if (pNZBFile->GetNZBInfo()->GetDeleteStatus() == NZBInfo::dsNone)
+	{
+		AddFileInfosToFileQueue(pNZBFile, m_DownloadQueue.GetFileQueue(), bAddFirst);
+	}
+
+	char szNZBName[1024];
+	strncpy(szNZBName, pNZBFile->GetNZBInfo()->GetName(), sizeof(szNZBName)-1);
+
+	Aspect aspect = { eaNZBFileAdded, &m_DownloadQueue, pNZBFile->GetNZBInfo(), NULL };
+	Notify(&aspect);
+	
+	if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
+	{
+		g_pDiskState->SaveDownloadQueue(&m_DownloadQueue);
+	}
+
+	m_mutexDownloadQueue.Unlock();
+
+	if (pNZBFile->GetNZBInfo()->GetDeleteStatus() == NZBInfo::dsNone)
+	{
+		info("Collection %s added to queue", szNZBName);
+	}
+}
+
+void QueueCoordinator::AddFileInfosToFileQueue(NZBFile* pNZBFile, FileQueue* pFileQueue, bool bAddFirst)
+{
+	debug("Adding NZBFile to queue");
+
 	FileQueue tmpFileQueue;
-	tmpFileQueue.clear();
 	FileQueue DupeList;
-	DupeList.clear();
 
 	int index1 = 0;
 	for (NZBFile::FileInfos::iterator it = pNZBFile->GetFileInfos()->begin(); it != pNZBFile->GetFileInfos()->end(); it++)
@@ -234,14 +315,9 @@ void QueueCoordinator::AddNZBFileToQueue(NZBFile* pNZBFile, bool bAddFirst)
 		index1++;
 		FileInfo* pFileInfo = *it;
 
-		if (g_pOptions->GetDupeCheck())
+		if (g_pOptions->GetDupeCheck() && !pNZBFile->GetNZBInfo()->GetDupeMode())
 		{
 			bool dupe = false;
-			if (IsDupe(pFileInfo))
-			{
-				warn("File \"%s\" seems to be duplicate, skipping", pFileInfo->GetFilename());
-				dupe = true;
-			}
 			int index2 = 0;
 			for (NZBFile::FileInfos::iterator it2 = pNZBFile->GetFileInfos()->begin(); it2 != pNZBFile->GetFileInfos()->end(); it2++)
 			{
@@ -252,7 +328,7 @@ void QueueCoordinator::AddNZBFileToQueue(NZBFile* pNZBFile, bool bAddFirst)
 					(pFileInfo->GetSize() < pFileInfo2->GetSize() || 
 					 (pFileInfo->GetSize() == pFileInfo2->GetSize() && index2 < index1)))
 				{
-					warn("File \"%s\" appears twice in nzb-request, adding only the biggest file", pFileInfo->GetFilename());
+					warn("File \"%s\" appears twice in collection, adding only the biggest file", pFileInfo->GetFilename());
 					dupe = true;
 					break;
 				}
@@ -260,6 +336,7 @@ void QueueCoordinator::AddNZBFileToQueue(NZBFile* pNZBFile, bool bAddFirst)
 			if (dupe)
 			{
 				DupeList.push_back(pFileInfo);
+				StatFileInfo(pFileInfo, false);
 				continue;
 			}
 		}
@@ -278,11 +355,11 @@ void QueueCoordinator::AddNZBFileToQueue(NZBFile* pNZBFile, bool bAddFirst)
 	{
 		if (bAddFirst)
 		{
-			m_DownloadQueue.GetFileQueue()->push_front(*it);
+			pFileQueue->push_front(*it);
 		}
 		else
 		{
-			m_DownloadQueue.GetFileQueue()->push_back(*it);
+			pFileQueue->push_back(*it);
 		}
 	}
 
@@ -296,19 +373,7 @@ void QueueCoordinator::AddNZBFileToQueue(NZBFile* pNZBFile, bool bAddFirst)
 		delete pFileInfo;
 	}
 
-	m_DownloadQueue.GetNZBInfoList()->Add(pNZBFile->GetNZBInfo());
-
 	pNZBFile->DetachFileInfos();
-
-	Aspect aspect = { eaNZBFileAdded, &m_DownloadQueue, pNZBFile->GetNZBInfo(), NULL };
-	Notify(&aspect);
-	
-	if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
-	{
-		g_pDiskState->SaveDownloadQueue(&m_DownloadQueue);
-	}
-
-	m_mutexDownloadQueue.Unlock();
 }
 
 /*
@@ -447,6 +512,8 @@ bool QueueCoordinator::DeleteQueueEntry(FileInfo* pFileInfo)
 	}
 	if (!hasDownloads)
 	{
+		StatFileInfo(pFileInfo, false);
+
 		Aspect aspect = { eaFileDeleted, &m_DownloadQueue, pFileInfo->GetNZBInfo(), pFileInfo };
 		Notify(&aspect);
 
@@ -547,10 +614,7 @@ bool QueueCoordinator::GetNextArticle(FileInfo* &pFileInfo, ArticleInfo* &pArtic
 		}
 	}
 
-	if (pCheckedFiles)
-	{
-		free(pCheckedFiles);
-	}
+	free(pCheckedFiles);
 
 	return bOK;
 }
@@ -567,7 +631,7 @@ void QueueCoordinator::StartArticleDownload(FileInfo* pFileInfo, ArticleInfo* pA
 	pArticleDownloader->SetConnection(pConnection);
 
 	char szInfoName[1024];
-	snprintf(szInfoName, 1024, "%s%c%s [%i/%i]", pFileInfo->GetNZBInfo()->GetName(), (int)PATH_SEPARATOR, pFileInfo->GetFilename(), pArticleInfo->GetPartNumber(), pFileInfo->GetArticles()->size());
+	snprintf(szInfoName, 1024, "%s%c%s [%i/%i]", pFileInfo->GetNZBInfo()->GetName(), (int)PATH_SEPARATOR, pFileInfo->GetFilename(), pArticleInfo->GetPartNumber(), (int)pFileInfo->GetArticles()->size());
 	szInfoName[1024-1] = '\0';
 	pArticleDownloader->SetInfoName(szInfoName);
 
@@ -616,10 +680,18 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* pArticleDownloader)
 	if (pArticleDownloader->GetStatus() == ArticleDownloader::adFinished)
 	{
 		pArticleInfo->SetStatus(ArticleInfo::aiFinished);
+		pFileInfo->SetSuccessSize(pFileInfo->GetSuccessSize() + pArticleInfo->GetSize());
+		pFileInfo->GetNZBInfo()->SetCurrentSuccessSize(pFileInfo->GetNZBInfo()->GetCurrentSuccessSize() + pArticleInfo->GetSize());
+		pFileInfo->GetNZBInfo()->SetParCurrentSuccessSize(pFileInfo->GetNZBInfo()->GetParCurrentSuccessSize() + (pFileInfo->GetParFile() ? pArticleInfo->GetSize() : 0));
+		pFileInfo->SetSuccessArticles(pFileInfo->GetSuccessArticles() + 1);
 	}
 	else if (pArticleDownloader->GetStatus() == ArticleDownloader::adFailed)
 	{
 		pArticleInfo->SetStatus(ArticleInfo::aiFailed);
+		pFileInfo->SetFailedSize(pFileInfo->GetFailedSize() + pArticleInfo->GetSize());
+		pFileInfo->GetNZBInfo()->SetCurrentFailedSize(pFileInfo->GetNZBInfo()->GetCurrentFailedSize() + pArticleInfo->GetSize());
+		pFileInfo->GetNZBInfo()->SetParCurrentFailedSize(pFileInfo->GetNZBInfo()->GetParCurrentFailedSize() + (pFileInfo->GetParFile() ? pArticleInfo->GetSize() : 0));
+		pFileInfo->SetFailedArticles(pFileInfo->GetFailedArticles() + 1);
 	}
 	else if (pArticleDownloader->GetStatus() == ArticleDownloader::adRetry)
 	{
@@ -632,6 +704,7 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* pArticleDownloader)
 		pFileInfo->SetRemainingSize(pFileInfo->GetRemainingSize() - pArticleInfo->GetSize());
 		pFileInfo->SetCompleted(pFileInfo->GetCompleted() + 1);
 		fileCompleted = (int)pFileInfo->GetArticles()->size() == pFileInfo->GetCompleted();
+		pFileInfo->GetNZBInfo()->GetServerStats()->Add(pArticleDownloader->GetServerStats());
 	}
 
 	if (!pFileInfo->GetFilenameConfirmed() &&
@@ -640,7 +713,10 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* pArticleDownloader)
 	{
 		pFileInfo->SetFilename(pArticleDownloader->GetArticleFilename());
 		pFileInfo->SetFilenameConfirmed(true);
-		if (g_pOptions->GetDupeCheck() && pFileInfo->IsDupe(pFileInfo->GetFilename()))
+		if (g_pOptions->GetDupeCheck() &&
+			pFileInfo->GetNZBInfo()->GetDupeMode() != dmForce &&
+			!pFileInfo->GetNZBInfo()->GetManyDupeFiles() &&
+			Util::FileExists(pFileInfo->GetNZBInfo()->GetDestDir(), pFileInfo->GetFilename()))
 		{
 			warn("File \"%s\" seems to be duplicate, cancelling download and deleting file from queue", pFileInfo->GetFilename());
 			fileCompleted = false;
@@ -651,23 +727,6 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* pArticleDownloader)
 
 	bool deleteFileObj = false;
 
-	if (pFileInfo->GetDeleted())
-	{
-		int cnt = 0;
-		for (ActiveDownloads::iterator it = m_ActiveDownloads.begin(); it != m_ActiveDownloads.end(); it++)
-		{
-			if ((*it)->GetFileInfo() == pFileInfo)
-			{
-				cnt++;
-			}
-		}
-		if (cnt == 1)
-		{
-			// this was the last Download for a file deleted from queue
-			deleteFileObj = true;
-		}
-	}
-
 	if (fileCompleted && !IsStopped() && !pFileInfo->GetDeleted())
 	{
 		// all jobs done
@@ -677,16 +736,22 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* pArticleDownloader)
 		deleteFileObj = true;
 	}
 
-	// delete Download from Queue
+	CheckHealth(pFileInfo);
+
+	bool hasOtherDownloaders = false;
 	for (ActiveDownloads::iterator it = m_ActiveDownloads.begin(); it != m_ActiveDownloads.end(); it++)
 	{
-		ArticleDownloader* pa = *it;
-		if (pa == pArticleDownloader)
+		ArticleDownloader* pDownloader = *it;
+		if (pDownloader != pArticleDownloader && pDownloader->GetFileInfo() == pFileInfo)
 		{
-			m_ActiveDownloads.erase(it);
+			hasOtherDownloaders = true;
 			break;
 		}
 	}
+	deleteFileObj |= pFileInfo->GetDeleted() && !hasOtherDownloaders;
+
+	// remove downloader from downloader list
+	m_ActiveDownloads.erase(std::find(m_ActiveDownloads.begin(), m_ActiveDownloads.end(), pArticleDownloader));
 
 	pFileInfo->SetActiveDownloads(pFileInfo->GetActiveDownloads() - 1);
 
@@ -696,6 +761,8 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* pArticleDownloader)
 		// delete File from Queue
 		pFileInfo->SetDeleted(true);
 
+		StatFileInfo(pFileInfo, fileCompleted);
+
 		Aspect aspect = { fileCompleted && !fileDeleted ? eaFileCompleted : eaFileDeleted, &m_DownloadQueue, pFileInfo->GetNZBInfo(), pFileInfo };
 		Notify(&aspect);
 		
@@ -709,6 +776,40 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* pArticleDownloader)
 	m_mutexDownloadQueue.Unlock();
 }
 
+void QueueCoordinator::StatFileInfo(FileInfo* pFileInfo, bool bCompleted)
+{
+	NZBInfo* pNZBInfo = pFileInfo->GetNZBInfo();
+	if (bCompleted || pNZBInfo->GetDeleting())
+	{
+		pNZBInfo->SetSuccessSize(pNZBInfo->GetSuccessSize() + pFileInfo->GetSuccessSize());
+		pNZBInfo->SetFailedSize(pNZBInfo->GetFailedSize() + pFileInfo->GetFailedSize());
+		pNZBInfo->SetFailedArticles(pNZBInfo->GetFailedArticles() + pFileInfo->GetFailedArticles() + pFileInfo->GetMissedArticles());
+		pNZBInfo->SetSuccessArticles(pNZBInfo->GetSuccessArticles() + pFileInfo->GetSuccessArticles());
+		if (pFileInfo->GetParFile())
+		{
+			pNZBInfo->SetParSuccessSize(pNZBInfo->GetParSuccessSize() + pFileInfo->GetSuccessSize());
+			pNZBInfo->SetParFailedSize(pNZBInfo->GetParFailedSize() + pFileInfo->GetFailedSize());
+		}
+	}
+	else if (!pNZBInfo->GetDeleting() && !pNZBInfo->GetParCleanup())
+	{
+		// file deleted but not the whole nzb and not par-cleanup
+		pNZBInfo->SetFileCount(pNZBInfo->GetFileCount() - 1);
+		pNZBInfo->SetSize(pNZBInfo->GetSize() - pFileInfo->GetSize());
+		pNZBInfo->SetCurrentSuccessSize(pNZBInfo->GetCurrentSuccessSize() - pFileInfo->GetSuccessSize());
+		pNZBInfo->SetFailedSize(pNZBInfo->GetFailedSize() - pFileInfo->GetMissedSize());
+		pNZBInfo->SetCurrentFailedSize(pNZBInfo->GetCurrentFailedSize() - pFileInfo->GetFailedSize() - pFileInfo->GetMissedSize());
+		pNZBInfo->SetTotalArticles(pNZBInfo->GetTotalArticles() - pFileInfo->GetTotalArticles());
+		if (pFileInfo->GetParFile())
+		{
+			pNZBInfo->SetParSize(pNZBInfo->GetParSize() - pFileInfo->GetSize());
+			pNZBInfo->SetParCurrentSuccessSize(pNZBInfo->GetParCurrentSuccessSize() - pFileInfo->GetSuccessSize());
+			pNZBInfo->SetParFailedSize(pNZBInfo->GetParFailedSize() - pFileInfo->GetMissedSize());
+			pNZBInfo->SetParCurrentFailedSize(pNZBInfo->GetParCurrentFailedSize() - pFileInfo->GetFailedSize() - pFileInfo->GetMissedSize());
+		}
+	}
+}
+
 void QueueCoordinator::DeleteFileInfo(FileInfo* pFileInfo, bool bCompleted)
 {
 	for (FileQueue::iterator it = m_DownloadQueue.GetFileQueue()->begin(); it != m_DownloadQueue.GetFileQueue()->end(); it++)
@@ -756,29 +857,30 @@ void QueueCoordinator::DiscardDiskFile(FileInfo* pFileInfo)
 	}
 }
 
-bool QueueCoordinator::IsDupe(FileInfo* pFileInfo)
+void QueueCoordinator::CheckHealth(FileInfo* pFileInfo)
 {
-	debug("Checking if the file is already queued");
-
-	// checking on disk
-	if (pFileInfo->IsDupe(pFileInfo->GetFilename()))
+	if (g_pOptions->GetHealthCheck() == Options::hcNone ||
+		pFileInfo->GetNZBInfo()->GetHealthPaused() ||
+		pFileInfo->GetNZBInfo()->GetDeleteStatus() == NZBInfo::dsHealth ||
+		pFileInfo->GetNZBInfo()->CalcHealth() >= pFileInfo->GetNZBInfo()->CalcCriticalHealth())
 	{
-		return true;
+		return;
 	}
 
-	// checking in queue
-	for (FileQueue::iterator it = m_DownloadQueue.GetFileQueue()->begin(); it != m_DownloadQueue.GetFileQueue()->end(); it++)
+	if (g_pOptions->GetHealthCheck() == Options::hcPause)
 	{
-		FileInfo* pQueueEntry = *it;
-		if (!strcmp(pFileInfo->GetNZBInfo()->GetDestDir(), pQueueEntry->GetNZBInfo()->GetDestDir()) &&
-			!strcmp(pFileInfo->GetFilename(), pQueueEntry->GetFilename()) &&
-			pFileInfo != pQueueEntry)
-		{
-			return true;
-		}
+		warn("Pausing %s due to health %.1f%% below critical %.1f%%", pFileInfo->GetNZBInfo()->GetName(),
+			pFileInfo->GetNZBInfo()->CalcHealth() / 10.0, pFileInfo->GetNZBInfo()->CalcCriticalHealth() / 10.0);
+		pFileInfo->GetNZBInfo()->SetHealthPaused(true);
+		m_QueueEditor.LockedEditEntry(&m_DownloadQueue, pFileInfo->GetID(), false, QueueEditor::eaGroupPause, 0, NULL);
+	}
+	else if (g_pOptions->GetHealthCheck() == Options::hcDelete)
+	{
+		warn("Cancelling download and deleting %s due to health %.1f%% below critical %.1f%%", pFileInfo->GetNZBInfo()->GetName(),
+			pFileInfo->GetNZBInfo()->CalcHealth() / 10.0, pFileInfo->GetNZBInfo()->CalcCriticalHealth() / 10.0);
+		pFileInfo->GetNZBInfo()->SetDeleteStatus(NZBInfo::dsHealth);
+		m_QueueEditor.LockedEditEntry(&m_DownloadQueue, pFileInfo->GetID(), false, QueueEditor::eaGroupDelete, 0, NULL);
 	}
-
-	return false;
 }
 
 void QueueCoordinator::LogDebugInfo()
@@ -1011,7 +1113,20 @@ bool QueueCoordinator::MergeQueueEntries(NZBInfo* pDestNZBInfo, NZBInfo* pSrcNZB
 	}
 
 	pDestNZBInfo->SetFileCount(pDestNZBInfo->GetFileCount() + pSrcNZBInfo->GetFileCount());
+	pDestNZBInfo->SetFullContentHash(0);
+	pDestNZBInfo->SetFilteredContentHash(0);
+
 	pDestNZBInfo->SetSize(pDestNZBInfo->GetSize() + pSrcNZBInfo->GetSize());
+	pDestNZBInfo->SetSuccessSize(pDestNZBInfo->GetSuccessSize() + pSrcNZBInfo->GetSuccessSize());
+	pDestNZBInfo->SetCurrentSuccessSize(pDestNZBInfo->GetCurrentSuccessSize() + pSrcNZBInfo->GetCurrentSuccessSize());
+	pDestNZBInfo->SetFailedSize(pDestNZBInfo->GetFailedSize() + pSrcNZBInfo->GetFailedSize());
+	pDestNZBInfo->SetCurrentFailedSize(pDestNZBInfo->GetCurrentFailedSize() + pSrcNZBInfo->GetCurrentFailedSize());
+
+	pDestNZBInfo->SetParSize(pDestNZBInfo->GetParSize() + pSrcNZBInfo->GetParSize());
+	pDestNZBInfo->SetParSuccessSize(pDestNZBInfo->GetParSuccessSize() + pSrcNZBInfo->GetParSuccessSize());
+	pDestNZBInfo->SetParCurrentSuccessSize(pDestNZBInfo->GetParCurrentSuccessSize() + pSrcNZBInfo->GetParCurrentSuccessSize());
+	pDestNZBInfo->SetParFailedSize(pDestNZBInfo->GetParFailedSize() + pSrcNZBInfo->GetParFailedSize());
+	pDestNZBInfo->SetParCurrentFailedSize(pDestNZBInfo->GetParCurrentFailedSize() + pSrcNZBInfo->GetParCurrentFailedSize());
 
 	// reattach completed file items to new NZBInfo-object
 	for (NZBInfo::Files::iterator it = pSrcNZBInfo->GetCompletedFiles()->begin(); it != pSrcNZBInfo->GetCompletedFiles()->end(); it++)
@@ -1068,20 +1183,20 @@ bool QueueCoordinator::SplitQueueEntries(FileQueue* pFileList, const char* szNam
 	}
 
 	NZBInfo* pNZBInfo = new NZBInfo();
-	pNZBInfo->AddReference();
+	pNZBInfo->Retain();
 	m_DownloadQueue.GetNZBInfoList()->Add(pNZBInfo);
 
 	pNZBInfo->SetFilename(pSrcNZBInfo->GetFilename());
 	pNZBInfo->SetName(szName);
 	pNZBInfo->SetCategory(pSrcNZBInfo->GetCategory());
+	pNZBInfo->SetFullContentHash(0);
+	pNZBInfo->SetFilteredContentHash(0);
 	pNZBInfo->BuildDestDirName();
 	pNZBInfo->SetQueuedFilename(pSrcNZBInfo->GetQueuedFilename());
+	pNZBInfo->GetParameters()->CopyFrom(pSrcNZBInfo->GetParameters());
 
-	for (NZBParameterList::iterator it = pSrcNZBInfo->GetParameters()->begin(); it != pSrcNZBInfo->GetParameters()->end(); it++)
-	{
-		NZBParameter* pNZBParameter = *it;
-		pNZBInfo->SetParameter(pNZBParameter->GetName(), pNZBParameter->GetValue());
-	}
+	pSrcNZBInfo->SetFullContentHash(0);
+	pSrcNZBInfo->SetFilteredContentHash(0);
 
 	for (FileQueue::iterator it = pFileList->begin(); it != pFileList->end(); it++)
 	{
@@ -1094,9 +1209,32 @@ bool QueueCoordinator::SplitQueueEntries(FileQueue* pFileList, const char* szNam
 
 		pSrcNZBInfo->SetFileCount(pSrcNZBInfo->GetFileCount() - 1);
 		pSrcNZBInfo->SetSize(pSrcNZBInfo->GetSize() - pFileInfo->GetSize());
+		pSrcNZBInfo->SetSuccessSize(pSrcNZBInfo->GetSuccessSize() - pFileInfo->GetSuccessSize());
+		pSrcNZBInfo->SetCurrentSuccessSize(pSrcNZBInfo->GetCurrentSuccessSize() - pFileInfo->GetSuccessSize());
+		pSrcNZBInfo->SetFailedSize(pSrcNZBInfo->GetFailedSize() - pFileInfo->GetFailedSize() - pFileInfo->GetMissedSize());
+		pSrcNZBInfo->SetCurrentFailedSize(pSrcNZBInfo->GetCurrentFailedSize() - pFileInfo->GetFailedSize() - pFileInfo->GetMissedSize());
 
 		pNZBInfo->SetFileCount(pNZBInfo->GetFileCount() + 1);
 		pNZBInfo->SetSize(pNZBInfo->GetSize() + pFileInfo->GetSize());
+		pNZBInfo->SetSuccessSize(pNZBInfo->GetSuccessSize() + pFileInfo->GetSuccessSize());
+		pNZBInfo->SetCurrentSuccessSize(pNZBInfo->GetCurrentSuccessSize() + pFileInfo->GetSuccessSize());
+		pNZBInfo->SetFailedSize(pNZBInfo->GetFailedSize() + pFileInfo->GetFailedSize() + pFileInfo->GetMissedSize());
+		pNZBInfo->SetCurrentFailedSize(pNZBInfo->GetCurrentFailedSize() + pFileInfo->GetFailedSize() + pFileInfo->GetMissedSize());
+
+		if (pFileInfo->GetParFile())
+		{
+			pSrcNZBInfo->SetParSize(pSrcNZBInfo->GetParSize() - pFileInfo->GetSize());
+			pSrcNZBInfo->SetParSuccessSize(pSrcNZBInfo->GetParSuccessSize() - pFileInfo->GetSuccessSize());
+			pSrcNZBInfo->SetParCurrentSuccessSize(pSrcNZBInfo->GetParCurrentSuccessSize() - pFileInfo->GetSuccessSize());
+			pSrcNZBInfo->SetParFailedSize(pSrcNZBInfo->GetParFailedSize() - pFileInfo->GetFailedSize() - pFileInfo->GetMissedSize());
+			pSrcNZBInfo->SetParCurrentFailedSize(pSrcNZBInfo->GetParCurrentFailedSize() - pFileInfo->GetFailedSize() - pFileInfo->GetMissedSize());
+
+			pNZBInfo->SetParSize(pNZBInfo->GetParSize() + pFileInfo->GetSize());
+			pNZBInfo->SetParSuccessSize(pNZBInfo->GetParSuccessSize() + pFileInfo->GetSuccessSize());
+			pNZBInfo->SetParCurrentSuccessSize(pNZBInfo->GetParCurrentSuccessSize() + pFileInfo->GetSuccessSize());
+			pNZBInfo->SetParFailedSize(pNZBInfo->GetParFailedSize() + pFileInfo->GetFailedSize() + pFileInfo->GetMissedSize());
+			pNZBInfo->SetParCurrentFailedSize(pNZBInfo->GetParCurrentFailedSize() + pFileInfo->GetFailedSize() + pFileInfo->GetMissedSize());
+		}
 	}
 
 	pNZBInfo->Release();
