@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 677 $
- * $Date: 2013-05-14 22:20:52 +0200 (Tue, 14 May 2013) $
+ * $Revision: 916 $
+ * $Date: 2013-12-10 21:25:07 +0100 (Tue, 10 Dec 2013) $
  *
  */
 
@@ -116,7 +116,14 @@ void QueueEditor::PauseUnpauseEntry(FileInfo* pFileInfo, bool bPause)
  */
 void QueueEditor::DeleteEntry(FileInfo* pFileInfo)
 {
-	info("Deleting file %s from download queue", pFileInfo->GetFilename());
+	if (pFileInfo->GetNZBInfo()->GetDeleting())
+	{
+		detail("Deleting file %s from download queue", pFileInfo->GetFilename());
+	}
+	else
+	{
+		info("Deleting file %s from download queue", pFileInfo->GetFilename());
+	}
 	g_pQueueCoordinator->DeleteQueueEntry(pFileInfo);
 }
 
@@ -221,85 +228,93 @@ bool QueueEditor::InternEditList(DownloadQueue* pDownloadQueue, IDList* pIDList,
 	ItemList cItemList;
 	PrepareList(pDownloadQueue, &cItemList, pIDList, bSmartOrder, eAction, iOffset);
 
-	if (eAction == eaFilePauseAllPars || eAction == eaFilePauseExtraPars)
+	switch (eAction)
 	{
-		PauseParsInGroups(&cItemList, eAction == eaFilePauseExtraPars);
-	}
-	else if (eAction == eaGroupMerge)
-	{
-		return MergeGroups(pDownloadQueue, &cItemList);
-	}
-	else if (eAction == eaFileSplit)
-	{
-		return SplitGroup(pDownloadQueue, &cItemList, szText);
-	}
-	else if (eAction == eaFileReorder)
-	{
-		ReorderFiles(pDownloadQueue, &cItemList);
-	}
-	else
-	{
-		for (ItemList::iterator it = cItemList.begin(); it != cItemList.end(); it++)
-		{
-			EditItem* pItem = *it;
-			switch (eAction)
+		case eaFilePauseAllPars:
+		case eaFilePauseExtraPars:
+			PauseParsInGroups(&cItemList, eAction == eaFilePauseExtraPars);	
+			break;
+
+		case eaGroupMerge:
+			return MergeGroups(pDownloadQueue, &cItemList);
+
+		case eaFileSplit:
+			return SplitGroup(pDownloadQueue, &cItemList, szText);
+
+		case eaFileReorder:
+			ReorderFiles(pDownloadQueue, &cItemList);
+			break;
+		
+		default:
+			for (ItemList::iterator it = cItemList.begin(); it != cItemList.end(); it++)
 			{
-				case eaFilePause:
-					PauseUnpauseEntry(pItem->m_pFileInfo, true);
-					break;
+				EditItem* pItem = *it;
+				switch (eAction)
+				{
+					case eaFilePause:
+						PauseUnpauseEntry(pItem->m_pFileInfo, true);
+						break;
 
-				case eaFileResume:
-					PauseUnpauseEntry(pItem->m_pFileInfo, false);
-					break;
+					case eaFileResume:
+						PauseUnpauseEntry(pItem->m_pFileInfo, false);
+						break;
 
-				case eaFileMoveOffset:
-				case eaFileMoveTop:
-				case eaFileMoveBottom:
-					MoveEntry(pDownloadQueue, pItem->m_pFileInfo, pItem->m_iOffset);
-					break;
+					case eaFileMoveOffset:
+					case eaFileMoveTop:
+					case eaFileMoveBottom:
+						MoveEntry(pDownloadQueue, pItem->m_pFileInfo, pItem->m_iOffset);
+						break;
 
-				case eaFileDelete:
-					DeleteEntry(pItem->m_pFileInfo);
-					break;
+					case eaFileDelete:
+						DeleteEntry(pItem->m_pFileInfo);
+						break;
 
-				case eaFileSetPriority:
-					SetPriorityEntry(pItem->m_pFileInfo, szText);
-					break;
+					case eaFileSetPriority:
+						SetPriorityEntry(pItem->m_pFileInfo, szText);
+						break;
 
-				case eaGroupSetCategory:
-					SetNZBCategory(pItem->m_pFileInfo->GetNZBInfo(), szText);
-					break;
+					case eaGroupSetCategory:
+						SetNZBCategory(pItem->m_pFileInfo->GetNZBInfo(), szText);
+						break;
 
-				case eaGroupSetName:
-					SetNZBName(pItem->m_pFileInfo->GetNZBInfo(), szText);
-					break;
+					case eaGroupSetName:
+						SetNZBName(pItem->m_pFileInfo->GetNZBInfo(), szText);
+						break;
 
-				case eaGroupSetParameter:
-					SetNZBParameter(pItem->m_pFileInfo->GetNZBInfo(), szText);
-					break;
+					case eaGroupSetDupeKey:
+					case eaGroupSetDupeScore:
+					case eaGroupSetDupeMode:
+						SetNZBDupeParam(pItem->m_pFileInfo->GetNZBInfo(), eAction, szText);
+						break;
 
-				case eaGroupPause:
-				case eaGroupResume:
-				case eaGroupDelete:
-				case eaGroupMoveTop:
-				case eaGroupMoveBottom:
-				case eaGroupMoveOffset:
-				case eaGroupPauseAllPars:
-				case eaGroupPauseExtraPars:
-				case eaGroupSetPriority:
-					EditGroup(pDownloadQueue, pItem->m_pFileInfo, eAction, iOffset, szText);
-					break;
+					case eaGroupSetParameter:
+						SetNZBParameter(pItem->m_pFileInfo->GetNZBInfo(), szText);
+						break;
 
-				case eaFilePauseAllPars:
-				case eaFilePauseExtraPars:
-				case eaGroupMerge:
-				case eaFileReorder:
-				case eaFileSplit:
-					// remove compiler warning "enumeration not handled in switch"
-					break;
+					case eaGroupPause:
+					case eaGroupResume:
+					case eaGroupDelete:
+					case eaGroupDupeDelete:
+					case eaGroupFinalDelete:
+					case eaGroupMoveTop:
+					case eaGroupMoveBottom:
+					case eaGroupMoveOffset:
+					case eaGroupPauseAllPars:
+					case eaGroupPauseExtraPars:
+					case eaGroupSetPriority:
+						EditGroup(pDownloadQueue, pItem->m_pFileInfo, eAction, iOffset, szText);
+						break;
+
+					case eaFilePauseAllPars:
+					case eaFilePauseExtraPars:
+					case eaGroupMerge:
+					case eaFileReorder:
+					case eaFileSplit:
+						// remove compiler warning "enumeration not handled in switch"
+						break;
+				}
+				delete pItem;
 			}
-			delete pItem;
-		}
 	}
 
 	return cItemList.size() > 0;
@@ -468,10 +483,7 @@ bool QueueEditor::BuildIDListFromNameList(DownloadQueue* pDownloadQueue, IDList*
 			}
 		}
 
-		if (pRegEx)
-		{
-			delete pRegEx;
-		}
+		delete pRegEx;
 
 		if (!bFound && (eMatchMode == mmName))
 		{
@@ -486,6 +498,7 @@ bool QueueEditor::EditGroup(DownloadQueue* pDownloadQueue, FileInfo* pFileInfo,
 {
 	IDList cIDList;
 	cIDList.clear();
+	bool bAllPaused = true;
 
 	// collecting files belonging to group
 	for (FileQueue::iterator it = pDownloadQueue->GetFileQueue()->begin(); it != pDownloadQueue->GetFileQueue()->end(); it++)
@@ -494,6 +507,7 @@ bool QueueEditor::EditGroup(DownloadQueue* pDownloadQueue, FileInfo* pFileInfo,
 		if (pFileInfo2->GetNZBInfo() == pFileInfo->GetNZBInfo())
 		{
 			cIDList.push_back(pFileInfo2->GetID());
+			bAllPaused &= pFileInfo2->GetPaused();
 		}
 	}
 
@@ -542,15 +556,21 @@ bool QueueEditor::EditGroup(DownloadQueue* pDownloadQueue, FileInfo* pFileInfo,
 		}
 		iOffset = iFileOffset;
 	}
-	else if (eAction == eaGroupDelete)
+	else if (eAction == eaGroupDelete || eAction == eaGroupDupeDelete || eAction == eaGroupFinalDelete)
 	{
-		pFileInfo->GetNZBInfo()->SetDeleted(true);
+		pFileInfo->GetNZBInfo()->SetDeleting(true);
+		pFileInfo->GetNZBInfo()->SetAvoidHistory(eAction == eaGroupFinalDelete);
+		pFileInfo->GetNZBInfo()->SetDeletePaused(bAllPaused);
+		if (eAction == eaGroupDupeDelete)
+		{
+			pFileInfo->GetNZBInfo()->SetDeleteStatus(NZBInfo::dsDupe);
+		}
 		pFileInfo->GetNZBInfo()->SetCleanupDisk(CanCleanupDisk(pDownloadQueue, pFileInfo->GetNZBInfo()));
 	}
 
 	EEditAction GroupToFileMap[] = { (EEditAction)0, eaFileMoveOffset, eaFileMoveTop, eaFileMoveBottom, eaFilePause,
 		eaFileResume, eaFileDelete, eaFilePauseAllPars, eaFilePauseExtraPars, eaFileSetPriority, eaFileReorder, eaFileSplit,
-		eaFileMoveOffset, eaFileMoveTop, eaFileMoveBottom, eaFilePause, eaFileResume, eaFileDelete, 
+		eaFileMoveOffset, eaFileMoveTop, eaFileMoveBottom, eaFilePause, eaFileResume, eaFileDelete, eaFileDelete, eaFileDelete,
 		eaFilePauseAllPars, eaFilePauseExtraPars, eaFileSetPriority,
 		(EEditAction)0, (EEditAction)0, (EEditAction)0 };
 
@@ -629,7 +649,7 @@ void QueueEditor::AlignAffectedGroups(DownloadQueue* pDownloadQueue, IDList* pID
 				}
 				if (iOffset > 0)
 				{
-					for (unsigned int i = iNum + 1; i <= cGroupList.size() - iOffset; i++)
+					for (int i = iNum + 1; i <= (int)cGroupList.size() - iOffset; i++)
 					{
 						if (!ItemExists(&cAffectedGroupList, cGroupList[i]))
 						{
@@ -815,7 +835,7 @@ void QueueEditor::SetNZBName(NZBInfo* pNZBInfo, const char* szName)
 }
 
 /**
-* Check if deletion of already downloaded files is possible (when nzb id deleted from queue).
+* Check if deletion of already downloaded files is possible (when nzb is deleted from queue).
 * The deletion is most always possible, except the case if all remaining files in queue 
 * (belonging to this nzb-file) are PARS.
 */
@@ -824,15 +844,18 @@ bool QueueEditor::CanCleanupDisk(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInf
     for (FileQueue::iterator it = pDownloadQueue->GetFileQueue()->begin(); it != pDownloadQueue->GetFileQueue()->end(); it++)
     {
         FileInfo* pFileInfo = *it;
-		char szLoFileName[1024];
-		strncpy(szLoFileName, pFileInfo->GetFilename(), 1024);
-		szLoFileName[1024-1] = '\0';
-		for (char* p = szLoFileName; *p; p++) *p = tolower(*p); // convert string to lowercase
-
-		if (!strstr(szLoFileName, ".par2"))
+		if (pFileInfo->GetNZBInfo() == pNZBInfo)
 		{
-			// non-par file found
-			return true;
+			char szLoFileName[1024];
+			strncpy(szLoFileName, pFileInfo->GetFilename(), 1024);
+			szLoFileName[1024-1] = '\0';
+			for (char* p = szLoFileName; *p; p++) *p = tolower(*p); // convert string to lowercase
+
+			if (!strstr(szLoFileName, ".par2"))
+			{
+				// non-par file found
+				return true;
+			}
 		}
 	}
 
@@ -953,7 +976,7 @@ void QueueEditor::SetNZBParameter(NZBInfo* pNZBInfo, const char* szParamString)
 	{
 		*szValue = '\0';
 		szValue++;
-		pNZBInfo->SetParameter(szStr, szValue);
+		pNZBInfo->GetParameters()->SetParameter(szStr, szValue);
 	}
 	else
 	{
@@ -962,3 +985,47 @@ void QueueEditor::SetNZBParameter(NZBInfo* pNZBInfo, const char* szParamString)
 
 	free(szStr);
 }
+
+void QueueEditor::SetNZBDupeParam(NZBInfo* pNZBInfo, EEditAction eAction, const char* szText)
+{
+	debug("QueueEditor: setting dupe parameter %i='%s' for '%s'", (int)eAction, szText, pNZBInfo->GetName());
+
+	switch (eAction) 
+	{
+		case eaGroupSetDupeKey:
+			pNZBInfo->SetDupeKey(szText);
+			break;
+
+		case eaGroupSetDupeScore:
+			pNZBInfo->SetDupeScore(atoi(szText));
+			break;
+
+		case eaGroupSetDupeMode:
+			{
+				EDupeMode eMode = dmScore;
+				if (!strcasecmp(szText, "SCORE"))
+				{
+					eMode = dmScore;
+				}
+				else if (!strcasecmp(szText, "ALL"))
+				{
+					eMode = dmAll;
+				}
+				else if (!strcasecmp(szText, "FORCE"))
+				{
+					eMode = dmForce;
+				}
+				else
+				{
+					error("Could not set duplicate mode for %s: incorrect mode (%s)", pNZBInfo->GetName(), szText);
+					return;
+				}
+				pNZBInfo->SetDupeMode(eMode);
+				break;
+			}
+
+		default:
+			// suppress compiler warning
+			break;
+	}
+}
