@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 711 $
- * $Date: 2013-06-17 22:39:46 +0200 (Mon, 17 Jun 2013) $
+ * $Revision: 911 $
+ * $Date: 2013-11-24 20:29:52 +0100 (Sun, 24 Nov 2013) $
  *
  */
 
@@ -76,10 +76,7 @@ RemoteClient::RemoteClient()
 
 RemoteClient::~RemoteClient()
 {
-	if (m_pConnection)
-	{
-		delete m_pConnection;
-	}
+	delete m_pConnection;
 }
 
 void RemoteClient::printf(const char * msg,...)
@@ -213,10 +210,7 @@ bool RemoteClient::RequestServerDownload(const char* szFilename, const char* szC
 	}
 
 	// Cleanup
-	if (szBuffer)
-	{
-		free(szBuffer);
-	}
+	free(szBuffer);
 
 	return OK;
 }
@@ -250,7 +244,7 @@ void RemoteClient::BuildFileList(SNZBListResponse* pListResponse, const char* pT
 			pNZBInfo->SetQueuedFilename(m_szQueuedFilename);
 			pNZBInfo->m_bMatch = ntohl(pListAnswer->m_bMatch);
 
-			pNZBInfo->AddReference();
+			pNZBInfo->Retain();
 			pDownloadQueue->GetNZBInfoList()->Add(pNZBInfo);
 
 			pBufPtr += sizeof(SNZBListResponseNZBEntry) + ntohl(pListAnswer->m_iFilenameLen) +
@@ -267,7 +261,7 @@ void RemoteClient::BuildFileList(SNZBListResponse* pListResponse, const char* pT
 			const char* szValue = pBufPtr + sizeof(SNZBListResponsePPPEntry) + ntohl(pListAnswer->m_iNameLen);
 
 			NZBInfo* pNZBInfo = pDownloadQueue->GetNZBInfoList()->at(ntohl(pListAnswer->m_iNZBIndex) - 1);
-			pNZBInfo->SetParameter(szName, szValue);
+			pNZBInfo->GetParameters()->SetParameter(szName, szValue);
 
 			pBufPtr += sizeof(SNZBListResponsePPPEntry) + ntohl(pListAnswer->m_iNameLen) +
 				ntohl(pListAnswer->m_iValueLen);
@@ -527,20 +521,20 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups, const char* szPa
 				{
 					if (szParameters[0] == '\0')
 					{
-						strncat(szParameters, " (", 1024);
+						strncat(szParameters, " (", sizeof(szParameters) - strlen(szParameters) - 1);
 					}
 					else
 					{
-						strncat(szParameters, ", ", 1024);
+						strncat(szParameters, ", ", sizeof(szParameters) - strlen(szParameters) - 1);
 					}
 					NZBParameter* pNZBParameter = *it;
-					strncat(szParameters, pNZBParameter->GetName(), 1024);
-					strncat(szParameters, "=", 1024);
-					strncat(szParameters, pNZBParameter->GetValue(), 1024);
+					strncat(szParameters, pNZBParameter->GetName(), sizeof(szParameters) - strlen(szParameters) - 1);
+					strncat(szParameters, "=", sizeof(szParameters) - strlen(szParameters) - 1);
+					strncat(szParameters, pNZBParameter->GetValue(), sizeof(szParameters) - strlen(szParameters) - 1);
 				}
 				if (szParameters[0] != '\0')
 				{
-					strncat(szParameters, ")", 1024);
+					strncat(szParameters, ")", sizeof(szParameters) - strlen(szParameters) - 1);
 				}
 
 				if (!szPattern || ((MatchedNZBInfo*)pGroupInfo->GetNZBInfo())->m_bMatch)
@@ -551,8 +545,6 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups, const char* szPa
 						szPaused, szThreads, szCategory, szParameters);
 					iMatches++;
 				}
-
-				delete pGroupInfo;
 			}
 
 			for (FileQueue::iterator it = cRemoteQueue.GetFileQueue()->begin(); it != cRemoteQueue.GetFileQueue()->end(); it++)
@@ -658,10 +650,10 @@ bool RemoteClient::RequestServerList(bool bFiles, bool bGroups, const char* szPa
 
 	if (ntohl(ListResponse.m_iPostJobCount) > 0 || ntohl(ListResponse.m_bPostPaused))
 	{
-		strncat(szServerState, strlen(szServerState) > 0 ? ", Post-Processing" : "Post-Processing", sizeof(szServerState));
+		strncat(szServerState, strlen(szServerState) > 0 ? ", Post-Processing" : "Post-Processing", sizeof(szServerState) - strlen(szServerState) - 1);
 		if (ntohl(ListResponse.m_bPostPaused))
 		{
-			strncat(szServerState, " paused", sizeof(szServerState));
+			strncat(szServerState, " paused", sizeof(szServerState) - strlen(szServerState) - 1);
 		}
 	}
 
