@@ -1,8 +1,8 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2005  Bo Cordes Petersen <placebodk@users.sourceforge.net>
- *  Copyright (C) 2007  Andrei Prygounkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2005 Bo Cordes Petersen <placebodk@users.sourceforge.net>
+ *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,10 +16,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 91 $
- * $Date: 2008-02-02 14:23:43 +0100 (Sa, 02 Feb 2008) $
+ * $Revision: 542 $
+ * $Date: 2013-01-17 20:07:13 +0100 (Thu, 17 Jan 2013) $
  *
  */
 
@@ -28,17 +28,16 @@
 #define REMOTESERVER_H
 
 #include "Thread.h"
-#include "NetAddress.h"
 #include "Connection.h"
 
 class RemoteServer : public Thread
 {
 private:
-	NetAddress*			m_pNetAddress;
+	bool				m_bTLS;
 	Connection*			m_pConnection;
 
 public:
-						RemoteServer();
+						RemoteServer(bool bTLS);
 						~RemoteServer();
 	virtual void		Run();
 	virtual void 		Stop();
@@ -47,11 +46,14 @@ public:
 class RequestProcessor : public Thread
 {
 private:
-	SOCKET				m_iSocket;
+	bool				m_bTLS;
+	Connection*			m_pConnection;
 
 public:
+						~RequestProcessor();
 	virtual void		Run();
-	void				SetSocket(SOCKET iSocket) { m_iSocket = iSocket; };
+	void				SetTLS(bool bTLS) { m_bTLS = bTLS; }
+	void				SetConnection(Connection* pConnection) { m_pConnection = pConnection; }
 };
 
 #endif
