@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 711 $
- * $Date: 2013-06-17 22:39:46 +0200 (Mon, 17 Jun 2013) $
+ * $Revision: 909 $
+ * $Date: 2013-11-18 21:37:20 +0100 (Mon, 18 Nov 2013) $
  *
  */
 
@@ -199,6 +199,7 @@ ScriptController::ScriptController()
 	m_szInfoName = NULL;
 	m_szLogPrefix = NULL;
 	m_bTerminated = false;
+	m_bDetached = false;
 	m_environmentStrings.InitFromCurrentProcess();
 }
 
@@ -228,6 +229,14 @@ void ScriptController::SetEnvVar(const char* szName, const char* szValue)
 	m_environmentStrings.Append(szVar);
 }
 
+void ScriptController::SetIntEnvVar(const char* szName, int iValue)
+{
+	char szValue[1024];
+	snprintf(szValue, 10, "%i", iValue);
+	szValue[1024-1] = '\0';
+	SetEnvVar(szName, szValue);
+}
+
 /**
  * If szStripPrefix is not NULL, only options, whose names start with the prefix
  * are processed. The prefix is then stripped from the names.
@@ -268,15 +277,26 @@ void ScriptController::PrepareEnvParameters(NZBInfo* pNZBInfo, const char* szStr
 	for (NZBParameterList::iterator it = pNZBInfo->GetParameters()->begin(); it != pNZBInfo->GetParameters()->end(); it++)
 	{
 		NZBParameter* pParameter = *it;
+		const char* szValue = pParameter->GetValue();
 		
+#ifdef WIN32
+		char* szAnsiValue = strdup(szValue);
+		WebUtil::Utf8ToAnsi(szAnsiValue, strlen(szAnsiValue) + 1);
+		szValue = szAnsiValue;
+#endif
+
 		if (szStripPrefix && !strncmp(pParameter->GetName(), szStripPrefix, iPrefixLen) && (int)strlen(pParameter->GetName()) > iPrefixLen)
 		{
-			SetEnvVarSpecial("NZBPR", pParameter->GetName() + iPrefixLen, pParameter->GetValue());
+			SetEnvVarSpecial("NZBPR", pParameter->GetName() + iPrefixLen, szValue);
 		}
 		else if (!szStripPrefix)
 		{
-			SetEnvVarSpecial("NZBPR", pParameter->GetName(), pParameter->GetValue());
+			SetEnvVarSpecial("NZBPR", pParameter->GetName(), szValue);
 		}
+
+#ifdef WIN32
+		free(szAnsiValue);
+#endif
 	}
 }
 
@@ -497,6 +517,15 @@ int ScriptController::Execute()
 		chdir(m_szWorkingDir);
 		environ = pEnvironmentStrings;
 		execvp(m_szScript, (char* const*)m_szArgs);
+
+		if (errno == EACCES)
+		{
+			fprintf(stdout, "[WARNING] Fixing permissions for %s\n", m_szScript);
+			fflush(stdout);
+			Util::FixExecPermission(m_szScript);
+			execvp(m_szScript, (char* const*)m_szArgs);
+		}
+
 		// NOTE: the text "[ERROR] Could not start " is checked later,
 		// by changing adjust the dependent code below.
 		fprintf(stdout, "[ERROR] Could not start %s: %s", m_szScript, strerror(errno));
@@ -517,8 +546,8 @@ int ScriptController::Execute()
 #endif
 
 	// open the read end
-	FILE* readpipe = fdopen(pipein, "r");
-	if (!readpipe)
+	m_pReadpipe = fdopen(pipein, "r");
+	if (!m_pReadpipe)
 	{
 		error("Could not open pipe to %s", m_szInfoName);
 		return -1;
@@ -537,9 +566,9 @@ int ScriptController::Execute()
 	debug("Entering pipe-loop");
 	bool bFirstLine = true;
 	bool bStartError = false;
-	while (!feof(readpipe) && !m_bTerminated)
+	while (!m_bTerminated && !m_bDetached && !feof(m_pReadpipe))
 	{
-		if (ReadLine(buf, 10240, readpipe))
+		if (ReadLine(buf, 10240, m_pReadpipe) && m_pReadpipe)
 		{
 #ifdef CHILD_WATCHDOG
 			if (!bChildConfirmed)
@@ -574,7 +603,10 @@ int ScriptController::Execute()
 #endif
 	
 	free(buf);
-	fclose(readpipe);
+	if (m_pReadpipe)
+	{
+		fclose(m_pReadpipe);
+	}
 
 	if (m_bTerminated)
 	{
@@ -583,6 +615,8 @@ int ScriptController::Execute()
 
 	iExitCode = 0;
 
+	if (!m_bTerminated && !m_bDetached)
+	{
 #ifdef WIN32
 	WaitForSingleObject(m_hProcess, INFINITE);
 	DWORD dExitCode = 0;
@@ -600,6 +634,7 @@ int ScriptController::Execute()
 		}
 	}
 #endif
+	}
 	
 #ifdef CHILD_WATCHDOG
 	}	// while (!bChildConfirmed && !m_bTerminated)
@@ -639,6 +674,16 @@ void ScriptController::Terminate()
 	debug("Stopped %s", m_szInfoName);
 }
 
+void ScriptController::Detach()
+{
+	debug("Detaching %s", m_szInfoName);
+	m_bDetached = true;
+	FILE* pReadpipe = m_pReadpipe;
+	m_pReadpipe = NULL;
+	fclose(pReadpipe);
+}
+	
+
 bool ScriptController::ReadLine(char* szBuf, int iBufSize, FILE* pStream)
 {
 	return fgets(szBuf, iBufSize, pStream);
@@ -658,27 +703,27 @@ void ScriptController::ProcessOutput(char* szText)
 
 	if (!strncmp(szText, "[INFO] ", 7))
 	{
-		PrintMessage(Message::mkInfo, szText + 7);
+		PrintMessage(Message::mkInfo, "%s", szText + 7);
 	}
 	else if (!strncmp(szText, "[WARNING] ", 10))
 	{
-		PrintMessage(Message::mkWarning, szText + 10);
+		PrintMessage(Message::mkWarning, "%s", szText + 10);
 	}
 	else if (!strncmp(szText, "[ERROR] ", 8))
 	{
-		PrintMessage(Message::mkError, szText + 8);
+		PrintMessage(Message::mkError, "%s", szText + 8);
 	}
 	else if (!strncmp(szText, "[DETAIL] ", 9))
 	{
-		PrintMessage(Message::mkDetail, szText + 9);
+		PrintMessage(Message::mkDetail, "%s", szText + 9);
 	}
 	else if (!strncmp(szText, "[DEBUG] ", 8))
 	{
-		PrintMessage(Message::mkDebug, szText + 8);
+		PrintMessage(Message::mkDebug, "%s", szText + 8);
 	}
 	else 
 	{
-		PrintMessage(Message::mkInfo, szText);
+		PrintMessage(Message::mkInfo, "%s", szText);
 	}
 
 	debug("Processing output received from script - completed");
@@ -740,6 +785,7 @@ void PostScriptController::StartJob(PostInfo* pPostInfo)
 	pScriptController->m_pPostInfo = pPostInfo;
 	pScriptController->SetWorkingDir(g_pOptions->GetDestDir());
 	pScriptController->SetAutoDestroy(false);
+	pScriptController->m_iPrefixLen = 0;
 
 	pPostInfo->SetPostThread(pScriptController);
 
@@ -806,6 +852,7 @@ void PostScriptController::ExecuteScript(const char* szScriptName, const char* s
 	SetInfoName(szInfoName);
 
 	SetLogPrefix(szDisplayName);
+	m_iPrefixLen = strlen(szDisplayName) + 2; // 2 = strlen(": ");
 	PrepareParams(szScriptName);
 
 	int iExitCode = Execute();
@@ -826,46 +873,43 @@ void PostScriptController::PrepareParams(const char* szScriptName)
 	// the locking is needed for accessing the members of NZBInfo
 	g_pDownloadQueueHolder->LockQueue();
 
-	char szNZBName[1024];
-	strncpy(szNZBName, m_pPostInfo->GetNZBInfo()->GetName(), 1024);
-	szNZBName[1024-1] = '\0';
-
-	int iParStatus[] = { 0, 0, 1, 2, 3, 4 };
-	char szParStatus[10];
-	snprintf(szParStatus, 10, "%i", iParStatus[m_pPostInfo->GetNZBInfo()->GetParStatus()]);
-	szParStatus[10-1] = '\0';
-
-	int iUnpackStatus[] = { 0, 0, 1, 2 };
-	char szUnpackStatus[10];
-	snprintf(szUnpackStatus, 10, "%i", iUnpackStatus[m_pPostInfo->GetNZBInfo()->GetUnpackStatus()]);
-	szUnpackStatus[10-1] = '\0';
-
-	char szDestDir[1024];
-	strncpy(szDestDir, m_pPostInfo->GetNZBInfo()->GetDestDir(), 1024);
-	szDestDir[1024-1] = '\0';
-	
-	char szNZBID[10];
-	snprintf(szNZBID, 10, "%i", m_pPostInfo->GetNZBInfo()->GetID());
-	szNZBID[10-1] = '\0';
-
-	char szNZBFilename[1024];
-	strncpy(szNZBFilename, m_pPostInfo->GetNZBInfo()->GetFilename(), 1024);
-	szNZBFilename[1024-1] = '\0';
-	
-	char szCategory[1024];
-	strncpy(szCategory, m_pPostInfo->GetNZBInfo()->GetCategory(), 1024);
-	szCategory[1024-1] = '\0';
-
 	// Reset
 	ResetEnv();
 
-	SetEnvVar("NZBPP_NZBNAME", szNZBName);
-	SetEnvVar("NZBPP_NZBID", szNZBID);
-	SetEnvVar("NZBPP_DIRECTORY", szDestDir);
-	SetEnvVar("NZBPP_NZBFILENAME", szNZBFilename);
-	SetEnvVar("NZBPP_PARSTATUS", szParStatus);
-	SetEnvVar("NZBPP_UNPACKSTATUS", szUnpackStatus);
-	SetEnvVar("NZBPP_CATEGORY", szCategory);
+	SetEnvVar("NZBPP_NZBNAME", m_pPostInfo->GetNZBInfo()->GetName());
+	SetEnvVar("NZBPP_DIRECTORY", m_pPostInfo->GetNZBInfo()->GetDestDir());
+	SetEnvVar("NZBPP_NZBFILENAME", m_pPostInfo->GetNZBInfo()->GetFilename());
+	SetEnvVar("NZBPP_FINALDIR", m_pPostInfo->GetNZBInfo()->GetFinalDir());
+	SetEnvVar("NZBPP_CATEGORY", m_pPostInfo->GetNZBInfo()->GetCategory());
+	SetIntEnvVar("NZBPP_HEALTH", m_pPostInfo->GetNZBInfo()->CalcHealth());
+	SetIntEnvVar("NZBPP_CRITICALHEALTH", m_pPostInfo->GetNZBInfo()->CalcCriticalHealth());
+
+	int iParStatus[] = { 0, 0, 1, 2, 3, 4 };
+	SetIntEnvVar("NZBPP_PARSTATUS", iParStatus[m_pPostInfo->GetNZBInfo()->GetParStatus()]);
+
+	int iUnpackStatus[] = { 0, 0, 1, 2, 3, 4 };
+	SetIntEnvVar("NZBPP_UNPACKSTATUS", iUnpackStatus[m_pPostInfo->GetNZBInfo()->GetUnpackStatus()]);
+
+	SetIntEnvVar("NZBPP_NZBID", m_pPostInfo->GetNZBInfo()->GetID());
+	SetIntEnvVar("NZBPP_HEALTHDELETED", (int)m_pPostInfo->GetNZBInfo()->GetDeleteStatus() == NZBInfo::dsHealth);
+	SetIntEnvVar("NZBPP_TOTALARTICLES", (int)m_pPostInfo->GetNZBInfo()->GetTotalArticles());
+	SetIntEnvVar("NZBPP_SUCCESSARTICLES", (int)m_pPostInfo->GetNZBInfo()->GetSuccessArticles());
+	SetIntEnvVar("NZBPP_FAILEDARTICLES", (int)m_pPostInfo->GetNZBInfo()->GetFailedArticles());
+
+	for (ServerStatList::iterator it = m_pPostInfo->GetNZBInfo()->GetServerStats()->begin(); it != m_pPostInfo->GetNZBInfo()->GetServerStats()->end(); it++)
+	{
+		ServerStat* pServerStat = *it;
+
+		char szName[50];
+
+		snprintf(szName, 50, "NZBPP_SERVER%i_SUCCESSARTICLES", pServerStat->GetServerID());
+		szName[50-1] = '\0';
+		SetIntEnvVar(szName, pServerStat->GetSuccessArticles());
+
+		snprintf(szName, 50, "NZBPP_SERVER%i_FAILEDARTICLES", pServerStat->GetServerID());
+		szName[50-1] = '\0';
+		SetIntEnvVar(szName, pServerStat->GetFailedArticles());
+	}
 
 	PrepareEnvParameters(m_pPostInfo->GetNZBInfo(), NULL);
 
@@ -922,9 +966,42 @@ ScriptStatus::EStatus PostScriptController::AnalyseExitCode(int iExitCode)
 
 void PostScriptController::AddMessage(Message::EKind eKind, const char* szText)
 {
-	if (!strncmp(szText, "[HISTORY] ", 10))
+	const char* szMsgText = szText + m_iPrefixLen;
+
+	if (!strncmp(szMsgText, "[NZB] ", 6))
 	{
-		m_pPostInfo->GetNZBInfo()->AppendMessage(eKind, 0, szText);
+		debug("Command %s detected", szMsgText + 6);
+		if (!strncmp(szMsgText + 6, "FINALDIR=", 9))
+		{
+			g_pDownloadQueueHolder->LockQueue();
+			m_pPostInfo->GetNZBInfo()->SetFinalDir(szMsgText + 6 + 9);
+			g_pDownloadQueueHolder->UnlockQueue();
+		}
+		else if (!strncmp(szMsgText + 6, "NZBPR_", 6))
+		{
+			char* szParam = strdup(szMsgText + 6 + 6);
+			char* szValue = strchr(szParam, '=');
+			if (szValue)
+			{
+				*szValue = '\0';
+				g_pDownloadQueueHolder->LockQueue();
+				m_pPostInfo->GetNZBInfo()->GetParameters()->SetParameter(szParam, szValue + 1);
+				g_pDownloadQueueHolder->UnlockQueue();
+			}
+			else
+			{
+				error("Invalid command \"%s\" received from %s", szMsgText, GetInfoName());
+			}
+			free(szParam);
+		}
+		else
+		{
+			error("Invalid command \"%s\" received from %s", szMsgText, GetInfoName());
+		}
+	}
+	else if (!strncmp(szMsgText, "[HISTORY] ", 10))
+	{
+		m_pPostInfo->GetNZBInfo()->AppendMessage(eKind, 0, szMsgText);
 	}
 	else
 	{
@@ -967,45 +1044,6 @@ void PostScriptController::Stop()
 	Terminate();
 }
 
-/**
- * DownloadQueue must be locked prior to call of this function.
- */
-void PostScriptController::InitParamsForNewNZB(NZBInfo* pNZBInfo)
-{
-	const char* szDefScript = g_pOptions->GetDefScript();
-
-	if (pNZBInfo->GetCategory() && strlen(pNZBInfo->GetCategory()) > 0)
-	{
-		Options::Category* pCategory = g_pOptions->FindCategory(pNZBInfo->GetCategory());
-		if (pCategory && pCategory->GetDefScript() && strlen(pCategory->GetDefScript()) > 0)
-		{
-			szDefScript = pCategory->GetDefScript();
-		}
-	}
-
-	if (!szDefScript || strlen(szDefScript) == 0)
-	{
-		return;
-	}
-
-	// split szDefScript into tokens and create pp-parameter for each token
-	char* szDefScript2 = strdup(szDefScript);
-	char* saveptr;
-	char* szScriptName = strtok_r(szDefScript2, ",;", &saveptr);
-	while (szScriptName)
-	{
-		szScriptName = Util::Trim(szScriptName);
-		if (szScriptName[0] != '\0')
-		{
-			char szParam[1024];
-			snprintf(szParam, 1024, "%s:", szScriptName);
-			szParam[1024-1] = '\0';
-			pNZBInfo->GetParameters()->SetParameter(szParam, "yes");
-		}
-		szScriptName = strtok_r(NULL, ",;", &saveptr);
-	}
-	free(szDefScript2);
-}
 
 void NZBScriptController::ExecuteScript(const char* szScript, const char* szNZBFilename, const char* szDirectory,
 	char** pNZBName, char** pCategory, int* iPriority, NZBParameterList* pParameters, bool* bAddTop, bool* bAddPaused)
@@ -1026,6 +1064,13 @@ void NZBScriptController::ExecuteScript(const char* szScript, const char* szNZBF
 	szInfoName[1024-1] = '\0';
 	pScriptController->SetInfoName(szInfoName);
 
+	pScriptController->SetEnvVar("NZBNP_FILENAME", szNZBFilename);
+	pScriptController->SetEnvVar("NZBNP_NZBNAME", strlen(*pNZBName) > 0 ? *pNZBName : Util::BaseFileName(szNZBFilename));
+	pScriptController->SetEnvVar("NZBNP_CATEGORY", *pCategory);
+	pScriptController->SetIntEnvVar("NZBNP_PRIORITY", *iPriority);
+	pScriptController->SetIntEnvVar("NZBNP_TOP", *bAddTop ? 1 : 0);
+	pScriptController->SetIntEnvVar("NZBNP_PAUSED", *bAddPaused ? 1 : 0);
+
 	// remove trailing slash
 	char szDir[1024];
 	strncpy(szDir, szDirectory, 1024);
@@ -1035,18 +1080,7 @@ void NZBScriptController::ExecuteScript(const char* szScript, const char* szNZBF
 	{
 		szDir[iLen-1] = '\0';
 	}
-
-	char szPriority[20];
-	snprintf(szPriority, 20, "%i", *iPriority);
-	szPriority[20-1] = '\0';
-
-	char szAddTop[10];
-	snprintf(szAddTop, 10, "%i", *bAddTop ? 1 : 0);
-	szAddTop[10-1] = '\0';
-
-	char szAddPaused[10];
-	snprintf(szAddPaused, 10, "%i", *bAddPaused ? 1 : 0);
-	szAddPaused[10-1] = '\0';
+	pScriptController->SetEnvVar("NZBNP_DIRECTORY", szDir);
 
 	char szLogPrefix[1024];
 	strncpy(szLogPrefix, Util::BaseFileName(szScript), 1024);
@@ -1055,14 +1089,6 @@ void NZBScriptController::ExecuteScript(const char* szScript, const char* szNZBF
 	pScriptController->SetLogPrefix(szLogPrefix);
 	pScriptController->m_iPrefixLen = strlen(szLogPrefix) + 2; // 2 = strlen(": ");
 
-	pScriptController->SetEnvVar("NZBNP_DIRECTORY", szDir);
-	pScriptController->SetEnvVar("NZBNP_FILENAME", szNZBFilename);
-	pScriptController->SetEnvVar("NZBNP_NZBNAME", strlen(*pNZBName) > 0 ? *pNZBName : Util::BaseFileName(szNZBFilename));
-	pScriptController->SetEnvVar("NZBNP_CATEGORY", *pCategory);
-	pScriptController->SetEnvVar("NZBNP_PRIORITY", szPriority);
-	pScriptController->SetEnvVar("NZBNP_TOP", szAddTop);
-	pScriptController->SetEnvVar("NZBNP_PAUSED", szAddPaused);
-
 	pScriptController->Execute();
 
 	delete pScriptController;
@@ -1150,13 +1176,8 @@ void NZBAddedScriptController::StartScript(DownloadQueue* pDownloadQueue, NZBInf
 		}
 	}
 
-	char buf[100];
-
-	snprintf(buf, 100, "%i", iLastID);
-	pScriptController->SetEnvVar("NZBNA_LASTID", buf);
-
-	snprintf(buf, 100, "%i", iMaxPriority);
-	pScriptController->SetEnvVar("NZBNA_PRIORITY", buf);
+	pScriptController->SetIntEnvVar("NZBNA_LASTID", iLastID);
+	pScriptController->SetIntEnvVar("NZBNA_PRIORITY", iMaxPriority);
 
 	pScriptController->PrepareEnvParameters(pNZBInfo, NULL);
 
