@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 677 $
- * $Date: 2013-05-14 22:20:52 +0200 (Tue, 14 May 2013) $
+ * $Revision: 911 $
+ * $Date: 2013-11-24 20:29:52 +0100 (Sun, 24 Nov 2013) $
  *
  */
 
@@ -275,18 +275,9 @@ TLSSocket::TLSSocket(SOCKET iSocket, bool bIsClient, const char* szCertFile, con
 
 TLSSocket::~TLSSocket()
 {
-	if (m_szCertFile)
-	{
-		free(m_szCertFile);
-	}
-	if (m_szKeyFile)
-	{
-		free(m_szKeyFile);
-	}
-	if (m_szCipher)
-	{
-		free(m_szCipher);
-	}
+	free(m_szCertFile);
+	free(m_szKeyFile);
+	free(m_szCipher);
 	Close();
 }
 
