@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 702 $
- * $Date: 2013-06-03 22:56:56 +0200 (Mon, 03 Jun 2013) $
+ * $Revision: 927 $
+ * $Date: 2013-12-25 23:43:58 +0100 (Wed, 25 Dec 2013) $
  *
  */
 
@@ -72,11 +72,6 @@ bool UnpackController::FileList::Exists(const char* szFilename)
 	return false;
 }
 
-UnpackController::~UnpackController()
-{
-	m_archiveFiles.Clear();
-}
-
 void UnpackController::StartJob(PostInfo* pPostInfo)
 {
 	UnpackController* pUnpackController = new UnpackController();
@@ -100,22 +95,18 @@ void UnpackController::Run()
 	m_szName[1024-1] = '\0';
 
 	m_bCleanedUpDisk = false;
-	bool bUnpack = true;
 	m_szPassword[0] = '\0';
 	m_szFinalDir[0] = '\0';
+	m_bFinalDirCreated = false;
 	
-	for (NZBParameterList::iterator it = m_pPostInfo->GetNZBInfo()->GetParameters()->begin(); it != m_pPostInfo->GetNZBInfo()->GetParameters()->end(); it++)
+	NZBParameter* pParameter = m_pPostInfo->GetNZBInfo()->GetParameters()->Find("*Unpack:", false);
+	bool bUnpack = !(pParameter && !strcasecmp(pParameter->GetValue(), "no"));
+
+	pParameter = m_pPostInfo->GetNZBInfo()->GetParameters()->Find("*Unpack:Password", false);
+	if (pParameter)
 	{
-		NZBParameter* pParameter = *it;
-		if (!strcasecmp(pParameter->GetName(), "*Unpack:") && !strcasecmp(pParameter->GetValue(), "no"))
-		{
-			bUnpack = false;
-		}
-		if (!strcasecmp(pParameter->GetName(), "*Unpack:Password"))
-		{
-			strncpy(m_szPassword, pParameter->GetValue(), 1024-1);
-			m_szPassword[1024-1] = '\0';
-		}
+		strncpy(m_szPassword, pParameter->GetValue(), 1024-1);
+		m_szPassword[1024-1] = '\0';
 	}
 	
 	g_pDownloadQueueHolder->UnlockQueue();
@@ -126,17 +117,7 @@ void UnpackController::Run()
 	snprintf(m_szInfoNameUp, 1024, "Unpack for %s", m_szName); // first letter in upper case
 	m_szInfoNameUp[1024-1] = '\0';
 
-	CheckStateFiles();
-
-#ifndef DISABLE_PARCHECK
-	if (bUnpack && m_bHasBrokenFiles && m_pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped && m_bHasParFiles)
-	{
-		PrintMessage(Message::mkInfo, "%s has broken files", m_szName);
-		RequestParCheck(false);
-		m_pPostInfo->SetWorking(false);
-		return;
-	}
-#endif
+	m_bHasParFiles = ParCoordinator::FindMainPars(m_szDestDir, NULL);
 
 	if (bUnpack)
 	{
@@ -157,6 +138,8 @@ void UnpackController::Run()
 
 		m_bUnpackOK = true;
 		m_bUnpackStartError = false;
+		m_bUnpackSpaceError = false;
+		m_bUnpackPasswordError = false;
 
 		if (m_bHasRarFiles || m_bHasNonStdRarFiles)
 		{
@@ -180,9 +163,10 @@ void UnpackController::Run()
 		PrintMessage(Message::mkInfo, (bUnpack ? "Nothing to unpack for %s" : "Unpack for %s skipped"), m_szName);
 
 #ifndef DISABLE_PARCHECK
-		if (bUnpack && m_pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped && m_bHasParFiles)
+		if (bUnpack && m_pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped && 
+			m_pPostInfo->GetNZBInfo()->GetRenameStatus() <= NZBInfo::rsSkipped && m_bHasParFiles)
 		{
-			RequestParCheck(m_pPostInfo->GetNZBInfo()->GetRenameStatus() <= NZBInfo::rsSkipped);
+			RequestParCheck();
 		}
 		else
 #endif
@@ -230,6 +214,8 @@ void UnpackController::ExecuteUnrar()
 
 	m_bUnpackOK = iExitCode == 0 && m_bAllOKMessageReceived && !GetTerminated();
 	m_bUnpackStartError = iExitCode == -1;
+	m_bUnpackSpaceError = iExitCode == 5;
+	m_bUnpackPasswordError = iExitCode == 11; // only for rar5-archives
 
 	if (!m_bUnpackOK && iExitCode > 0)
 	{
@@ -295,51 +281,44 @@ void UnpackController::Completed()
 		PrintMessage(Message::mkInfo, "%s %s", m_szInfoNameUp, "successful");
 		m_pPostInfo->GetNZBInfo()->SetUnpackStatus(NZBInfo::usSuccess);
 		m_pPostInfo->GetNZBInfo()->SetUnpackCleanedUpDisk(m_bCleanedUpDisk);
+		if (g_pOptions->GetParRename())
+		{
+			//request par-rename check for extracted files
+			m_pPostInfo->GetNZBInfo()->SetRenameStatus(NZBInfo::rsNone);
+		}
 		m_pPostInfo->SetStage(PostInfo::ptQueued);
 	}
 	else
 	{
 #ifndef DISABLE_PARCHECK
-		if (!m_bUnpackOK && m_pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped && !m_bUnpackStartError && !GetTerminated() && m_bHasParFiles)
+		if (!m_bUnpackOK && m_pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped &&
+			!m_bUnpackStartError && !m_bUnpackSpaceError && !m_bUnpackPasswordError &&
+			!GetTerminated() && m_bHasParFiles)
 		{
-			RequestParCheck(false);
+			RequestParCheck();
 		}
 		else
 #endif
 		{
 			PrintMessage(Message::mkError, "%s failed", m_szInfoNameUp);
-			m_pPostInfo->GetNZBInfo()->SetUnpackStatus(NZBInfo::usFailure);
+			m_pPostInfo->GetNZBInfo()->SetUnpackStatus(
+				m_bUnpackSpaceError ? NZBInfo::usSpace :
+				m_bUnpackPasswordError ? NZBInfo::usPassword :
+				NZBInfo::usFailure);
 			m_pPostInfo->SetStage(PostInfo::ptQueued);
 		}
 	}
 }
 
 #ifndef DISABLE_PARCHECK
-void UnpackController::RequestParCheck(bool bRename)
+void UnpackController::RequestParCheck()
 {
-	PrintMessage(Message::mkInfo, "%s requested %s", m_szInfoNameUp, bRename ? "par-rename": "par-check/repair");
-	if (bRename)
-	{
-		m_pPostInfo->SetRequestParRename(true);
-	}
-	else
-	{
-		m_pPostInfo->SetRequestParCheck(true);
-	}
+	PrintMessage(Message::mkInfo, "%s requested par-check/repair", m_szInfoNameUp);
+	m_pPostInfo->SetRequestParCheck(true);
 	m_pPostInfo->SetStage(PostInfo::ptFinished);
 }
 #endif
 
-void UnpackController::CheckStateFiles()
-{
-	char szBrokenLog[1024];
-	snprintf(szBrokenLog, 1024, "%s%c%s", m_szDestDir, PATH_SEPARATOR, "_brokenlog.txt");
-	szBrokenLog[1024-1] = '\0';
-	m_bHasBrokenFiles = Util::FileExists(szBrokenLog);
-
-	m_bHasParFiles = ParCoordinator::FindMainPars(m_szDestDir, NULL);
-}
-
 void UnpackController::CreateUnpackDir()
 {
 	m_bInterDir = strlen(g_pOptions->GetInterDir()) > 0 &&
@@ -349,6 +328,7 @@ void UnpackController::CreateUnpackDir()
 		m_pPostInfo->GetNZBInfo()->BuildFinalDirName(m_szFinalDir, 1024);
 		m_szFinalDir[1024-1] = '\0';
 		snprintf(m_szUnpackDir, 1024, "%s%c%s", m_szFinalDir, PATH_SEPARATOR, "_unpack");
+		m_bFinalDirCreated = !Util::DirectoryExists(m_szFinalDir);
 	}
 	else
 	{
@@ -374,7 +354,8 @@ void UnpackController::CheckArchiveFiles(bool bScanNonStdFiles)
 	RegEx regExRar(".*\\.rar$");
 	RegEx regExRarMultiSeq(".*\\.(r|s)[0-9][0-9]$");
 	RegEx regExSevenZip(".*\\.7z$");
-	RegEx regExSevenZipMulti(".*\\.7z\\.[0-9]*$");
+	RegEx regExSevenZipMulti(".*\\.7z\\.[0-9]+$");
+	RegEx regExNumExt(".*\\.[0-9]+$");
 
 	DirBrowser dir(m_szDestDir);
 	while (const char* filename = dir.Next())
@@ -397,28 +378,37 @@ void UnpackController::CheckArchiveFiles(bool bScanNonStdFiles)
 			{
 				m_bHasSevenZipMultiFiles = true;
 			}
-			else if (bScanNonStdFiles && !m_bHasNonStdRarFiles && !regExRarMultiSeq.Match(filename))
+			else if (bScanNonStdFiles && !m_bHasNonStdRarFiles &&
+				!regExRarMultiSeq.Match(filename) && regExNumExt.Match(filename) &&
+				FileHasRarSignature(szFullFilename))
 			{
-				// Check if file has RAR signature
-				char rarSignature[] = {0x52, 0x61, 0x72, 0x21, 0x1A, 0x07, 0x00};
-				char fileSignature[7];
-
-				FILE* infile;
-				infile = fopen(szFullFilename, "rb");
-				if (infile)
-				{
-					int cnt = (int)fread(fileSignature, 1, sizeof(fileSignature), infile);
-					fclose(infile);
-					if (cnt == sizeof(fileSignature) && !strcmp(rarSignature, fileSignature))
-					{
-						m_bHasNonStdRarFiles = true;
-					}
-				}
+				m_bHasNonStdRarFiles = true;
 			}
 		}
 	}
 }
 
+bool UnpackController::FileHasRarSignature(const char* szFilename)
+{
+	char rar4Signature[] = { 0x52, 0x61, 0x72, 0x21, 0x1A, 0x07, 0x00 };
+	char rar5Signature[] = { 0x52, 0x61, 0x72, 0x21, 0x1A, 0x07, 0x01, 0x00 };
+
+	char fileSignature[8];
+
+	int cnt = 0;
+	FILE* infile;
+	infile = fopen(szFilename, "rb");
+	if (infile)
+	{
+		cnt = (int)fread(fileSignature, 1, sizeof(fileSignature), infile);
+		fclose(infile);
+	}
+
+	bool bRar = cnt == sizeof(fileSignature) && 
+		(!strcmp(rar4Signature, fileSignature) || !strcmp(rar5Signature, fileSignature));
+	return bRar;
+}
+
 bool UnpackController::Cleanup()
 {
 	// By success:
@@ -438,7 +428,8 @@ bool UnpackController::Cleanup()
 		DirBrowser dir(m_szUnpackDir);
 		while (const char* filename = dir.Next())
 		{
-			if (strcmp(filename, ".") && strcmp(filename, ".."))
+			if (strcmp(filename, ".") && strcmp(filename, "..") &&
+				strcmp(filename, ".AppleDouble") && strcmp(filename, ".DS_Store"))
 			{
 				char szSrcFile[1024];
 				snprintf(szSrcFile, 1024, "%s%c%s", m_szUnpackDir, PATH_SEPARATOR, filename);
@@ -467,35 +458,19 @@ bool UnpackController::Cleanup()
 		PrintMessage(Message::mkError, "Could not remove temporary directory %s", m_szUnpackDir);
 	}
 
+	if (!m_bUnpackOK && m_bFinalDirCreated)
+	{
+		Util::RemoveDirectory(m_szFinalDir);
+	}		
+
 	if (m_bUnpackOK && bOK && g_pOptions->GetUnpackCleanupDisk())
 	{
 		PrintMessage(Message::mkInfo, "Deleting archive files");
 
-		// Delete rar-files (only files which were used by unrar)
-		for (FileList::iterator it = m_archiveFiles.begin(); it != m_archiveFiles.end(); it++)
-		{
-			char* szFilename = *it;
-
-			if (m_bInterDir || !extractedFiles.Exists(szFilename))
-			{
-				char szFullFilename[1024];
-				snprintf(szFullFilename, 1024, "%s%c%s", m_szDestDir, PATH_SEPARATOR, szFilename);
-				szFullFilename[1024-1] = '\0';
-
-				PrintMessage(Message::mkInfo, "Deleting file %s", szFilename);
-
-				if (remove(szFullFilename) != 0)
-				{
-					PrintMessage(Message::mkError, "Could not delete file %s", szFullFilename);
-				}
-			}
-		}
-
-		// Unfortunately 7-Zip doesn't print the processed archive-files to the output.
-		// Therefore we don't know for sure which files were extracted.
-		// We just delete all 7z-files in the directory.
-
-		RegEx regExSevenZip(".*\\.7z$|.*\\.7z\\.[0-9]*$");
+		RegEx regExRar(".*\\.rar$");
+		RegEx regExRarMultiSeq(".*\\.[r-z][0-9][0-9]$");
+		RegEx regExSevenZip(".*\\.7z$|.*\\.7z\\.[0-9]+$");
+		RegEx regExNumExt(".*\\.[0-9]+$");
 
 		DirBrowser dir(m_szDestDir);
 		while (const char* filename = dir.Next())
@@ -504,8 +479,12 @@ bool UnpackController::Cleanup()
 			snprintf(szFullFilename, 1024, "%s%c%s", m_szDestDir, PATH_SEPARATOR, filename);
 			szFullFilename[1024-1] = '\0';
 
-			if (strcmp(filename, ".") && strcmp(filename, "..") && !Util::DirectoryExists(szFullFilename) 
-				&& regExSevenZip.Match(filename) && (m_bInterDir || !extractedFiles.Exists(filename)))
+			if (strcmp(filename, ".") && strcmp(filename, "..") &&
+				!Util::DirectoryExists(szFullFilename) &&
+				(m_bInterDir || !extractedFiles.Exists(filename)) &&
+				(regExRar.Match(filename) || regExSevenZip.Match(filename) ||
+				(regExRarMultiSeq.Match(filename) && FileHasRarSignature(szFullFilename)) ||
+				(m_bHasNonStdRarFiles && regExNumExt.Match(filename) && FileHasRarSignature(szFullFilename))))
 			{
 				PrintMessage(Message::mkInfo, "Deleting file %s", filename);
 
@@ -622,7 +601,6 @@ void UnpackController::AddMessage(Message::EKind eKind, const char* szText)
 	{
 		const char *szFilename = szText + 23;
 		debug("Filename: %s", szFilename);
-		m_archiveFiles.push_back(strdup(szFilename));
 		SetProgressLabel(szText + 7);
 	}
 
@@ -719,24 +697,15 @@ bool MoveController::MoveFiles()
 	DirBrowser dir(m_szInterDir);
 	while (const char* filename = dir.Next())
 	{
-		if (strcmp(filename, ".") && strcmp(filename, ".."))
+		if (strcmp(filename, ".") && strcmp(filename, "..") &&
+			strcmp(filename, ".AppleDouble") && strcmp(filename, ".DS_Store"))
 		{
 			char szSrcFile[1024];
 			snprintf(szSrcFile, 1024, "%s%c%s", m_szInterDir, PATH_SEPARATOR, filename);
 			szSrcFile[1024-1] = '\0';
 
 			char szDstFile[1024];
-			snprintf(szDstFile, 1024, "%s%c%s", m_szDestDir, PATH_SEPARATOR, filename);
-			szDstFile[1024-1] = '\0';
-
-			// prevent overwriting of existing files
-			int dupcount = 0;
-			while (Util::FileExists(szDstFile))
-			{
-				dupcount++;
-				snprintf(szDstFile, 1024, "%s%c%s_duplicate%d", m_szDestDir, PATH_SEPARATOR, filename, dupcount);
-				szDstFile[1024-1] = '\0';
-			}
+			Util::MakeUniqueFilename(szDstFile, 1024, m_szDestDir, filename);
 
 			PrintMessage(Message::mkInfo, "Moving file %s to %s", Util::BaseFileName(szSrcFile), m_szDestDir);
 			if (!Util::MoveFile(szSrcFile, szDstFile))
@@ -747,7 +716,10 @@ bool MoveController::MoveFiles()
 		}
 	}
 
-	Util::RemoveDirectory(m_szInterDir);
+	if (bOK && !Util::DeleteDirectoryWithContent(m_szInterDir))
+	{
+		PrintMessage(Message::mkError, "Could not remove intermediate directory %s", m_szInterDir);
+	}
 
 	return bOK;
 }
