@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 677 $
- * $Date: 2013-05-14 22:20:52 +0200 (Tue, 14 May 2013) $
+ * $Revision: 911 $
+ * $Date: 2013-11-24 20:29:52 +0100 (Sun, 24 Nov 2013) $
  *
  */
 
@@ -62,6 +62,8 @@ WebDownloader::WebDownloader()
 	m_bConfirmedLength = false;
 	m_eStatus = adUndefined;
 	m_szOriginalFilename = NULL;
+	m_bForce = false;
+	m_bRetry = true;
 	SetLastUpdateTimeNow();
 }
 
@@ -69,22 +71,10 @@ WebDownloader::~WebDownloader()
 {
 	debug("Destroying WebDownloader");
 
-	if (m_szURL)
-	{
-		free(m_szURL);
-	}
-	if (m_szInfoName)
-	{
-		free(m_szInfoName);
-	}
-	if (m_szOutputFilename)
-	{
-		free(m_szOutputFilename);
-	}
-	if (m_szOriginalFilename)
-	{
-		free(m_szOriginalFilename);
-	}
+	free(m_szURL);
+	free(m_szInfoName);
+	free(m_szOutputFilename);
+	free(m_szOriginalFilename);
 }
 
 void WebDownloader::SetOutputFilename(const char* v)
@@ -99,6 +89,7 @@ void WebDownloader::SetInfoName(const char* v)
 
 void WebDownloader::SetURL(const char * szURL)
 {
+	free(m_szURL);
 	m_szURL = strdup(szURL);
 }
 
@@ -116,7 +107,13 @@ void WebDownloader::Run()
 
 	int iRemainedDownloadRetries = g_pOptions->GetRetries() > 0 ? g_pOptions->GetRetries() : 1;
 	int iRemainedConnectRetries = iRemainedDownloadRetries > 10 ? iRemainedDownloadRetries : 10;
+	if (!m_bRetry)
+	{
+		iRemainedDownloadRetries = 1;
+		iRemainedConnectRetries = 1;
+	}
 
+	m_iRedirects = 0;
 	EStatus Status = adFailed;
 
 	while (!IsStopped() && iRemainedDownloadRetries > 0 && iRemainedConnectRetries > 0)
@@ -127,19 +124,19 @@ void WebDownloader::Run()
 
 		if ((((Status == adFailed) && (iRemainedDownloadRetries > 1)) ||
 			((Status == adConnectError) && (iRemainedConnectRetries > 1)))
-			&& !IsStopped() && !(g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2()))
+			&& !IsStopped() && !(!m_bForce && (g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2())))
 		{
 			detail("Waiting %i sec to retry", g_pOptions->GetRetryInterval());
 			int msec = 0;
 			while (!IsStopped() && (msec < g_pOptions->GetRetryInterval() * 1000) && 
-				!(g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2()))
+				!(!m_bForce && (g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2())))
 			{
 				usleep(100 * 1000);
 				msec += 100;
 			}
 		}
 
-		if (IsStopped() || g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2())
+		if (IsStopped() || (!m_bForce && (g_pOptions->GetPauseDownload() || g_pOptions->GetPauseDownload2())))
 		{
 			Status = adRetry;
 			break;
@@ -150,6 +147,17 @@ void WebDownloader::Run()
 			break;
 		}
 
+		if (Status == adRedirect)
+		{
+			m_iRedirects++;
+			if (m_iRedirects > 5)
+			{
+				warn("Too many redirects for %s", m_szInfoName);
+				Status = adFailed;
+				break;
+			}
+		}
+
 		if (Status != adConnectError)
 		{
 			iRemainedDownloadRetries--;
@@ -312,6 +320,8 @@ WebDownloader::EStatus WebDownloader::DownloadHeaders()
 	m_iContentLen = -1;
 	bool bFirstLine = true;
 	m_bGZip = false;
+	m_bRedirecting = false;
+	m_bRedirected = false;
 
 	// Headers
 	while (!IsStopped())
@@ -354,7 +364,14 @@ WebDownloader::EStatus WebDownloader::DownloadHeaders()
 			break;
 		}
 
+		Util::TrimRight(line);
 		ProcessHeader(line);
+
+		if (m_bRedirected)
+		{
+			Status = adRedirect;
+			break;
+		}
 	}
 
 	free(szLineBuf);
@@ -397,7 +414,7 @@ WebDownloader::EStatus WebDownloader::DownloadBody()
 		// Have we encountered a timeout?
 		if (iLen <= 0)
 		{
-			if (m_iContentLen == -1)
+			if (m_iContentLen == -1 && iWrittenLen > 0)
 			{
 				bEnd = true;
 				break;
@@ -430,10 +447,7 @@ WebDownloader::EStatus WebDownloader::DownloadBody()
 	free(szLineBuf);
 
 #ifndef DISABLE_GZIP
-	if (m_pGUnzipStream)
-	{
-		delete m_pGUnzipStream;
-	}
+	delete m_pGUnzipStream;
 #endif
 
 	if (m_pOutFile)
@@ -485,6 +499,11 @@ WebDownloader::EStatus WebDownloader::CheckResponse(const char* szResponse)
 		warn("URL %s failed: %s", m_szInfoName, szHTTPResponse);
 		return adNotFound;
 	}
+	else if (!strncmp(szHTTPResponse, "301", 3) || !strncmp(szHTTPResponse, "302", 3))
+	{
+		m_bRedirecting = true;
+		return adRunning;
+	}
 	else if (!strncmp(szHTTPResponse, "200", 3))
 	{
 		// OK
@@ -500,21 +519,24 @@ WebDownloader::EStatus WebDownloader::CheckResponse(const char* szResponse)
 
 void WebDownloader::ProcessHeader(const char* szLine)
 {
-	if (!strncmp(szLine, "Content-Length: ", 16))
+	if (!strncasecmp(szLine, "Content-Length: ", 16))
 	{
 		m_iContentLen = atoi(szLine + 16);
 		m_bConfirmedLength = true;
 	}
-
-	if (!strncmp(szLine, "Content-Encoding: gzip", 22))
+	else if (!strncasecmp(szLine, "Content-Encoding: gzip", 22))
 	{
 		m_bGZip = true;
 	}
-
-	if (!strncmp(szLine, "Content-Disposition: ", 21))
+	else if (!strncasecmp(szLine, "Content-Disposition: ", 21))
 	{
 		ParseFilename(szLine);
 	}
+	else if (m_bRedirecting && !strncasecmp(szLine, "Location: ", 10))
+	{
+		ParseRedirect(szLine + 10);
+		m_bRedirected = true;
+	}
 }
 
 void WebDownloader::ParseFilename(const char* szContentDisposition)
@@ -551,15 +573,36 @@ void WebDownloader::ParseFilename(const char* szContentDisposition)
 
 	WebUtil::HttpUnquote(fname);
 
-	if (m_szOriginalFilename)
-	{
-		free(m_szOriginalFilename);
-	}
+	free(m_szOriginalFilename);
 	m_szOriginalFilename = strdup(Util::BaseFileName(fname));
 
 	debug("OriginalFilename: %s", m_szOriginalFilename);
 }
 
+void WebDownloader::ParseRedirect(const char* szLocation)
+{
+	const char* szNewURL = szLocation;
+	char szUrlBuf[1024];
+	URL newUrl(szNewURL);
+	if (!newUrl.IsValid())
+	{
+		// relative address
+		URL oldUrl(m_szURL);
+		if (oldUrl.GetPort() > 0)
+		{
+			snprintf(szUrlBuf, 1024, "%s://%s:%i%s", oldUrl.GetProtocol(), oldUrl.GetHost(), oldUrl.GetPort(), szNewURL);
+		}
+		else
+		{
+			snprintf(szUrlBuf, 1024, "%s://%s%s", oldUrl.GetProtocol(), oldUrl.GetHost(), szNewURL);
+		}
+		szUrlBuf[1024-1] = '\0';
+		szNewURL = szUrlBuf;
+	}
+	detail("URL %s redirected to %s", m_szURL, szNewURL);
+	SetURL(szNewURL);
+}
+
 bool WebDownloader::Write(void* pBuffer, int iLen)
 {
 	if (!m_pOutFile && !PrepareFile())
