@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 709 $
- * $Date: 2013-06-15 17:07:11 +0200 (Sat, 15 Jun 2013) $
+ * $Revision: 911 $
+ * $Date: 2013-11-24 20:29:52 +0100 (Sun, 24 Nov 2013) $
  *
  */
 
@@ -66,18 +66,9 @@ WebProcessor::WebProcessor()
 
 WebProcessor::~WebProcessor()
 {
-	if (m_szRequest)
-	{
-		free(m_szRequest);
-	}
-	if (m_szUrl)
-	{
-		free(m_szUrl);
-	}
-	if (m_szOrigin)
-	{
-		free(m_szOrigin);
-	}
+	free(m_szRequest);
+	free(m_szUrl);
+	free(m_szOrigin);
 }
 
 void WebProcessor::SetUrl(const char* szUrl)
@@ -185,7 +176,8 @@ void WebProcessor::Execute()
 
 	debug("Final URL=%s", m_szUrl);
 
-	if (strlen(g_pOptions->GetControlPassword()) > 0)
+	if (strlen(g_pOptions->GetControlPassword()) > 0 &&
+		!(strlen(g_pOptions->GetAuthorizedIP()) > 0 && IsAuthorizedIP(m_pConnection->GetRemoteAddr())))
 	{
 		if (strlen(szAuthInfo) == 0)
 		{
@@ -225,6 +217,30 @@ void WebProcessor::Execute()
 	Dispatch();
 }
 
+bool WebProcessor::IsAuthorizedIP(const char* szRemoteAddr)
+{
+	const char* szRemoteIP = m_pConnection->GetRemoteAddr();
+
+	// split option AuthorizedIP into tokens and check each token
+	bool bAuthorized = false;
+	char* szAuthorizedIP = strdup(g_pOptions->GetAuthorizedIP());
+	char* saveptr;
+	char* szIP = strtok_r(szAuthorizedIP, ",;", &saveptr);
+	while (szIP)
+	{
+		szIP = Util::Trim(szIP);
+		if (szIP[0] != '\0' && !strcmp(szIP, szRemoteIP))
+		{
+			bAuthorized = true;
+			break;
+		}
+		szIP = strtok_r(NULL, ",;", &saveptr);
+	}
+	free(szAuthorizedIP);
+	
+	return bAuthorized;
+}
+
 void WebProcessor::Dispatch()
 {
 	if (*m_szUrl != '/')
@@ -425,10 +441,7 @@ void WebProcessor::SendBodyResponse(const char* szBody, int iBodyLen, const char
 	m_pConnection->Send(szBody, iBodyLen);
 	
 #ifndef DISABLE_GZIP
-	if (szGBuf)
-	{
-		free(szGBuf);
-	}
+	free(szGBuf);
 #endif
 }
 
