@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 711 $
- * $Date: 2013-06-17 22:39:46 +0200 (Mon, 17 Jun 2013) $
+ * $Revision: 911 $
+ * $Date: 2013-11-24 20:29:52 +0100 (Sun, 24 Nov 2013) $
  *
  */
 
@@ -75,6 +75,8 @@
 #include "ParChecker.h"
 #include "Scheduler.h"
 #include "Scanner.h"
+#include "FeedCoordinator.h"
+#include "Maintenance.h"
 #include "Util.h"
 #ifdef WIN32
 #include "NTService.h"
@@ -115,6 +117,8 @@ PrePostProcessor* g_pPrePostProcessor = NULL;
 DiskState* g_pDiskState = NULL;
 Scheduler* g_pScheduler = NULL;
 Scanner* g_pScanner = NULL;
+FeedCoordinator* g_pFeedCoordinator = NULL;
+Maintenance* g_pMaintenance = NULL;
 int g_iArgumentCount;
 char* (*g_szEnvironmentVariables)[] = NULL;
 char* (*g_szArguments)[] = NULL;
@@ -147,6 +151,8 @@ int main(int argc, char *argv[], char *argp[])
 	DisableCout();
 #endif
 
+	srand (time(NULL));
+
 	g_iArgumentCount = argc;
 	g_szArguments = (char*(*)[])argv;
 	g_szEnvironmentVariables = (char*(*)[])argp;
@@ -205,6 +211,12 @@ void Run(bool bReload)
 
 	g_pServerPool = new ServerPool();
 	g_pScheduler = new Scheduler();
+	g_pQueueCoordinator = new QueueCoordinator();
+	g_pDownloadSpeedMeter = g_pQueueCoordinator;
+	g_pDownloadQueueHolder = g_pQueueCoordinator;
+	g_pUrlCoordinator = new UrlCoordinator();
+	g_pFeedCoordinator = new FeedCoordinator();
+	g_pMaintenance = new Maintenance();
 
 	debug("Reading options");
 	g_pOptions = new Options(g_iArgumentCount, *g_szArguments);
@@ -254,9 +266,6 @@ void Run(bool bReload)
 	if (!g_pOptions->GetRemoteClientMode())
 	{
 		g_pServerPool->InitConnections();
-#ifdef DEBUG
-		g_pServerPool->LogDebugInfo();
-#endif
 	}
 
 #ifndef WIN32
@@ -286,16 +295,6 @@ void Run(bool bReload)
 		return;
 	}
 
-	// Create the queue coordinator
-	if (!g_pOptions->GetRemoteClientMode())
-	{                                    
-		g_pQueueCoordinator = new QueueCoordinator();
-		g_pDownloadSpeedMeter = g_pQueueCoordinator;
-		g_pDownloadQueueHolder = g_pQueueCoordinator;
-
-		g_pUrlCoordinator = new UrlCoordinator();
-	}
-
 	// Setup the network-server
 	if (g_pOptions->GetServerMode())
 	{
@@ -365,11 +364,13 @@ void Run(bool bReload)
 		g_pQueueCoordinator->Start();
 		g_pUrlCoordinator->Start();
 		g_pPrePostProcessor->Start();
+		g_pFeedCoordinator->Start();
 
 		// enter main program-loop
 		while (g_pQueueCoordinator->IsRunning() || 
 			g_pUrlCoordinator->IsRunning() || 
-			g_pPrePostProcessor->IsRunning())
+			g_pPrePostProcessor->IsRunning() ||
+			g_pFeedCoordinator->IsRunning())
 		{
 			if (!g_pOptions->GetServerMode() && 
 				!g_pQueueCoordinator->HasMoreJobs() && 
@@ -389,6 +390,10 @@ void Run(bool bReload)
 				{
 					g_pPrePostProcessor->Stop();
 				}
+				if (!g_pFeedCoordinator->IsStopped())
+				{
+					g_pFeedCoordinator->Stop();
+				}
 			}
 			usleep(100 * 1000);
 		}
@@ -397,6 +402,7 @@ void Run(bool bReload)
 		debug("QueueCoordinator stopped");
 		debug("UrlCoordinator stopped");
 		debug("PrePostProcessor stopped");
+		debug("FeedCoordinator stopped");
 	}
 
 	// Stop network-server
@@ -595,6 +601,7 @@ void ExitProc()
 			g_pQueueCoordinator->Stop();
 			g_pUrlCoordinator->Stop();
 			g_pPrePostProcessor->Stop();
+			g_pFeedCoordinator->Stop();
 		}
 	}
 }
@@ -719,64 +726,40 @@ void Cleanup()
 	debug("Cleaning up global objects");
 
 	debug("Deleting UrlCoordinator");
-	if (g_pUrlCoordinator)
-	{
-		delete g_pUrlCoordinator;
-		g_pUrlCoordinator = NULL;
-	}
+	delete g_pUrlCoordinator;
+	g_pUrlCoordinator = NULL;
 	debug("UrlCoordinator deleted");
 
 	debug("Deleting RemoteServer");
-	if (g_pRemoteServer)
-	{
-		delete g_pRemoteServer;
-		g_pRemoteServer = NULL;
-	}
+	delete g_pRemoteServer;
+	g_pRemoteServer = NULL;
 	debug("RemoteServer deleted");
 
 	debug("Deleting RemoteSecureServer");
-	if (g_pRemoteSecureServer)
-	{
-		delete g_pRemoteSecureServer;
-		g_pRemoteSecureServer = NULL;
-	}
+	delete g_pRemoteSecureServer;
+	g_pRemoteSecureServer = NULL;
 	debug("RemoteSecureServer deleted");
 
 	debug("Deleting PrePostProcessor");
-	if (g_pPrePostProcessor)
-	{
-		delete g_pPrePostProcessor;
-		g_pPrePostProcessor = NULL;
-	}
-	if (g_pScanner)
-	{
-		delete g_pScanner;
-		g_pScanner = NULL;
-	}
+	delete g_pPrePostProcessor;
+	g_pPrePostProcessor = NULL;
+	delete g_pScanner;
+	g_pScanner = NULL;
 	debug("PrePostProcessor deleted");
 
 	debug("Deleting Frontend");
-	if (g_pFrontend)
-	{
-		delete g_pFrontend;
-		g_pFrontend = NULL;
-	}
+	delete g_pFrontend;
+	g_pFrontend = NULL;
 	debug("Frontend deleted");
 
 	debug("Deleting QueueCoordinator");
-	if (g_pQueueCoordinator)
-	{
-		delete g_pQueueCoordinator;
-		g_pQueueCoordinator = NULL;
-	}
+	delete g_pQueueCoordinator;
+	g_pQueueCoordinator = NULL;
 	debug("QueueCoordinator deleted");
 
 	debug("Deleting DiskState");
-	if (g_pDiskState)
-	{
-		delete g_pDiskState;
-		g_pDiskState = NULL;
-	}
+	delete g_pDiskState;
+	g_pDiskState = NULL;
 	debug("DiskState deleted");
 
 	debug("Deleting Options");
@@ -793,21 +776,25 @@ void Cleanup()
 	debug("Options deleted");
 
 	debug("Deleting ServerPool");
-	if (g_pServerPool)
-	{
-		delete g_pServerPool;
-		g_pServerPool = NULL;
-	}
+	delete g_pServerPool;
+	g_pServerPool = NULL;
 	debug("ServerPool deleted");
 
 	debug("Deleting Scheduler");
-	if (g_pScheduler)
-	{
-		delete g_pScheduler;
-		g_pScheduler = NULL;
-	}
+	delete g_pScheduler;
+	g_pScheduler = NULL;
 	debug("Scheduler deleted");
 
+	debug("Deleting FeedCoordinator");
+	delete g_pFeedCoordinator;
+	g_pFeedCoordinator = NULL;
+	debug("FeedCoordinator deleted");
+
+	debug("Deleting Maintenance");
+	delete g_pMaintenance;
+	g_pMaintenance = NULL;
+	debug("Maintenance deleted");
+
 	if (!g_bReloading)
 	{
 		Connection::Final();
@@ -816,11 +803,8 @@ void Cleanup()
 
 	debug("Global objects cleaned up");
 
-	if (g_pLog)
-	{
-		delete g_pLog;
-		g_pLog = NULL;
-	}
+	delete g_pLog;
+	g_pLog = NULL;
 }
 
 #ifndef WIN32
