@@ -17,8 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 710 $
- * $Date: 2013-06-16 15:00:57 +0200 (Sun, 16 Jun 2013) $
+ * $Revision: 836 $
+ * $Date: 2013-09-23 22:18:54 +0200 (Mon, 23 Sep 2013) $
  *
  */
 
@@ -172,9 +172,12 @@ var Upload = (new function($)
 	{
 		var inp = $('#AddDialog_Input');
 
-		// Reset file input control (needed for IE10)
-		inp.wrap('<form>').closest('form').get(0).reset();
-		inp.unwrap();
+		// Reset file input control; needed for IE10 but produce problems with opera (the old non-webkit one).
+		if ($.browser.msie)
+		{
+			inp.wrap('<form>').closest('form').get(0).reset();
+			inp.unwrap();
+		}
 		
 		inp.click();
 	}
@@ -219,6 +222,8 @@ var Upload = (new function($)
 		$('#AddDialog_FilesHelp').show();
 		$('#AddDialog_URLLabel img').hide();
 		$('#AddDialog_URLLabel i').hide();
+		$('#AddDialog_Paused').prop('checked', false);
+		$('#AddDialog_DupeForce').prop('checked', false);
 		enableAllButtons();
 
 		var v = $('#AddDialog_Priority');
@@ -328,7 +333,9 @@ var Upload = (new function($)
 			var category = $('#AddDialog_Category').val();
 			var priority = parseInt($('#AddDialog_Priority').val());
 			var filename = file.name.replace(/\.queued$/g, '');
-			RPC.call('append', [filename, category, priority, false, base64str], fileCompleted, fileFailure);
+			var addPaused = $('#AddDialog_Paused').is(':checked');
+			var dupeMode = $('#AddDialog_DupeForce').is(':checked') ? "FORCE" : "SCORE";
+			RPC.call('append', [filename, category, priority, false, base64str, addPaused, '', 0, dupeMode], fileCompleted, fileFailure);
 		};
 
 		if (reader.readAsBinaryString)
@@ -369,8 +376,10 @@ var Upload = (new function($)
 
 		var category = $('#AddDialog_Category').val();
 		var priority = parseInt($('#AddDialog_Priority').val());
+		var addPaused = $('#AddDialog_Paused').is(':checked');
+		var dupeMode = $('#AddDialog_DupeForce').is(':checked') ? "FORCE" : "SCORE";
 
-		RPC.call('appendurl', ['', category, priority, false, url], urlCompleted, urlFailure);
+		RPC.call('appendurl', ['', category, priority, false, url, addPaused, '', 0, dupeMode], urlCompleted, urlFailure);
 	}
 
 	function urlCompleted(result)
