@@ -17,8 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 689 $
- * $Date: 2013-05-21 22:41:08 +0200 (Tue, 21 May 2013) $
+ * $Revision: 881 $
+ * $Date: 2013-10-17 21:35:43 +0200 (Thu, 17 Oct 2013) $
  *
  */
 
@@ -221,7 +221,7 @@ var Util = (new function($)
 		var $elem = $(dialog);
 		if (center)
 		{
-			var top = ($(window).height() - $elem.outerHeight())/2;
+			var top = ($(window).height() - $elem.outerHeight()) * 0.4;
 			top = top > 0 ? top : 0;
 			$elem.css({ top: top});
 		}
@@ -230,6 +230,7 @@ var Util = (new function($)
 			$elem.css({ top: '' });
 		}
 	}
+	
 }(jQuery));
 
 
@@ -243,8 +244,25 @@ var TabDialog = (new function($)
 	{
 		dialog.restoreTab = restoreTab;
 		dialog.switchTab = switchTab;
+		dialog.maximize = maximize;
 	}
 	
+	function maximize(options)
+	{
+		var bodyPadding = 15;
+		var dialog = this;
+		var body = $('.modal-body', dialog);
+		var footer = $('.modal-footer', dialog);
+		var header = $('.modal-header', dialog);
+		body.css({top: header.outerHeight(), bottom: footer.outerHeight()});
+		if (options.mini)
+		{
+			var scrollheader = $('.modal-scrollheader', dialog);
+			var scroll = $('.modal-inner-scroll', dialog);
+			scroll.css('min-height', dialog.height() - header.outerHeight() - footer.outerHeight() - scrollheader.height() - bodyPadding*2);
+		}
+	}
+
 	function restoreTab()
 	{
 		var dialog = this;
@@ -400,7 +418,7 @@ var RPC = (new function($)
 	this.defaultFailureCallback;
 	this.connectErrorMessage = 'Cannot establish connection';
 
-	this.call = function(method, params, completed_callback, failure_callback)
+	this.call = function(method, params, completed_callback, failure_callback, timeout)
 	{
 		var _this = this;
 		
@@ -408,6 +426,11 @@ var RPC = (new function($)
 		var xhr = new XMLHttpRequest();
 
 		xhr.open('post', this.rpcUrl);
+		
+		if (timeout)
+		{
+			xhr.timeout = timeout;
+		}
 
 		// Example for cross-domain access:
 		//xhr.open('post', 'http://localhost:6789/jsonrpc');
